

--Creaci de taules

CREATE TABLE Persona
(
  NIF             CHAR(9)      NOT NULL,
  telefon         NUMBER        NOT NULL,
  email           VARCHAR2(30)  NOT NULL,

  CONSTRAINT PK_Persona PRIMARY KEY (NIF),
  CONSTRAINT UK_Telefon UNIQUE (telefon)
);

CREATE TABLE Usuari
(
  NIF             CHAR(9)       NOT NULL,
  pais            CHAR(2)       NOT NULL,
  operador        VARCHAR2(30)  NOT NULL,

  CONSTRAINT PK_Usuari PRIMARY KEY (NIF),
  CONSTRAINT FK_Usuari FOREIGN KEY (NIF) REFERENCES Persona(NIF)
);
CREATE TABLE Desenvolupador
(
  NIF              CHAR(9)      NOT NULL,
  dataNaixement    DATE  ,       
  llocNaixement    VARCHAR2(30) ,
  pais             VARCHAR2(30)  NOT NULL,

  CONSTRAINT PK_Desenvolupador PRIMARY KEY (NIF),
  CONSTRAINT FK_Desenvolupador FOREIGN KEY (NIF) REFERENCES Persona(NIF)
);

CREATE TABLE SO
(
  sistemaOperatiu  VARCHAR2(30) NOT NULL,
 
  CONSTRAINT PK_SO PRIMARY KEY (sistemaOperatiu)
);

CREATE TABLE Dispositiu
(
   IMEI            NUMBER       NOT NULL,
   resolucio       NUMBER       NOT NULL,
   modelo          VARCHAR2(30) NOT NULL,
   usuaris         CHAR(9)      NOT NULL,
   sistemaOperatiu VARCHAR2(30) NOT NULL,

  CONSTRAINT PK_Dispositiu PRIMARY KEY (IMEI),
  CONSTRAINT FK_SODispositiu FOREIGN KEY (sistemaOperatiu) REFERENCES SO(sistemaOperatiu),
  CONSTRAINT FK_UsuariDispositiu FOREIGN KEY (usuaris) REFERENCES Usuari(NIF)
);

CREATE TABLE Pais
(
   codiPais        CHAR(2)       NOT NULL, 

   CONSTRAINT PK_Pais PRIMARY KEY (codiPais)
);


CREATE TABLE Aplicacio
(
   identificador   NUMBER        NOT NULL,
   versio          VARCHAR2(30)  NOT NULL,
   dataPujada      DATE          NOT NULL,
   URL             VARCHAR2(100) , 
   activa          CHAR(2)       NOT NULL,
   resolucio       NUMBER        NOT NULL,

   CONSTRAINT PK_Aplicacio PRIMARY KEY (identificador),
   CONSTRAINT CK_Aplicacio CHECK (activa IN('SI', 'NO'))
);

CREATE TABLE Empresa
(
   codi            VARCHAR2(30)  NOT NULL,
   nom             VARCHAR2(30)  NOT NULL,
   representant    VARCHAR2(30)  NOT NULL,
   pais            CHAR(2)       NOT NULL,
   adrea          VARCHAR2(30)  NOT NULL,
   telefon         NUMBER        NOT NULL,

   CONSTRAINT PK_Empresa PRIMARY KEY (codi),
   CONSTRAINT UK_EmpresaTlf UNIQUE (telefon)
);

CREATE TABLE Descarrega
(
   usuari          CHAR(9)       NOT NULL,
   aplicacio       NUMBER        NOT NULL,
   dataDesc        DATE          NOT NULL,
   pagament        VARCHAR2(30)  NOT NULL,
   operador        VARCHAR2(30)  NOT NULL,
   pais            CHAR(2)       NOT NULL,
   dispositiu      NUMBER        NOT NULL,

   CONSTRAINT PK_Descarrega PRIMARY KEY (usuari, aplicacio),
   CONSTRAINT FK_UsuariDescarrega FOREIGN KEY (usuari) REFERENCES Usuari(NIF),
   CONSTRAINT FK_AplicacioDescarrega FOREIGN KEY (aplicacio) REFERENCES Aplicacio(identificador),
   CONSTRAINT FK_DispositiuDescarrega FOREIGN KEY (dispositiu) REFERENCES  Dispositiu(IMEI)
);

CREATE TABLE Creacio
(
   desenvolupador  CHAR(9)  NOT NULL,
   aplicacio       NUMBER   NOT NULL,

   CONSTRAINT PK_Creacio PRIMARY KEY (desenvolupador,aplicacio),
   CONSTRAINT FK_DesenvolupadorCreacio FOREIGN KEY (desenvolupador) REFERENCES Desenvolupador(NIF),
   CONSTRAINT FK_AplicacioCreacio FOREIGN KEY (aplicacio) REFERENCES Aplicacio(identificador)
);

CREATE TABLE Treballa
(
   desenvolupador  CHAR(9)           NOT NULL,
   empresa         VARCHAR2(30)      NOT NULL,

   CONSTRAINT PK_Treballa PRIMARY KEY (desenvolupador, empresa),
   CONSTRAINT FK_DesenvolupadorTreballa FOREIGN KEY (desenvolupador) REFERENCES Desenvolupador(NIF),
   CONSTRAINT FK_EmpresaTreballa FOREIGN KEY (empresa) REFERENCES Empresa(codi)
);


CREATE TABLE Viable
(
   aplicacio      NUMBER        NOT NULL,
   so             VARCHAR2(30)  NOT NULL,
   mida           NUMBER        NOT NULL,
   enlla         VARCHAR2(100) ,
 
   CONSTRAINT PK_Viable PRIMARY KEY (aplicacio, so),
   CONSTRAINT FK_AplicacioViable FOREIGN KEY (aplicacio) REFERENCES Aplicacio(identificador),
   CONSTRAINT FK_SOViable FOREIGN KEY (so) REFERENCES SO(sistemaOperatiu),
   CONSTRAINT CK_Viable CHECK (mida>0)
);

CREATE TABLE Registre
(
   aplicacio      NUMBER         NOT NULL,
   pais           CHAR(2)        NOT NULL,
   preu           NUMBER         NOT NULL,
   descripcio     VARCHAR2(100),
  
   CONSTRAINT PK_Registre PRIMARY KEY (aplicacio, pais),
   CONSTRAINT FK_AplicacioRegistre FOREIGN KEY (aplicacio) REFERENCES Aplicacio(identificador),
   CONSTRAINT FK_PaisRegistre FOREIGN KEY (pais) REFERENCES Pais(codiPais),
   CONSTRAINT CK_Registre CHECK (preu>0)
);

CREATE TABLE EST1
(
   totalDescarregues  NUMBER            NOT NULL,

   CONSTRAINT PK_EST1 PRIMARY KEY (totalDescarregues)

);

CREATE TABLE EST2
(
  
   totalEuros         NUMBER NOT NULL,

   CONSTRAINT PK_EST2 PRIMARY KEY (totalEuros)

);

CREATE TABLE EST3
(
   ao       NUMBER             NOT NULL,
   mitjana   NUMBER   DEFAULT 0 NOT NULL,

   CONSTRAINT PK_EST3 PRIMARY KEY (ao)

);

CREATE TABLE EST4
(
   ident          NUMBER           NOT NULL,
   ao            NUMBER           NOT NULL,
   desenvolupador CHAR(9)          NOT NULL,
   nombre         NUMBER DEFAULT 0 NOT NULL,

   CONSTRAINT PK_EST4 PRIMARY KEY (ident)

);

CREATE TABLE EST5
( 
   ident          NUMBER           NOT NULL,
   ao            NUMBER           NOT NULL,
   desenvolupador CHAR(9)          NOT NULL,
   aplicacio      NUMBER           NOT NULL,
   recaptacio     NUMBER  NOT NULL,

   CONSTRAINT PK_EST5 PRIMARY KEY (ident)


);

CREATE TABLE EST6
(
   ident          NUMBER           NOT NULL,
   ao            NUMBER           NOT NULL,
   pais           CHAR(2)          NOT NULL,
   nombreUsuaris  NUMBER DEFAULT 0 NOT NULL, 

   CONSTRAINT PK_EST6 PRIMARY KEY (ident)

);

CREATE TABLE EST7
(
   ident          NUMBER           NOT NULL,
   ao            NUMBER           NOT NULL,
   pais           CHAR(2)          NOT NULL,
   ingressos     NUMBER DEFAULT 0  NOT NULL, 

   CONSTRAINT PK_EST7 PRIMARY KEY (ident)


);

CREATE TABLE EST8
(
   ident               NUMBER           NOT NULL,
   ao                 NUMBER           NOT NULL,
   pais                CHAR(2)          NOT NULL,
   nombreDescarregues  NUMBER DEFAULT 0 NOT NULL, 

   CONSTRAINT PK_EST8 PRIMARY KEY (ident)


);

CREATE TABLE Logs
(
   ident         NUMBER       NOT NULL,
   dataHora      VARCHAR2(30) NOT NULL,
   procediment   VARCHAR2(30) NOT NULL,
   param_entrada VARCHAR2(220) NOT NULL,
   param_sortida VARCHAR2(300) NOT NULL,

   CONSTRAINT PK_LOG PRIMARY KEY (ident)
);


/

-- Parell seqncia - disparador per inserci de codi automtic en el mdul estadstic i LOGS

CREATE SEQUENCE SQ_LOGS INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_LOGS 
  BEFORE INSERT ON LOGS
  FOR EACH ROW
BEGIN
  SELECT SQ_LOGS.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_LOGS; 
/

CREATE SEQUENCE SQ_EST4 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_EST4
  BEFORE INSERT ON EST4
  FOR EACH ROW
BEGIN
  SELECT SQ_EST4.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_EST4; 
/


CREATE SEQUENCE SQ_EST5 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_EST5
  BEFORE INSERT ON EST5
  FOR EACH ROW
BEGIN
  SELECT SQ_EST5.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_EST5; 
/

CREATE SEQUENCE SQ_EST6 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_EST6
  BEFORE INSERT ON EST6
  FOR EACH ROW
BEGIN
  SELECT SQ_EST6.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_EST6; 
/

CREATE SEQUENCE SQ_EST7 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_EST7
  BEFORE INSERT ON EST7
  FOR EACH ROW
BEGIN
  SELECT SQ_EST7.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_EST7; 
/
CREATE SEQUENCE SQ_EST8 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_EST8
  BEFORE INSERT ON EST8
  FOR EACH ROW
BEGIN
  SELECT SQ_EST8.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_EST8; 
/










