create or replace PROCEDURE SP_ALTA_LOGS (
   p_datahora        IN LOGS.datahora%TYPE,
   p_procediment     IN LOGS.procediment%TYPE,
   p_param_entrada   IN LOGS.param_entrada%TYPE,
   p_param_sortida   IN LOGS.param_sortida%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_LOGS
   Propsit:            afegeix una entrada a la taula LOGS
                   
   Parmetres entrada:  p_data_hora, p_nom_procediment, p_param_entrada, p_param_sortida            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP  VARCHAR2 (500);
error_msg  VARCHAR2 (200);
valor_nul VARCHAR2(20);
E_NUL     EXCEPTION;

PRAGMA AUTONOMOUS_TRANSACTION;

BEGIN

   param_entrada:= 'p_datahora: '||p_datahora||
                 ', pprocediment: '||p_procediment||
                 ', p_param_entrada: '||p_param_entrada||
                 ', p_param_sortida: '||p_param_sortida;

  --verifica que els camps obligatoris no siguin nuls
   IF (p_datahora IS NULL) THEN 
     valor_nul:= 'dataHora';
     RAISE E_NUL;
   ELSIF (p_procediment IS NULL) THEN 
     valor_nul:= 'procediment';     
     RAISE E_NUL;
   ELSIF (p_param_entrada IS NULL) THEN 
     valor_nul:= 'param_entrada';
     RAISE E_NUL;     
   ELSIF (p_param_sortida IS NULL) THEN 
     valor_nul:= 'param_sortida';
     RAISE E_NUL;     
   END IF;
 

  -- inserta la nova entrada a la taula log
   INSERT INTO LOGS (datahora, procediment, param_entrada, param_sortida)
   VALUES (p_datahora, p_procediment, p_param_entrada, p_param_sortida);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR LOGS: el camp ' || valor_nul || ' no pot ser nul'; 
        INSERT INTO LOGS (datahora, procediment, param_entrada, param_sortida)
        VALUES (to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LOGS', p_param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);

        
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR LOGS: ' || error_msg;
        INSERT INTO LOGS (datahora, procediment, param_entrada, param_sortida)
        VALUES (to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LOGS', p_param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);

              
END SP_ALTA_LOGS;
