/*********************************************************   EST3    ***********************************************************************/
/******************************************************************************
   Nom:                 SP_ALTA_EST3
   Propsit:            Donat un any passat com a parmetre d'entrada calcula la mitjana d'aplicacions descarregades, s a dir,
                        el total d'aplicacions descarregades en l'any donat dividit pel total d'usuaris diferents que han fet descrregues
                        aquell any.
                   
   Parmetres entrada:  p_any           

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
CREATE OR REPLACE PROCEDURE SP_ALTA_EST3 (
    p_any              NUMBER
    )
    
IS

--variables


param_entrada     VARCHAR2(50);
error_msg         VARCHAR2(100);
RSP               VARCHAR2(200);
mitja             NUMBER(9,2);
comptador         INTEGER;
nombreUsuaris     INTEGER;
nombreAplicacions INTEGER;
E_NUL             EXCEPTION;


BEGIN

  param_entrada:= 'Any: '||p_any;

--controlem que els parmetres d'entrada no siguin NULL

  IF(p_any IS NULL) THEN
     RAISE E_NUL;
  END IF;
  
--calculem el total d'aplicacions descarregades i el total d'usuaris diferents que  han descarregat aplicacions; Calculem la mitja

  SELECT COUNT(aplicacio) INTO nombreAplicacions FROM DESCARREGA de WHERE  EXTRACT(YEAR FROM dataDesc)=p_any;
  SELECT COUNT(DISTINCT usuari) INTO nombreUsuaris FROM DESCARREGA WHERE EXTRACT(YEAR FROM dataDesc)=p_any;
  
  mitja:=nombreAplicacions/nombreUsuaris;

--inserim els valors si la taula EST3 est buida o la modifiquem amb la nova mitja sin est buida
  
  SELECT COUNT(*) INTO comptador FROM EST3 WHERE ao=p_any;
  
  IF(comptador=0) THEN
  
      INSERT INTO EST3(ao, mitjana) VALUES (p_any,mitja);
      
  ELSE
    
      UPDATE EST3 SET mitjana=mitja WHERE ao=p_any;
      
  END IF;

--inserci o modificaci correcta
   
    RSP:='OK';
    SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST3', param_entrada, 'RSP: '||RSP);
  

    
COMMIT;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST3', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
 

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST3', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST3;