
/****************************************************************   EST4   ************************************************************************/
/**************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST4
   Propsit:            Donat un any passat com a parmetre d'entrada, retorna el desenvolupador o desenvolupadors que han 
                        creat de l'aplicaci o aplicacions ms descarregades en la plataforma l'any donat, junt amb el nombre de 
                        descrregues que han tingut.
   Parmetres entrada:  p_any            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
****************************************************************************************************************************************************/
create or replace PROCEDURE SP_ALTA_EST4(
     p_any          NUMBER
     )
IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
desenv          VARCHAR2(10);
totalDesc       INTEGER;
comptador       INTEGER;
maxim           INTEGER;

E_NUL           EXCEPTION;

type refcursor  IS REF CURSOR;
CUR  refcursor;


BEGIN

   param_entrada:='Any: '||p_any;

-- controlem que el parmetre d'entrada no sigui null

   IF (p_any IS NULL) THEN
      RAISE E_NUL;
   END IF;

--obrim un cursor amb tots els desenvolupadors i el nombre d'aplicacions descarregades en l'any donat
   
   OPEN CUR FOR SELECT cr.desenvolupador, COUNT(de.aplicacio) AS total from DESCARREGA de,APLICACIO ap,CREACIO cr
   WHERE de.aplicacio=ap.identificador  AND ap.identificador=cr.aplicacio AND EXTRACT(YEAR FROM DE.dataDesc)=p_any
   GROUP BY cr.desenvolupador ORDER BY total ASC;

-- anem inserint en la taula EST4 el valor de les variables per cada iteraci de manera que no hi hagi files repetides en les iteracions
   
   LOOP
     FETCH CUR INTO desenv,totalDesc;
     EXIT WHEN CUR%NOTFOUND;

     SELECT COUNT(*) INTO comptador FROM EST4 WHERE ao=p_any AND desenvolupador=desenv AND nombre=totalDesc;

     
     IF(comptador=0) THEN
     INSERT INTO EST4(ao,desenvolupador,nombre) VALUES(p_any,desenv,totalDesc);
     END IF;

  END LOOP;
  CLOSE CUR;

--seleccionem el nombre mxim de descrregues i esborrem totes aquelles files que no tinguin el mxim nombre de descrregues; 
-- aix en la taula quedaran nomes l'any, i els desenvolupadors que han tingut mxim nombre de descrregues junt amb aquest nombre.
   
   
  
       SELECT MAX(nombre) INTO maxim FROM EST4 WHERE ao=p_any;

       DELETE FROM EST4 WHERE ao=p_any AND nombre<>maxim;
       
       
  

--inserci o modificaci correcta
  
   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST4', param_entrada, 'RSP: '||RSP);
 


COMMIT;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST4', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST4', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

END SP_ALTA_EST4;