/*******************************************************************    EST5   ******************************************************************/
/******************************************************************************
   Nom:                 SP_ALTA_EST5
   Propsit:            Donat un any passat com a parmetre d'entrada, retorna el desenvolupador o desenvolupadors que han creat l'aplicaci o aplicacions (en cas
                        de que dues aplicacions hagin recaudat el mateix) que han recaudat ms diners en la plataforma. 
                   
   Parmetres entrada:  p_any            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/

create or replace PROCEDURE SP_ALTA_EST5(
     p_any          NUMBER
     )
IS


--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);

aplic           NUMBER;
price           NUMBER;
desenv          VARCHAR2(10);
maxRecap        NUMBER;
total           INTEGER:=0;

E_NUL           EXCEPTION;

-- emmagatzemem en el cursor els desenvolupadors, aplicacions i suma de preus obtingut en la descrrega

CURSOR CUR IS SELECT DISTINCT cr.desenvolupador, de.aplicacio, SUM(r.preu) FROM creacio cr, DESCARREGA de,
   APLICACIO ap,registre r
   WHERE de.aplicacio=ap.identificador AND ap.identificador=r.aplicacio  AND de.pais=r.pais 
   AND cr.aplicacio=ap.identificador  AND EXTRACT(YEAR FROM DE.dataDesc)=p_any
   GROUP BY cr.desenvolupador,de.aplicacio;


BEGIN

   param_entrada:='Any: '||p_any;

--controlem que el parmetre d'entrada no sigui null

   IF (p_any IS NULL) THEN
       RAISE E_NUL;
   END IF;

--obrim el cursor i iterem. S'insereixen noms aquells valors que no estiguin repetits per evitar repeticions posteriorment de registres
-- iguals amb la mxima recaptaci.
   
   OPEN CUR; 
   
   LOOP
     FETCH CUR INTO desenv,aplic,price;
     EXIT WHEN CUR%NOTFOUND;
      SELECT COUNT(*) INTO total FROM EST5 WHERE desenvolupador=desenv AND aplicacio=aplic AND recaptacio=price;
       IF (total=0) THEN
       INSERT INTO EST5(ao,desenvolupador,aplicacio,recaptacio) VALUES (p_any,desenv,aplic,price);
       END IF;
   END LOOP;
   CLOSE CUR;

--calculem la mxima recaptaci de cada any i esborrem tots aquells registres que no tinguin la recaptaci mxima
  
   SELECT MAX(recaptacio) INTO maxRecap FROM EST5 WHERE ao=p_any ;
   DELETE FROM est5 WHERE ao=p_any AND recaptacio<>maxRecap;
   
-- execuci correcta

   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST5', param_entrada, 'RSP: '||RSP);  
 

   

COMMIT;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST5', param_entrada, 'RSP: '||RSP);  
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST5', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

END SP_ALTA_EST5;
