/******************************************************************  EST6 *************************************************************************/
/****************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST6
   Propsit:            Donats un any i un pas passats com a parmetre d'entrada retorna el nombre d'usuaris que han fet 
                        com mnim una descrrega. 
                   
   Parmetres entrada:  p_any, p_pais            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
**********************************************************************************************************************************************/
create or replace PROCEDURE SP_ALTA_EST6(
        p_any        IN EST6.ao%TYPE,
        p_pais       IN EST6.pais%TYPE
  )

IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
valor_nul       VARCHAR2(30);
comptador       INTEGER;
suma            INTEGER;
E_NUL           EXCEPTION;

--emmagatzemem en el cursor l'any, el pais i el nombre total d'usuaris diferents que ha fet descrregues en aquell any i pas.

CURSOR CUR IS SELECT EXTRACT(year from datadesc),pais,COUNT(DISTINCT usuari) FROM DESCARREGA WHERE pais=p_pais and
              EXTRACT(year from datadesc)=p_any GROUP BY EXTRACT(year from datadesc),pais;

anyo           NUMBER;
country        CHAR(2);
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any||
                      ', Pais: '||p_pais;

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;
  END IF;

--obrim el cursor i iterem. Anem inserint en la taula EST6 l'any, el pas i el total de d'usuaris diferents que han fet descrregues i de manera
-- que no hi hagi files repetides

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,country,total;
         EXIT WHEN CUR%NOTFOUND;
        
            SELECT COUNT(*) INTO suma FROM EST6 WHERE ao=p_any AND pais=p_pais AND nombreUsuaris=total;

            IF(suma=0) THEN
            INSERT INTO EST6(ao,pais,nombreUsuaris) VALUES (p_any,p_pais,total);
            END IF;
       
       END LOOP;
     CLOSE CUR;
       
-- esborrem tots el registres que no tinguin el mxim nmero d'usuaris que fan descrregues.

       SELECT MAX(nombreusuaris) INTO total FROM est6 WHERE pais=p_pais AND ao=p_any;
       DELETE FROM EST6 WHERE nombreusuaris<>total AND ao=p_any AND pais=p_pais;
     

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST6', param_entrada, 'RSP: '||RSP);
       



EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST6', param_entrada, 'RSP: '||RSP);  
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST6', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST6;