/*****************************************************************  EST8 ************************************************************************/
/******************************************************************************
   Nom:                 SP_ALTA_EST8
   Propsit:            Donat un any i un pas passats com a parmetre d'entrada retorna el nombre d'aplicacions diferents descarregades
                        almenys una vegada en aquell any i pas.
                   
   Parmetres entrada:  p_any, p_pais            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/

create or replace
PROCEDURE SP_ALTA_EST8(
        p_any        IN EST8.ao%TYPE,
        p_pais       IN EST8.pais%TYPE
  )

IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
valor_nul       VARCHAR2(30);
comptador       INTEGER;
suma            INTEGER;
E_NUL           EXCEPTION;

-- emmagatzemem l'any, el pas i el nombre total de descrregues d'aplicacions diferents 

CURSOR CUR IS SELECT EXTRACT(year from dataDesc),pais,COUNT(DISTINCT aplicacio) FROM DESCARREGA 
              WHERE pais=p_pais AND EXTRACT(year from datadesc)=p_any 
              GROUP BY pais,EXTRACT(year from dataDesc);

country        CHAR(2);
anyo           NUMBER;
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any||
                      ', Pais: '||p_pais;

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;
  END IF;

--es van inserint en la taula EST8 de manera que no hi hagi registre repetits amb el mxim total d'aplicacions diferents

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,country,total;
         EXIT WHEN CUR%NOTFOUND;
         SELECT COUNT(*) INTO comptador FROM EST8 WHERE p_pais=pais AND ao=p_any AND total=nombreDescarregues;
         IF(comptador=0) THEN
            INSERT INTO EST8(ao,pais,nombreDescarregues) VALUES (p_any,p_pais,total);
         END IF;
       
       END LOOP;
     CLOSE CUR;

--esborrem tots aquells registres en qu el nombre d'aplicacions diferents s diferent de l'ltim comptat. D'aquest manera noms hi haur una 
-- fila en la taula.
       
       SELECT MAX(nombreDescarregues) INTO total FROM EST8 WHERE pais=p_pais AND ao=p_any;
       DELETE FROM EST8 WHERE nombreDescarregues<>total AND ao=p_any AND pais=p_pais;
     

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST8', param_entrada, 'RSP: '||RSP); 
      



EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST8', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST8', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST8;