
/***********************************************************  EST1    *******************************************************/
/***********************************************************************************************************************************************
   Nom:                 SP_ALTA_EST1
   Propsit:            Fa el clcul de totes les descrregues fetes i si s el primer valor calculat, l'afegeix  a la taula EST1
                        i si no s el primer, l'afegeix a EST1 havent esborrat abans l'antic valor de la taula. Aix doncs, en la taula 
                        sempre i haur un nic valor: el total de descrregues fetes.
   Parmetres entrada:  Sense parmetres           

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
***************************************************************************************************************************************************/

Create or replace PROCEDURE SP_ALTA_EST1

IS

--variables

param_entrada VARCHAR2(500);
totalDesc     INTEGER;
comptador     INTEGER;
RSP           VARCHAR2(500);
error_msg     VARCHAR2(500);


BEGIN
   param_entrada:='Sense parmetres';

-- calculem les descrregues fetes i el total de files inserides en la taula EST1

   SELECT COUNT(*) INTO totalDesc FROM DESCARREGA;
   SELECT COUNT(*) INTO comptador FROM EST1;

-- si la taula EST1 est buida afegim el valor del total de descrregues que s'han fet i si no est buida
-- esborrem l'ltim valor de la taula EST1 i afegim el nou valor (que ser l'antic +1)

   IF(comptador=0) THEN
   
   INSERT INTO EST1(totalDescarregues) VALUES (totaldesc);

   ELSE

    SELECT totalDescarregues INTO comptador FROM EST1;
    DELETE FROM EST1 WHERE totalDescarregues=comptador;
    INSERT INTO EST1(totalDescarregues) VALUES (totalDesc);

   END IF;

--  inserci correcta
   
    RSP:='OK';
    SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST1', param_entrada, 'RSP: '||RSP);
  


COMMIT;

EXCEPTION
 
 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST1', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST1;

/
/*************************************************************** EST2  *****************************************************************************************/
/*****************************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST2
   Propsit:            Calcula el total d'euros generats per la plataforma en descrregues fins el moment
                        del clcul. Afegeix aquest nombre a la taula EST2 si la taula est buida i sin, esborra
                        l'antic valor de EST2 i afegeix el nou valor: per tant, sempre hi haur un nic valor
                        que ser el total d'euros generats per la plataforma.
                   
   Parmetres entrada:  Sense parmetres            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
*****************************************************************************************************************************************************************/

Create or replace PROCEDURE SP_ALTA_EST2

IS

--variables

param_entrada VARCHAR2(500);
comptador     INTEGER;


SUMA            INTEGER:=0;
PRICE           NUMBER;
RSP             VARCHAR2(500);
error_msg       VARCHAR2(500);
TYPE REFCURSOR  IS REF CURSOR;
CUR REFCURSOR;


BEGIN

  param_entrada:='Sense parmetres';

--obrim un cursor amb tots els preus de cada pas de cada aplicaci descarregada. S'itera sumant cada preu per obtenir el total de euros recaudats

  OPEN CUR FOR SELECT R.PREU  FROM REGISTRE R,DESCARREGA DE,APLICACIO AP WHERE R.PAIS=DE.PAIS AND AP.IDENTIFICADOR=DE.APLICACIO AND
  AP.IDENTIFICADOR=R.APLICACIO;
  LOOP
   
    FETCH CUR INTO PRICE;
    EXIT WHEN CUR%NOTFOUND;
 
    suma:=suma+price;

  END LOOP;
  CLOSE CUR;

-- anem modificant el valor de la taula EST2 per cada descrrega. Esborrem el valor antic de la taula EST2 i inserim la nova suma.

  SELECT COUNT(*) INTO comptador FROM EST2;

  IF(comptador=0) THEN

     INSERT INTO EST2(totalEuros) VALUES (suma);

  ELSE
   
     SELECT totalEuros INTO comptador FROM EST2;
     DELETE FROM EST2 WHERE totalEuros=comptador;
     INSERT INTO EST2(totalEuros) VALUES (suma);
    
  END IF;

--  inserci correcta

    RSP:='OK';
    SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST2', param_entrada, 'RSP: '||RSP);
  


COMMIT;

EXCEPTION

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST2', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST2;

/

/*********************************************************   EST3    ***********************************************************************/
/********************************************************************************************************************************************
   Nom:                 SP_ALTA_EST3
   Propsit:            Donat un any passat com a parmetre d'entrada calcula la mitjana d'aplicacions descarregades, s a dir,
                        el total d'aplicacions descarregades en l'any donat dividit pel total d'usuaris diferents que han fet descrregues
                        aquell any.
                   
   Parmetres entrada:  p_any           

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
*******************************************************************************************************************************************************/
CREATE OR REPLACE PROCEDURE SP_ALTA_EST3 (
    p_any              NUMBER
    )
    
IS

--variables


param_entrada     VARCHAR2(50);
error_msg         VARCHAR2(100);
RSP               VARCHAR2(200);
mitja             NUMBER(9,2);
comptador         INTEGER;
nombreUsuaris     INTEGER;
nombreAplicacions INTEGER;
E_NUL             EXCEPTION;


BEGIN

  param_entrada:= 'Any: '||p_any;

--controlem que els parmetres d'entrada no siguin NULL

  IF(p_any IS NULL) THEN
     RAISE E_NUL;
  END IF;
  
--calculem el total d'aplicacions descarregades i el total d'usuaris diferents que  han descarregat aplicacions; Calculem la mitja

  SELECT COUNT(aplicacio) INTO nombreAplicacions FROM DESCARREGA de WHERE  EXTRACT(YEAR FROM dataDesc)=p_any;
  SELECT COUNT(DISTINCT usuari) INTO nombreUsuaris FROM DESCARREGA WHERE EXTRACT(YEAR FROM dataDesc)=p_any;
  
  mitja:=nombreAplicacions/nombreUsuaris;

--inserim els valors si la taula EST3 est buida o la modifiquem amb la nova mitja sin est buida
  
  SELECT COUNT(*) INTO comptador FROM EST3 WHERE ao=p_any;
  
  IF(comptador=0) THEN
  
      INSERT INTO EST3(ao, mitjana) VALUES (p_any,mitja);
      
  ELSE
    
      UPDATE EST3 SET mitjana=mitja WHERE ao=p_any;
      
  END IF;

--inserci o modificaci correcta
   
    RSP:='OK';
    SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST3', param_entrada, 'RSP: '||RSP);
  

    
COMMIT;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST3', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
 

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST3', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST3;

/

/****************************************************************   EST4   ************************************************************************/
/**************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST4
   Propsit:            Donat un any passat com a parmetre d'entrada, retorna el desenvolupador o desenvolupadors que han 
                        creat de l'aplicaci o aplicacions ms descarregades en la plataforma l'any donat, junt amb el nombre de 
                        descrregues que han tingut.
   Parmetres entrada:  p_any            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
****************************************************************************************************************************************************/
create or replace PROCEDURE SP_ALTA_EST4(
     p_any          NUMBER
     )
IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
desenv          VARCHAR2(10);
totalDesc       INTEGER;
comptador       INTEGER;
maxim           INTEGER;

E_NUL           EXCEPTION;

type refcursor  IS REF CURSOR;
CUR  refcursor;


BEGIN

   param_entrada:='Any: '||p_any;

-- controlem que el parmetre d'entrada no sigui null

   IF (p_any IS NULL) THEN
      RAISE E_NUL;
   END IF;

--obrim un cursor amb tots els desenvolupadors i el nombre d'aplicacions descarregades en l'any donat
   
   OPEN CUR FOR SELECT cr.desenvolupador, COUNT(de.aplicacio) AS total from DESCARREGA de,APLICACIO ap,CREACIO cr
   WHERE de.aplicacio=ap.identificador  AND ap.identificador=cr.aplicacio AND EXTRACT(YEAR FROM DE.dataDesc)=p_any
   GROUP BY cr.desenvolupador ORDER BY total ASC;

-- anem inserint en la taula EST4 el valor de les variables per cada iteraci de manera que no hi hagi files repetides en les iteracions
   
   LOOP
     FETCH CUR INTO desenv,totalDesc;
     EXIT WHEN CUR%NOTFOUND;

     SELECT COUNT(*) INTO comptador FROM EST4 WHERE ao=p_any AND desenvolupador=desenv AND nombre=totalDesc;

     
     IF(comptador=0) THEN
     INSERT INTO EST4(ao,desenvolupador,nombre) VALUES(p_any,desenv,totalDesc);
     END IF;

  END LOOP;
  CLOSE CUR;

--seleccionem el nombre mxim de descrregues i esborrem totes aquelles files que no tinguin el mxim nombre de descrregues; 
-- aix en la taula quedaran nomes l'any, i els desenvolupadors que han tingut mxim nombre de descrregues junt amb aquest nombre.
   
   
  
       SELECT MAX(nombre) INTO maxim FROM EST4 WHERE ao=p_any;

       DELETE FROM EST4 WHERE ao=p_any AND nombre<>maxim;
       
       
  

--inserci o modificaci correcta
  
   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST4', param_entrada, 'RSP: '||RSP);
 


COMMIT;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST4', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST4', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

END SP_ALTA_EST4;

/

/*******************************************************************    EST5   ******************************************************************/
/********************************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST5
   Propsit:            Donat un any passat com a parmetre d'entrada, retorna el desenvolupador o desenvolupadors que han creat l'aplicaci o aplicacions (en cas
                        de que dues aplicacions hagin recaudat el mateix) que han recaudat ms diners en la plataforma. 
                   
   Parmetres entrada:  p_any            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
*******************************************************************************************************************************************************************/

create or replace PROCEDURE SP_ALTA_EST5(
     p_any          NUMBER
     )
IS


--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);

aplic           NUMBER;
price           NUMBER;
desenv          VARCHAR2(10);
maxRecap        NUMBER;
total           INTEGER:=0;

E_NUL           EXCEPTION;

-- emmagatzemem en el cursor els desenvolupadors, aplicacions i suma de preus obtingut en la descrrega

CURSOR CUR IS SELECT DISTINCT cr.desenvolupador, de.aplicacio, SUM(r.preu) FROM creacio cr, DESCARREGA de,
   APLICACIO ap,registre r
   WHERE de.aplicacio=ap.identificador AND ap.identificador=r.aplicacio  AND de.pais=r.pais 
   AND cr.aplicacio=ap.identificador  AND EXTRACT(YEAR FROM DE.dataDesc)=p_any
   GROUP BY cr.desenvolupador,de.aplicacio;


BEGIN

   param_entrada:='Any: '||p_any;

--controlem que el parmetre d'entrada no sigui null

   IF (p_any IS NULL) THEN
       RAISE E_NUL;
   END IF;

--obrim el cursor i iterem. S'insereixen noms aquells valors que no estiguin repetits per evitar repeticions posteriorment de registres
-- iguals amb la mxima recaptaci.
   
   OPEN CUR; 
   
   LOOP
     FETCH CUR INTO desenv,aplic,price;
     EXIT WHEN CUR%NOTFOUND;
      SELECT COUNT(*) INTO total FROM EST5 WHERE desenvolupador=desenv AND aplicacio=aplic AND recaptacio=price;
       IF (total=0) THEN
       INSERT INTO EST5(ao,desenvolupador,aplicacio,recaptacio) VALUES (p_any,desenv,aplic,price);
       END IF;
   END LOOP;
   CLOSE CUR;

--calculem la mxima recaptaci de cada any i esborrem tots aquells registres que no tinguin la recaptaci mxima
  
   SELECT MAX(recaptacio) INTO maxRecap FROM EST5 WHERE ao=p_any ;
   DELETE FROM est5 WHERE ao=p_any AND recaptacio<>maxRecap;
   
-- execuci correcta

   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST5', param_entrada, 'RSP: '||RSP);  
 

   

COMMIT;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST5', param_entrada, 'RSP: '||RSP);  
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST5', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

END SP_ALTA_EST5;

/


/******************************************************************  EST6 *************************************************************************/
/****************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST6
   Propsit:            Donats un any i un pas passats com a parmetre d'entrada retorna el nombre d'usuaris que han fet 
                        com mnim una descrrega. 
                   
   Parmetres entrada:  p_any, p_pais            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
**********************************************************************************************************************************************/
create or replace PROCEDURE SP_ALTA_EST6(
        p_any        IN EST6.ao%TYPE,
        p_pais       IN EST6.pais%TYPE
  )

IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
valor_nul       VARCHAR2(30);
comptador       INTEGER;
suma            INTEGER;
E_NUL           EXCEPTION;

--emmagatzemem en el cursor l'any, el pais i el nombre total d'usuaris diferents que ha fet descrregues en aquell any i pas.

CURSOR CUR IS SELECT EXTRACT(year from datadesc),pais,COUNT(DISTINCT usuari) FROM DESCARREGA WHERE pais=p_pais and
              EXTRACT(year from datadesc)=p_any GROUP BY EXTRACT(year from datadesc),pais;

anyo           NUMBER;
country        CHAR(2);
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any||
                      ', Pais: '||p_pais;

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;
  END IF;

--obrim el cursor i iterem. Anem inserint en la taula EST6 l'any, el pas i el total de d'usuaris diferents que han fet descrregues i de manera
-- que no hi hagi files repetides

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,country,total;
         EXIT WHEN CUR%NOTFOUND;
        
            SELECT COUNT(*) INTO suma FROM EST6 WHERE ao=p_any AND pais=p_pais AND nombreUsuaris=total;

            IF(suma=0) THEN
            INSERT INTO EST6(ao,pais,nombreUsuaris) VALUES (p_any,p_pais,total);
            END IF;
       
       END LOOP;
     CLOSE CUR;
       
-- esborrem tots el registres que no tinguin el mxim nmero d'usuaris que fan descrregues.

       SELECT MAX(nombreusuaris) INTO total FROM est6 WHERE pais=p_pais AND ao=p_any;
       DELETE FROM EST6 WHERE nombreusuaris<>total AND ao=p_any AND pais=p_pais;
     

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST6', param_entrada, 'RSP: '||RSP);
       



EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST6', param_entrada, 'RSP: '||RSP);  
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST6', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST6;


/

/*******************************************************************  EST7   ************************************************************************/
/*******************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST7
   Propsit:            Donat una any i un pas passats com a parmetre d'entrada retorna els ingressos que han generat els usuaris d'aquell
                        pas en aquell any en descrregues d'aplicacions. 
                   
   Parmetres entrada:  p_any,p_pais            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************************************************************************************/
create or replace PROCEDURE SP_ALTA_EST7(
        p_any        IN EST7.ao%TYPE,
        p_pais       IN EST7.pais%TYPE
  )

IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
valor_nul       VARCHAR2(30);
comptador       INTEGER;
suma            INTEGER;
E_NUL           EXCEPTION;

-- emmagatzemem en el cursor l'any, el pas de descrrega i la suma d'ingresos per aquell pas i any

CURSOR CUR IS SELECT EXTRACT(year from de.dataDesc),de.pais,SUM(r.preu) FROM DESCARREGA de,APLICACIO ap,REGISTRE r WHERE r.pais=p_pais AND
              de.pais=p_pais AND extract(year from de.datadesc)=p_any AND de.aplicacio=ap.identificador AND ap.identificador=r.aplicacio
              GROUP BY de.pais,EXTRACT(year from de.dataDesc);

country        CHAR(2);
anyo           NUMBER;
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any||
                      ', Pais: '||p_pais;

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;
  END IF;

--es van inserint en la taula EST7 de manera que no hi hagi registre repetits amb el mxim nombre d'ingressos.

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,country,total;
         EXIT WHEN CUR%NOTFOUND;
         SELECT COUNT(*) INTO comptador FROM EST7 WHERE p_pais=pais AND ao=p_any AND total=ingressos;
         IF(comptador=0) THEN
            INSERT INTO EST7(ao,pais,ingressos) VALUES (p_any,p_pais,total);
         END IF;
       
       END LOOP;
     CLOSE CUR;
       
-- esborrem de la taula tots aquells registres que no tinguin el major nombre d'ingressos per a l'any i pas donat.

       SELECT MAX(ingressos) INTO total FROM est7 WHERE pais=p_pais AND ao=p_any;
       DELETE FROM EST7 WHERE ingressos<>total AND ao=p_any AND pais=p_pais;
     

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST7', param_entrada, 'RSP: '||RSP); 
        



EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST7', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST7', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST7;


/

/*****************************************************************  EST8 ************************************************************************/
/************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST8
   Propsit:            Donat un any i un pas passats com a parmetre d'entrada retorna el nombre d'aplicacions diferents descarregades
                        almenys una vegada en aquell any i pas.
                   
   Parmetres entrada:  p_any, p_pais            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
************************************************************************************************************************************************/

create or replace
PROCEDURE SP_ALTA_EST8(
        p_any        IN EST8.ao%TYPE,
        p_pais       IN EST8.pais%TYPE
  )

IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
valor_nul       VARCHAR2(30);
comptador       INTEGER;
suma            INTEGER;
E_NUL           EXCEPTION;

-- emmagatzemem l'any, el pas i el nombre total de descrregues d'aplicacions diferents 

CURSOR CUR IS SELECT EXTRACT(year from dataDesc),pais,COUNT(DISTINCT aplicacio) FROM DESCARREGA 
              WHERE pais=p_pais AND EXTRACT(year from datadesc)=p_any 
              GROUP BY pais,EXTRACT(year from dataDesc);

country        CHAR(2);
anyo           NUMBER;
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any||
                      ', Pais: '||p_pais;

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;
  END IF;

--es van inserint en la taula EST8 de manera que no hi hagi registre repetits amb el mxim total d'aplicacions diferents

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,country,total;
         EXIT WHEN CUR%NOTFOUND;
         SELECT COUNT(*) INTO comptador FROM EST8 WHERE p_pais=pais AND ao=p_any AND total=nombreDescarregues;
         IF(comptador=0) THEN
            INSERT INTO EST8(ao,pais,nombreDescarregues) VALUES (p_any,p_pais,total);
         END IF;
       
       END LOOP;
     CLOSE CUR;

--esborrem tots aquells registres en qu el nombre d'aplicacions diferents s diferent de l'ltim comptat. D'aquest manera noms hi haur una 
-- fila en la taula.
       
       SELECT MAX(nombreDescarregues) INTO total FROM EST8 WHERE pais=p_pais AND ao=p_any;
       DELETE FROM EST8 WHERE nombreDescarregues<>total AND ao=p_any AND pais=p_pais;
     

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST8', param_entrada, 'RSP: '||RSP); 
      



EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST8', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST8', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST8;
/




















  
