CREATE OR REPLACE PROCEDURE SP_ALTA_USUARI (
    p_NIF      IN  PERSONA.NIF%TYPE,
    p_telefon  IN  PERSONA.telefon%TYPE,
    p_email    IN  PERSONA.email%TYPE,
    p_pais     IN  USUARI.pais%TYPE,
    p_operador IN  USUARI.operador%TYPE,
    RSP        OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_ALTA_USUARI
   Funci:                   Afegeix un usuari a la taula USUARI i si no existeix en la taula PERSONA, l'afegeix tamb
   Parmetre d'entrada:      - p_NIF: el NIF de l'usuari
                             - p_telefon: telefon de l'usuari
                             - p_email: email de l'usuari
                             - p_pais: pais de registre de l'usuari 
                             - p_operador: operador telefnic de l'usuari

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/

param_entradaPersona  VARCHAR2(500);
param_entradaUsuari   VARCHAR2(500);
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;
E_PK1 EXCEPTION;

BEGIN

  param_entradaPersona:= 'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', email: '||p_email;

  param_entradaUsuari:= 'NIF: '||p_NIF||
                        ', Pais: '||p_pais||
                        ', Operador: '||p_operador;
 
  param_entrada:=       'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', Email: '||p_email||
                         ', Pais: '||p_pais||
                         ', Operador: '||p_operador;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  ELSIF (p_operador IS NULL) THEN
      valor_nul:='operador';
      RAISE E_NUL;
  END IF;

-- inserim el usuari en la taula Persona si prviament no existeix; sin generem una exepci

   SELECT COUNT(*) INTO comptador FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0) THEN
      
       RSP:='OK';
   
      INSERT INTO PERSONA (NIF,telefon,email) VALUES (p_NIF,p_telefon,p_email);
  
   ELSE 

       RAISE E_PK1;

   END IF;

-- inserim el nou usuari si no existeix; sin generem una excepci

   SELECT COUNT(*) INTO comptador FROM USUARI U WHERE U.NIF=p_NIF;
 
   IF (comptador=0) THEN

      INSERT INTO USUARI(NIF, pais, operador) VALUES (p_NIF,p_pais, p_operador);
  
   ELSE
      RAISE E_PK;
  
   END IF;

-- inserci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:='ERROR: Ja existeix aquest usuari en la taula USUARI';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
    WHEN E_PK1 THEN
      RSP:='Ja existeix aquest usuari en la taula PERSONA';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
      ROLLBACK;

      --violaci de la clau nica que s el telfon, ja que les altres claus niques estan controlades per E_PK i E_PK1
      IF (SQLCODE=-00001) THEN
          RSP:='ERROR: ja existeix un usuari o un desenvolupador amb el mateix telefon';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entradaPersona, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);

      ELSE
         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

      END IF;
END SP_ALTA_USUARI;