CREATE OR REPLACE PROCEDURE SP_DESCARREGA (
      p_usuari     IN   DESCARREGA.usuari%TYPE,
      p_aplicacio  IN   DESCARREGA.aplicacio%TYPE,
      p_data       IN   DESCARREGA.dataDesc%TYPE,
      p_pagament   IN   DESCARREGA.pagament%TYPE,
      p_operador   IN   DESCARREGA.operador%TYPE,
      p_pais       IN   DESCARREGA.pais%TYPE,
      p_dispositiu IN   DESCARREGA.dispositiu%TYPE,
      RSP          OUT  VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_DESCARREGA
   Funci:                   Emmagatzema una descrrega d'una aplicaci en el sistema
   Parmetre d'entrada:      - p_usuari: usuari que realitza la descrrega
                             - p_aplicacio: identificador de l'aplicaci
                             - p_data: data de descrrega de l'aplicaci
                             - p_pagament: mode de pagament 
                             - p_operador: operador de telefonia en el moment de la descrrega
                             _ p_dispositiu: identificador del dispositiu en qu s'ha fet la descrrega (IMEI).

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada       VARCHAR2(500);
error_msg           VARCHAR2(200);
valor_nul           VARCHAR2(100);
comptador           INTEGER;
resolucioAplicacio  NUMBER;
resolucioDispositiu NUMBER;
valor               VARCHAR2(100);
E_NUL               EXCEPTION;
E_FK                EXCEPTION;
E_RESOL             EXCEPTION;
E_DATE              EXCEPTION;




BEGIN
  param_entrada:= 'Usuari: '||p_usuari||
                  ', Aplicacio: '||p_aplicacio||
                  ', Data: '||p_data||
                  ', Pagament: '||p_pagament||
                  ', Operador: '||p_operador||
                  ', Pais: '||p_pais||
                  ', Dispositiu: '||p_dispositiu;

-- verifiquem que no hi hagi valors nuls

   IF (p_usuari IS NULL) THEN
       valor_nul:='usuari';
       RAISE E_NUL;
   ELSIF (p_aplicacio IS NULL) THEN
       valor_nul:='aplicacio';
       RAISE E_NUL;
   ELSIF (p_data IS NULL) THEN
       valor_nul:='data';
       RAISE E_NUL;
   ELSIF (p_pagament IS NULL) THEN
       valor_nul:='pagament';
       RAISE E_NUL;
   ELSIF (p_operador IS NULL) THEN
       valor_nul:='operador';
       RAISE E_NUL;
   ELSIF (p_pais IS NULL) THEN
       valor_nul:='pais';
       RAISE E_NUL;
   ELSIF (p_dispositiu IS NULL) THEN
       valor_nul:='dispositiu';
       RAISE E_NUL;
       
   END IF;

-- verifiquem que l'usuari existeix prviament en la taula USUARI

    SELECT COUNT(*) INTO comptador FROM USUARI WHERE NIF=p_usuari;
    
    IF(comptador=0) THEN
      valor:='usuari';
      RAISE E_FK;
    END IF;

-- verifiquem que l'aplicaci existeix prviament en la taula APLICACIO

    SELECT COUNT(*) INTO comptador FROM APLICACIO WHERE identificador=p_aplicacio;
    
    IF(comptador=0) THEN
      valor:='aplicacio';
      RAISE E_FK;
    END IF;

-- verifiquem que el dispositiu existeix prviament en la taula DISPOSITIU

    SELECT COUNT(*) INTO comptador FROM DISPOSITIU WHERE IMEI=p_dispositiu;
    
    IF(comptador=0) THEN
      valor:='dispositiu';
      RAISE E_FK;
    END IF;

-- verifiquem que la resoluci de pantalla sigui major que la donada per l'aplicaci

    SELECT resolucio INTO resolucioAplicacio FROM APLICACIO WHERE identificador=p_aplicacio;
    SELECT resolucio INTO resolucioDispositiu FROM DISPOSITIU WHERE IMEI=p_dispositiu;

    IF (resolucioAplicacio > resolucioDispositiu ) THEN
       RAISE E_RESOL;
    END IF;

-- verifiquem que la data de descarrega no sigui major que l'actual
    
    IF (to_date(p_data)>to_date(sysdate,'DD/MM/YY')) THEN
       RAISE E_DATE;
    END IF;


-- insertem la nova descrrega en la taula DESCARREGA

    INSERT INTO DESCARREGA(usuari,aplicacio,dataDesc,pagament,operador,pais,dispositiu) VALUES (p_usuari,p_aplicacio,p_data,p_pagament,p_operador,p_pais,p_dispositiu);

-- crida als procediments estadstics de manera que estiguin actualitzats en cada descrrega de la plataforma


       SP_ALTA_EST1;
       SP_ALTA_EST2;
       SP_ALTA_EST3(EXTRACT(YEAR FROM P_DATA));
       SP_ALTA_EST4(EXTRACT(YEAR FROM P_DATA));
       SP_ALTA_EST5(EXTRACT(YEAR FROM P_DATA));
       SP_ALTA_EST6(EXTRACT(YEAR FROM P_DATA),p_pais);
       SP_ALTA_EST7(EXTRACT(YEAR FROM P_DATA),p_pais);
       SP_ALTA_EST8(EXTRACT(YEAR FROM P_DATA),p_pais);


--  inserci correcta
    RSP:='OK';
    SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);



COMMIT;


EXCEPTION

   WHEN E_NUL THEN
        RSP:= 'ERROR: el valor ' || valor_nul|| ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
    WHEN E_FK THEN  
        RSP:= 'ERROR: el ' || valor || ' no existeix en la taula '||UPPER(valor);       
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
    

    WHEN E_RESOL THEN
         RSP:= 'ERROR: La resoluci del dispositiu s menor que la demanada per l-aplicaci';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

    WHEN E_DATE THEN
         RSP:= 'ERROR: La data de descrrega no pot ser major a l-actual';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

        
   WHEN OTHERS THEN
        ROLLBACK;
       
        
          error_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || error_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);
        
              
END SP_DESCARREGA;