CREATE OR REPLACE PROCEDURE SP_BAIXA_USUARI (
    p_NIF      IN  PERSONA.NIF%TYPE,
    RSP        OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_BAIXA_USUARI
   Funci:                   Esborra un usuari a la taula USUARI i si no existeix en la taula DESENVOLUPADOR
                             l'esborra de la taula PERSONA
   Parmetre d'entrada:      - p_NIF: el NIF de l'usuari
                             

   Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(50);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
comptador2 INTEGER;
E_NUL  EXCEPTION;
E_EXISTS EXCEPTION;


BEGIN
 
  param_entrada:= 'NIF: '||p_NIF;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  
  END IF;
  
  -- esborrem l'usuari de la taula USUARI; sin existeix generem una excepci
   
   SELECT COUNT(*) INTO comptador FROM USUARI U WHERE U.NIF=p_NIF;
 
   IF (comptador>0) THEN

      DELETE FROM USUARI WHERE NIF=p_NIF;
  
   ELSE
     
      RAISE E_EXISTS;
  
   END IF;


-- si l'usuari no s desenvolupador alhora, l'esborrem de la taula PERSONA

   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR D WHERE D.NIF=p_NIF;
   SELECT COUNT(*) INTO comptador2 FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0 AND comptador2=1) THEN
   
      DELETE FROM PERSONA WHERE NIF=p_NIF;
  
   END IF;


-- esborrament correcte

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_EXISTS THEN
      RSP:='ERROR: aquest usuari no existeix en la taula USUARI';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

    
   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


      
END SP_BAIXA_USUARI;