
CREATE OR REPLACE PROCEDURE SP_ALTA_DESENVOLUPADOR (
    p_NIF           IN  PERSONA.NIF%TYPE,
    p_telefon       IN  PERSONA.telefon%TYPE,
    p_email         IN  PERSONA.email%TYPE,
    p_pais          IN  USUARI.pais%TYPE,
    p_dataNaixement IN  DESENVOLUPADOR.dataNaixement%TYPE,
    p_llocNaixement IN  DESENVOLUPADOR.llocNaixement%TYPE,
    RSP             OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_ALTA_DESENVOLUPADOR
   Funci:                   Afegeix un desenvolupador a la taula DESENVOLUPADOR i si no existeix en la taula PERSONA,
                             l'afegeix tamb
   Parmetre d'entrada:      - p_NIF: el NIF del desenvolupador
                             - p_telefon: telefon del desenvolupador
                             - p_email: email del desenvolupador
                             - p_pais: pais de registre del desenvolupador
                             - p_dataNaixement: data de naixement del desenvolupador
                             - P_llocNaixement: poblaci de naixement del desenvolupador

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entradaPersona  VARCHAR2(500);
param_entradaDesenvolupador   VARCHAR2(500);
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK_DESENVOLUPADOR EXCEPTION;

BEGIN

  param_entradaPersona:= 'NIF: '||p_NIF||
                         ', telefon: '||p_telefon||
                         ', email: '||p_email;

  param_entradaDesenvolupador:= 'NIF: '||p_NIF||
                        ', pais: '||p_pais||
                        ', dataNaixement: '||p_dataNaixement||
                        ', llocNaixement: '||p_llocNaixement;
 
  param_entrada:=        'NIF: '||p_NIF||
                         ', telefon: '||p_telefon||
                         ', email: '||p_email||
                         ', pais: '||p_pais||
                         ', dataNaixement: '||p_dataNaixement||
                         ', llocNaixement: '||p_llocNaixement;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  ELSIF (p_dataNaixement IS NULL) THEN
      valor_nul:='dataNaixement';
      RAISE E_NUL;
  END IF;

-- inserim el usuari en la taula Persona si prviament no existeix.

   SELECT COUNT(*) INTO comptador FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0) THEN
      
       INSERT INTO PERSONA (NIF,telefon,email) VALUES (p_NIF,p_telefon,p_email);
   
   END IF;


-- inserim el nou usuari sin existeix en la taula DESENVOLUPADOR; si existeix generem una excepci

   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR D WHERE D.NIF=p_NIF;
 
   IF (comptador=0) THEN

      INSERT INTO DESENVOLUPADOR(NIF, pais, dataNaixement, llocNaixement) VALUES (p_NIF,p_pais, p_dataNaixement,p_llocNaixement);
     
   ELSE
     
      RAISE E_PK_DESENVOLUPADOR;
  
   END IF;

-- inserci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entradaDesenvolupador, 'RSP: '||RSP);
   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
  
    WHEN E_PK_DESENVOLUPADOR THEN
      RSP:='ERROR: Ja existeix aquest Desenvolupador en la taula DESENVOLUPADOR';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entradaDesenvolupador, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
      ROLLBACK;

      --violaci de la clau nica
      IF (SQLCODE=-00001) THEN
          RSP:='ERROR: ja existeix un usuari en la taula PERSONA amb el mateix telefon';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entradaPersona, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);

      -- qualsevol error no descrit anteriorment
      ELSE
         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

      END IF;
END SP_ALTA_DESENVOLUPADOR;
    
    