CREATE OR REPLACE PROCEDURE SP_MODIFICACIO_DESENVOLUPADOR (
    p_NIF           IN  PERSONA.NIF%TYPE,
    p_telefon       IN  PERSONA.telefon%TYPE,
    p_email         IN  PERSONA.email%TYPE,
    p_llocNaixement IN  DESENVOLUPADOR.llocNaixement%TYPE,
    p_dataNaixement IN  DESENVOLUPADOR.dataNaixement%TYPE,
    p_pais          IN  DESENVOLUPADOR.pais%TYPE,
    RSP             OUT VARCHAR2
)
IS


/************************************************************************************************************************
   Nom:                      SP_MODIFICACIO_DESENVOLUPADOR
   Funci:                   Modifica les dades d'un desenvolupador de la taula DESENVOLUPADOR i de la taula PERSONA
   Parmetre d'entrada:      - p_NIF: el NIF del desenvolupador
                             - p_telefon: telefon del desenvolupador
                             - p_email: email del desenvolupador
                             - p_pais: pais de registre del desenvolupador
                             - p_dataNaixement: data de naixement del desenvolupador
                             - P_llocNaixement: poblaci de naixement del desenvolupador

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
comptador2 INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;
E_UQ EXCEPTION;


BEGIN

 
  param_entrada:=        'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', email: '||p_email||
                         ', LlocNaixement: '||p_llocNaixement||
                         ', DataNaixement: '||p_dataNaixement||
                         ', Pais: '||p_pais;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  END IF;

--verifiquem que el desenvolupador existeixi en la taula PERSONA i en la taula DESENVOLUPADOR

   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR  WHERE NIF=p_NIF;
   SELECT COUNT(*) INTO comptador2 FROM PERSONA WHERE NIF=p_NIF;
 
   IF (comptador=0 AND comptador2=0) THEN

      RAISE E_PK;
  
   END IF;
  

-- verifiquem que el nmero de telefon que es vol modificar no existeixi.

   
   SELECT COUNT(*) INTO comptador FROM PERSONA WHERE telefon=p_telefon;
   
   IF (comptador>0) THEN
   
      RAISE E_UQ;
   
   END IF;

-- modificaci dels valors
   
   UPDATE PERSONA SET telefon=p_telefon, email=p_email WHERE NIF=p_NIF;
   UPDATE DESENVOLUPADOR SET pais=p_pais, dataNaixement=p_dataNaixement,llocNaixement=p_llocNaixement WHERE NIF=p_NIF;

-- modificaci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:='ERROR: No existeix aquest desenvolupador';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_UQ THEN
      RSP:='ERROR: el telfon que es vol modificar ja existeix en la base da dades';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
    
   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

     
END SP_MODIFICACIO_DESENVOLUPADOR;