CREATE OR REPLACE PROCEDURE SP_ALTA_APLICACIO (
    p_identificador IN  APLICACIO.identificador%TYPE,
    p_versio        IN  APLICACIO.versio%TYPE,
    p_dataPujada    IN  APLICACIO.dataPujada%TYPE,
    p_URL           IN  APLICACIO.URL%TYPE,
    p_activa        IN  APLICACIO.activa%TYPE,
    p_resolucio     IN APLICACIO.resolucio%TYPE,
    RSP             OUT VARCHAR2
)
IS


/************************************************************************************************************************
   Nom:                      SP_ALTA_APLICACIO
   Funci:                   Afegeix una aplicaci a la taula APLICACIO
   Parmetre d'entrada:      - p_identificador: identificador de l'aplicaci
                             - p_versio: versi de l'aplicaci
                             - p_dataPujada: data de pujada de l'aplicaci
                             - p_URL: enlla al video de demostraci de l'aplicaci
                             - p_resoluci: resoluci mnima de pantalla que ha de tenir el dispositiu on es descarregar

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador  INTEGER;
E_NUL      EXCEPTION;
E_DATE     EXCEPTION;


BEGIN

 
  param_entrada:=        'Identificador: '||p_identificador||
                         ', Versio: '||p_versio||
                         ', DataPujada: '||p_dataPujada||
                         ', URL: '||p_URL||
                         ', Activa: '||p_activa||
                         ', Resolucio: '||p_resolucio;



-- verificaci de parmetres no nuls

  IF (p_identificador IS NULL) THEN
      valor_nul:='identificador';
      RAISE E_NUL;
  ELSIF (p_versio IS NULL) THEN
      valor_nul:='versio';
      RAISE E_NUL;
  ELSIF (p_dataPujada IS NULL) THEN
      valor_nul:='dataPujada';
      RAISE E_NUL;
  ELSIF (p_activa IS NULL) THEN
      valor_nul:='activa';
      RAISE E_NUL;
  ELSIF (p_resolucio IS NULL) THEN
      valor_nul:='resolucio';
      RAISE E_NUL;
  END IF;

-- verifiquem que la data de descarrega no sigui major que l'actual
    
    IF (to_date(p_dataPujada)>to_date(sysdate,'DD/MM/YY')) THEN
       
       RAISE E_DATE;
  
    END IF;



-- inserim l'aplicaci
   
   
      INSERT INTO APLICACIO(identificador,versio,dataPujada,URL,activa,resolucio) VALUES (p_identificador,p_versio,p_dataPujada,p_URL,p_activa,p_resolucio);
  

-- inserci correcta

     RSP:='OK';
     SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_DATE THEN
         RSP:= 'ERROR: La data de pujada no pot ser major a l-actual';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
  
   WHEN OTHERS THEN
      ROLLBACK;

      --es comproba que les entrades amb clusules check siguin correctes
      IF (SQLCODE=-02290) THEN
          RSP:='ERROR: el valor activa ha de ser SI  NO';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);
    
      ELSIF(SQLCODE=-00001) THEN
          RSP:='ERROR: la aplicaci ja existeix. Violaci de la clau primria';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);
          
      ELSE
         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
         
      END IF;

END SP_ALTA_APLICACIO;
    