CREATE OR REPLACE PROCEDURE SP_BAIXA_APLICACIO (
    p_ident       IN  APLICACIO.identificador%TYPE,
    RSP           OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_BAIXA_APLICACIO
   Funci:                   Esborra una aplicaci de la taula APLICACIO
   Parmetre d'entrada:      - p_identificador: identificador de l'aplicaci

   Parmetre de sortida:     RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;

BEGIN

 
  param_entrada:=        'Identificador: '||p_ident;
                         

-- verificaci de parmetres no nuls

  IF (p_ident IS NULL) THEN
      valor_nul:='ident';
      RAISE E_NUL;
  
  END IF;

-- comprovaci de que l'aplicaci existeixi
   
   
      SELECT COUNT(*) INTO comptador FROM APLICACIO WHERE identificador=p_ident;
      
      IF(comptador>0) THEN

        DELETE FROM APLICACIO WHERE identificador=p_ident;
  
      ELSE
        
         RAISE E_PK;
   
     END IF; 
  -- operacio correcta

     RSP:='OK';
     SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);


   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:= 'ERROR: L-aplicaci no existeix en la taula APLICACIO';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
      
END SP_BAIXA_APLICACIO;
    