CREATE OR REPLACE PROCEDURE SP_MODIFICACIO_APLICACIO (
    p_ident       IN  APLICACIO.identificador%TYPE,
    p_versio      IN  APLICACIO.versio%TYPE,
    p_dataPujada  IN  APLICACIO.dataPujada%TYPE,
    p_URL         IN  APLICACIO.URL%TYPE,
    p_activa      IN  APLICACIO.activa%TYPE,
    p_resolucio   IN APLICACIO.resolucio%TYPE,
    RSP           OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_MOFICACIO_APLICACIO
   Funci:                   Modifica les dades d'una aplicaci de la taula APLICACI
   Parmetre d'entrada:      - p_identificador: identificador de l'aplicaci
                             - p_versio: versi de l'aplicaci
                             - p_dataPujada: data de pujada de l'aplicaci
                             - p_URL: enlla al video de demostraci de l'aplicaci
                             - p_resoluci: resoluci mnima de pantalla que ha de tenir el dispositiu on es descarregar

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;

BEGIN

 
  param_entrada:=        'Identificador: '||p_ident||
                         ' Versio: '||p_versio||
                         ', DataPujada: '||p_dataPujada||
                         ', URL: '||p_URL||
                         ', Activa: '||p_activa||
                         ', Resolucio: '||p_resolucio;



-- verificaci de parmetres no nuls

  IF (p_ident IS NULL) THEN
      valor_nul:='identificador';
      RAISE E_NUL;
  ELSIF(p_versio IS NULL) THEN
      valor_nul:='versio';
  ELSIF (p_dataPujada IS NULL) THEN
      valor_nul:='dataPujada';
      RAISE E_NUL;
  ELSIF (p_activa IS NULL) THEN
      valor_nul:='activa';
      RAISE E_NUL;
  ELSIF (p_resolucio IS NULL) THEN
      valor_nul:='resolucio';
      RAISE E_NUL;
  END IF;

-- comprovaci de que l'aplicaci existeixi
   
   
      SELECT COUNT(*) INTO comptador FROM APLICACIO WHERE identificador=p_ident;
      
      IF(comptador>0) THEN

         UPDATE APLICACIO SET versio=p_versio,dataPujada=p_dataPujada,URL=p_URL,activa=UPPER(p_activa),resolucio=p_resolucio
         WHERE identificador=p_ident;
  
      ELSE
        
         RAISE E_PK;
   
      END IF;

  -- inserci correcta

     RSP:='OK';
     SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada, 'RSP: '||RSP);


   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:= 'ERROR: l-aplicaci no existeix en la taula APLICACIO';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada,'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      

   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
      
END SP_MODIFICACIO_APLICACIO;
    