

CREATE OR REPLACE PROCEDURE SP_ALTA_USUARI (
    p_NIF      IN  PERSONA.NIF%TYPE,
    p_telefon  IN  PERSONA.telefon%TYPE,
    p_email    IN  PERSONA.email%TYPE,
    p_pais     IN  USUARI.pais%TYPE,
    p_operador IN  USUARI.operador%TYPE,
    RSP        OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_ALTA_USUARI
   Funci:                   Afegeix un usuari a la taula USUARI i si no existeix en la taula PERSONA, l'afegeix tamb
   Parmetre d'entrada:      - p_NIF: el NIF de l'usuari
                             - p_telefon: telefon de l'usuari
                             - p_email: email de l'usuari
                             - p_pais: pais de registre de l'usuari 
                             - p_operador: operador telefnic de l'usuari

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/

param_entradaPersona  VARCHAR2(500);
param_entradaUsuari   VARCHAR2(500);
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;
E_PK1 EXCEPTION;

BEGIN

  param_entradaPersona:= 'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', email: '||p_email;

  param_entradaUsuari:= 'NIF: '||p_NIF||
                        ', Pais: '||p_pais||
                        ', Operador: '||p_operador;
 
  param_entrada:=       'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', Email: '||p_email||
                         ', Pais: '||p_pais||
                         ', Operador: '||p_operador;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  ELSIF (p_operador IS NULL) THEN
      valor_nul:='operador';
      RAISE E_NUL;
  END IF;

-- inserim el usuari en la taula Persona si prviament no existeix; sin generem una exepci

   SELECT COUNT(*) INTO comptador FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0) THEN
      
       RSP:='OK';
   
      INSERT INTO PERSONA (NIF,telefon,email) VALUES (p_NIF,p_telefon,p_email);
  
   ELSE 

       RAISE E_PK1;

   END IF;

-- inserim el nou usuari si no existeix; sin generem una excepci

   SELECT COUNT(*) INTO comptador FROM USUARI U WHERE U.NIF=p_NIF;
 
   IF (comptador=0) THEN

      INSERT INTO USUARI(NIF, pais, operador) VALUES (p_NIF,p_pais, p_operador);
  
   ELSE
      RAISE E_PK;
  
   END IF;

-- inserci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:='ERROR: ja existeix aquest usuari en la taula USUARI';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
    WHEN E_PK1 THEN
      RSP:='ERROR: ja existeix aquest usuari en la taula PERSONA';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
      ROLLBACK;

      --violaci de la clau nica que s el telfon, ja que les altres claus niques estan controlades per E_PK i E_PK1
      IF (SQLCODE=-00001) THEN
          RSP:='ERROR: ja existeix un usuari o un desenvolupador amb el mateix telefon';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entradaPersona, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);

      ELSE
         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

      END IF;
END SP_ALTA_USUARI;
    
/

CREATE OR REPLACE PROCEDURE SP_BAIXA_USUARI (
    p_NIF      IN  PERSONA.NIF%TYPE,
    RSP        OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_BAIXA_USUARI
   Funci:                   Esborra un usuari a la taula USUARI i si no existeix en la taula DESENVOLUPADOR
                             l'esborra de la taula PERSONA
   Parmetre d'entrada:      - p_NIF: el NIF de l'usuari
                             

   Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(50);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
comptador2 INTEGER;
E_NUL  EXCEPTION;
E_EXISTS EXCEPTION;


BEGIN
 
  param_entrada:= 'NIF: '||p_NIF;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  
  END IF;
  
  -- esborrem l'usuari de la taula USUARI; sin existeix generem una excepci
   
   SELECT COUNT(*) INTO comptador FROM USUARI U WHERE U.NIF=p_NIF;
 
   IF (comptador>0) THEN

      DELETE FROM USUARI WHERE NIF=p_NIF;
  
   ELSE
     
      RAISE E_EXISTS;
  
   END IF;


-- si l'usuari no s desenvolupador alhora, l'esborrem de la taula PERSONA

   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR D WHERE D.NIF=p_NIF;
   SELECT COUNT(*) INTO comptador2 FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0 AND comptador2=1) THEN
   
      DELETE FROM PERSONA WHERE NIF=p_NIF;
  
   END IF;


-- esborrament correcte

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_EXISTS THEN
      RSP:='ERROR: aquest usuari no existeix en la taula USUARI';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

    
   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


      
END SP_BAIXA_USUARI;
    
/

CREATE OR REPLACE PROCEDURE SP_MODIFICACIO_USUARI (
    p_NIF      IN  PERSONA.NIF%TYPE,
    p_telefon  IN  PERSONA.telefon%TYPE,
    p_email    IN  PERSONA.email%TYPE,
    p_pais     IN  USUARI.pais%TYPE,
    p_operador IN  USUARI.operador%TYPE,
    RSP        OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_MODIFICACIO_USUARI
   Funci:                   Modifica les dades d'un usuari en la taula USUARI i PERSONA
                             - p_telefon: telefon de l'usuari 
                             - p_email: email de l'usuari
                             - p_pais: pais de registre de l'usuari 
                             - p_operador: operador telefnic de l'usuari

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/

param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
comptador2 INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;
E_UQ EXCEPTION;


BEGIN

 
 
  param_entrada:=       'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', email: '||p_email||
                         ', Pais: '||p_pais||
                         ', Operador: '||p_operador;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  ELSIF (p_operador IS NULL) THEN
      valor_nul:='operador';
      RAISE E_NUL;
  END IF;

--verifiquem que l'usuari existeixi en la taula PERSONA i en la taula USUARI sin generem una excepci

   SELECT COUNT(*) INTO comptador FROM USUARI  WHERE NIF=p_NIF;
   SELECT COUNT(*) INTO comptador2 FROM PERSONA WHERE NIF=p_NIF;
 
   IF (comptador=0 AND comptador2=0) THEN

      RAISE E_PK;
  
   END IF;
  

-- verifiquem que el nmero de telefon que es vol modificar no existeixi sin generem una excepci

   
   SELECT COUNT(*) INTO comptador FROM PERSONA  WHERE telefon=p_telefon;
   
   IF (comptador>0) THEN
   
       RAISE E_UQ;
   
   END IF;

-- modificaci dels valors
   
   UPDATE PERSONA SET telefon=p_telefon, email=p_email WHERE NIF=p_NIF;
   UPDATE USUARI SET pais=p_pais, operador=p_operador WHERE NIF=p_NIF;

-- modificaci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARI', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_PK THEN
      RSP:='ERROR: No existeix aquest usuari en la taula USUARI';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_UQ THEN
      RSP:='ERROR: el telfon que es vol modificar ja existeix en la base da dades';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

    
   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARI', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


     
END SP_MODIFICACIO_USUARI;
    
/

CREATE OR REPLACE PROCEDURE SP_ALTA_DESENVOLUPADOR (
    p_NIF           IN  PERSONA.NIF%TYPE,
    p_telefon       IN  PERSONA.telefon%TYPE,
    p_email         IN  PERSONA.email%TYPE,
    p_pais          IN  USUARI.pais%TYPE,
    p_dataNaixement IN  DESENVOLUPADOR.dataNaixement%TYPE,
    p_llocNaixement IN  DESENVOLUPADOR.llocNaixement%TYPE,
    RSP             OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_ALTA_DESENVOLUPADOR
   Funci:                   Afegeix un desenvolupador a la taula DESENVOLUPADOR i si no existeix en la taula PERSONA,
                             l'afegeix tamb
   Parmetre d'entrada:      - p_NIF: el NIF del desenvolupador
                             - p_telefon: telefon del desenvolupador
                             - p_email: email del desenvolupador
                             - p_pais: pais de registre del desenvolupador
                             - p_dataNaixement: data de naixement del desenvolupador
                             - P_llocNaixement: poblaci de naixement del desenvolupador

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entradaPersona  VARCHAR2(500);
param_entradaDesenvolupador   VARCHAR2(500);
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK_DESENVOLUPADOR EXCEPTION;

BEGIN

  param_entradaPersona:= 'NIF: '||p_NIF||
                         ', telefon: '||p_telefon||
                         ', email: '||p_email;

  param_entradaDesenvolupador:= 'NIF: '||p_NIF||
                        ', pais: '||p_pais||
                        ', dataNaixement: '||p_dataNaixement||
                        ', llocNaixement: '||p_llocNaixement;
 
  param_entrada:=        'NIF: '||p_NIF||
                         ', telefon: '||p_telefon||
                         ', email: '||p_email||
                         ', pais: '||p_pais||
                         ', dataNaixement: '||p_dataNaixement||
                         ', llocNaixement: '||p_llocNaixement;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  ELSIF (p_dataNaixement IS NULL) THEN
      valor_nul:='dataNaixement';
      RAISE E_NUL;
  END IF;

-- inserim el usuari en la taula Persona si prviament no existeix.

   SELECT COUNT(*) INTO comptador FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0) THEN
      
       INSERT INTO PERSONA (NIF,telefon,email) VALUES (p_NIF,p_telefon,p_email);
   
   END IF;


-- inserim el nou usuari sin existeix en la taula DESENVOLUPADOR; si existeix generem una excepci

   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR D WHERE D.NIF=p_NIF;
 
   IF (comptador=0) THEN

      INSERT INTO DESENVOLUPADOR(NIF, pais, dataNaixement, llocNaixement) VALUES (p_NIF,p_pais, p_dataNaixement,p_llocNaixement);
     
   ELSE
     
      RAISE E_PK_DESENVOLUPADOR;
  
   END IF;

-- inserci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entradaDesenvolupador, 'RSP: '||RSP);
   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
  
    WHEN E_PK_DESENVOLUPADOR THEN
      RSP:='ERROR: Ja existeix aquest desenvolupador en la taula DESENVOLUPADOR';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entradaDesenvolupador, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
      ROLLBACK;

      --violaci de la clau nica
      IF (SQLCODE=-00001) THEN
          RSP:='ERROR: ja existeix un usuari o desenvolupador amb el mateix telfon';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entradaPersona, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);

      -- qualsevol error no descrit anteriorment
      ELSE
         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

      END IF;
END SP_ALTA_DESENVOLUPADOR;
    
    
/

CREATE OR REPLACE PROCEDURE SP_BAIXA_DESENVOLUPADOR (
    p_NIF      IN  PERSONA.NIF%TYPE,
    RSP        OUT VARCHAR2
)
IS


/************************************************************************************************************************
   Nom:                      SP_BAIXA_DESENVOLUPADOR
   Funci:                   Esborra un desenvolupador a la taula DESENVOLUPADOR i si no existeix en la taula USUARI
                             l'esborra, tamb, de la taula PERSONA
   Parmetre d'entrada:      - p_NIF: el NIF del desenvolupador

   Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(50);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
comptador2 INTEGER;
E_NUL  EXCEPTION;
E_EXISTS EXCEPTION;


BEGIN
 
  param_entrada:= 'NIF: '||p_NIF;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  
  END IF;
  
  -- esborrem, si existeix, el desenvolupador de la taula DESENVOLUPADOR 
   
   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR  WHERE NIF=p_NIF;
 
   IF (comptador>0) THEN

      DELETE FROM DESENVOLUPADOR WHERE NIF=p_NIF;
  
   ELSE
     
      RAISE E_EXISTS;
  
   END IF;

-- si el desenvolupador no s usuari alhora, l'esborrem de la taula PERSONA

   SELECT COUNT(*) INTO comptador FROM USUARI U WHERE U.NIF=p_NIF;
   SELECT COUNT(*) INTO comptador2 FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0 AND comptador2=1) THEN
   
      DELETE FROM PERSONA WHERE NIF=p_NIF;
  
   END IF;



-- esborrament correcte

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_EXISTS THEN
      RSP:='ERROR: aquest desenvolupador no existeix en la taula DESENVOLUPADOR';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
    
   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

      
END SP_BAIXA_DESENVOLUPADOR;
    
/

CREATE OR REPLACE PROCEDURE SP_MODIFICACIO_DESENVOLUPADOR (
    p_NIF           IN  PERSONA.NIF%TYPE,
    p_telefon       IN  PERSONA.telefon%TYPE,
    p_email         IN  PERSONA.email%TYPE,
    p_llocNaixement IN  DESENVOLUPADOR.llocNaixement%TYPE,
    p_dataNaixement IN  DESENVOLUPADOR.dataNaixement%TYPE,
    p_pais          IN  DESENVOLUPADOR.pais%TYPE,
    RSP             OUT VARCHAR2
)
IS


/************************************************************************************************************************
   Nom:                      SP_MODIFICACIO_DESENVOLUPADOR
   Funci:                   Modifica les dades d'un desenvolupador de la taula DESENVOLUPADOR i de la taula PERSONA
   Parmetre d'entrada:      - p_NIF: el NIF del desenvolupador
                             - p_telefon: telefon del desenvolupador
                             - p_email: email del desenvolupador
                             - p_pais: pais de registre del desenvolupador
                             - p_dataNaixement: data de naixement del desenvolupador
                             - P_llocNaixement: poblaci de naixement del desenvolupador

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
comptador2 INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;
E_UQ EXCEPTION;


BEGIN

 
  param_entrada:=        'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', email: '||p_email||
                         ', LlocNaixement: '||p_llocNaixement||
                         ', DataNaixement: '||p_dataNaixement||
                         ', Pais: '||p_pais;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  END IF;

--verifiquem que el desenvolupador existeixi en la taula PERSONA i en la taula DESENVOLUPADOR

   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR  WHERE NIF=p_NIF;
   SELECT COUNT(*) INTO comptador2 FROM PERSONA WHERE NIF=p_NIF;
 
   IF (comptador=0 AND comptador2=0) THEN

      RAISE E_PK;
  
   END IF;
  

-- verifiquem que el nmero de telefon que es vol modificar no existeixi.

   
   SELECT COUNT(*) INTO comptador FROM PERSONA WHERE telefon=p_telefon;
   
   IF (comptador>0) THEN
   
      RAISE E_UQ;
   
   END IF;

-- modificaci dels valors
   
   UPDATE PERSONA SET telefon=p_telefon, email=p_email WHERE NIF=p_NIF;
   UPDATE DESENVOLUPADOR SET pais=p_pais, dataNaixement=p_dataNaixement,llocNaixement=p_llocNaixement WHERE NIF=p_NIF;

-- modificaci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:='ERROR: No existeix aquest desenvolupador en la taula DESENVOLUPADOR';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_UQ THEN
      RSP:='ERROR: el telfon que es vol modificar ja existeix en la base da dades';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
    
   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

     
END SP_MODIFICACIO_DESENVOLUPADOR;
    
/

CREATE OR REPLACE PROCEDURE SP_ALTA_APLICACIO (
    p_identificador IN  APLICACIO.identificador%TYPE,
    p_versio        IN  APLICACIO.versio%TYPE,
    p_dataPujada    IN  APLICACIO.dataPujada%TYPE,
    p_URL           IN  APLICACIO.URL%TYPE,
    p_activa        IN  APLICACIO.activa%TYPE,
    p_resolucio     IN APLICACIO.resolucio%TYPE,
    RSP             OUT VARCHAR2
)
IS


/************************************************************************************************************************
   Nom:                      SP_ALTA_APLICACIO
   Funci:                   Afegeix una aplicaci a la taula APLICACIO
   Parmetre d'entrada:      - p_identificador: identificador de l'aplicaci
                             - p_versio: versi de l'aplicaci
                             - p_dataPujada: data de pujada de l'aplicaci
                             - p_URL: enlla al video de demostraci de l'aplicaci
                             - p_resoluci: resoluci mnima de pantalla que ha de tenir el dispositiu on es descarregar

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador  INTEGER;
E_NUL      EXCEPTION;
E_DATE     EXCEPTION;


BEGIN

 
  param_entrada:=        'Identificador: '||p_identificador||
                         ', Versio: '||p_versio||
                         ', DataPujada: '||p_dataPujada||
                         ', URL: '||p_URL||
                         ', Activa: '||p_activa||
                         ', Resolucio: '||p_resolucio;



-- verificaci de parmetres no nuls

  IF (p_identificador IS NULL) THEN
      valor_nul:='identificador';
      RAISE E_NUL;
  ELSIF (p_versio IS NULL) THEN
      valor_nul:='versio';
      RAISE E_NUL;
  ELSIF (p_dataPujada IS NULL) THEN
      valor_nul:='dataPujada';
      RAISE E_NUL;
  ELSIF (p_activa IS NULL) THEN
      valor_nul:='activa';
      RAISE E_NUL;
  ELSIF (p_resolucio IS NULL) THEN
      valor_nul:='resolucio';
      RAISE E_NUL;
  END IF;

-- verifiquem que la data de descarrega no sigui major que l'actual
    
    IF (to_date(p_dataPujada)>to_date(sysdate,'DD/MM/YY')) THEN
       
       RAISE E_DATE;
  
    END IF;



-- inserim l'aplicaci
   
   
      INSERT INTO APLICACIO(identificador,versio,dataPujada,URL,activa,resolucio) VALUES (p_identificador,p_versio,p_dataPujada,p_URL,p_activa,p_resolucio);
  

-- inserci correcta

     RSP:='OK';
     SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_DATE THEN
         RSP:= 'ERROR: La data de pujada no pot ser major a l-actual';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
  
   WHEN OTHERS THEN
      ROLLBACK;

      --es comproba que les entrades amb clusules check siguin correctes
      IF (SQLCODE=-02290) THEN
          RSP:='ERROR: el valor activa ha de ser SI  NO';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);
    
      ELSIF(SQLCODE=-00001) THEN
          RSP:='ERROR: la aplicaci ja existeix en la taula APLICACIO';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);
          
      ELSE
         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
         
      END IF;

END SP_ALTA_APLICACIO;
    
/

CREATE OR REPLACE PROCEDURE SP_BAIXA_APLICACIO (
    p_ident       IN  APLICACIO.identificador%TYPE,
    RSP           OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_BAIXA_APLICACIO
   Funci:                   Esborra una aplicaci de la taula APLICACIO
   Parmetre d'entrada:      - p_identificador: identificador de l'aplicaci

   Parmetre de sortida:     RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;

BEGIN

 
  param_entrada:=        'Identificador: '||p_ident;
                         

-- verificaci de parmetres no nuls

  IF (p_ident IS NULL) THEN
      valor_nul:='ident';
      RAISE E_NUL;
  
  END IF;

-- comprovaci de que l'aplicaci existeixi
   
   
      SELECT COUNT(*) INTO comptador FROM APLICACIO WHERE identificador=p_ident;
      
      IF(comptador>0) THEN

        DELETE FROM APLICACIO WHERE identificador=p_ident;
  
      ELSE
        
         RAISE E_PK;
   
     END IF; 
  -- operacio correcta

     RSP:='OK';
     SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);


   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:= 'ERROR: L-aplicaci no existeix en la taula APLICACIO';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
      
END SP_BAIXA_APLICACIO;
    
/

CREATE OR REPLACE PROCEDURE SP_MODIFICACIO_APLICACIO (
    p_ident       IN  APLICACIO.identificador%TYPE,
    p_versio      IN  APLICACIO.versio%TYPE,
    p_dataPujada  IN  APLICACIO.dataPujada%TYPE,
    p_URL         IN  APLICACIO.URL%TYPE,
    p_activa      IN  APLICACIO.activa%TYPE,
    p_resolucio   IN APLICACIO.resolucio%TYPE,
    RSP           OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_MOFICACIO_APLICACIO
   Funci:                   Modifica les dades d'una aplicaci de la taula APLICACI
   Parmetre d'entrada:      - p_identificador: identificador de l'aplicaci
                             - p_versio: versi de l'aplicaci
                             - p_dataPujada: data de pujada de l'aplicaci
                             - p_URL: enlla al video de demostraci de l'aplicaci
                             - p_resoluci: resoluci mnima de pantalla que ha de tenir el dispositiu on es descarregar

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;

BEGIN

 
  param_entrada:=        'Identificador: '||p_ident||
                         ' Versio: '||p_versio||
                         ', DataPujada: '||p_dataPujada||
                         ', URL: '||p_URL||
                         ', Activa: '||p_activa||
                         ', Resolucio: '||p_resolucio;



-- verificaci de parmetres no nuls

  IF (p_ident IS NULL) THEN
      valor_nul:='identificador';
      RAISE E_NUL;
  ELSIF(p_versio IS NULL) THEN
      valor_nul:='versio';
  ELSIF (p_dataPujada IS NULL) THEN
      valor_nul:='dataPujada';
      RAISE E_NUL;
  ELSIF (p_activa IS NULL) THEN
      valor_nul:='activa';
      RAISE E_NUL;
  ELSIF (p_resolucio IS NULL) THEN
      valor_nul:='resolucio';
      RAISE E_NUL;
  END IF;

-- comprovaci de que l'aplicaci existeixi
   
   
      SELECT COUNT(*) INTO comptador FROM APLICACIO WHERE identificador=p_ident;
      
      IF(comptador>0) THEN

         UPDATE APLICACIO SET versio=p_versio,dataPujada=p_dataPujada,URL=p_URL,activa=UPPER(p_activa),resolucio=p_resolucio
         WHERE identificador=p_ident;
  
      ELSE
        
         RAISE E_PK;
   
      END IF;

  -- inserci correcta

     RSP:='OK';
     SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada, 'RSP: '||RSP);


   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:= 'ERROR: l-aplicaci no existeix en la taula APLICACIO';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada,'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      

   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
      
END SP_MODIFICACIO_APLICACIO;
    
/

CREATE OR REPLACE PROCEDURE SP_DESCARREGA (
      p_usuari     IN   DESCARREGA.usuari%TYPE,
      p_aplicacio  IN   DESCARREGA.aplicacio%TYPE,
      p_data       IN   DESCARREGA.dataDesc%TYPE,
      p_pagament   IN   DESCARREGA.pagament%TYPE,
      p_operador   IN   DESCARREGA.operador%TYPE,
      p_pais       IN   DESCARREGA.pais%TYPE,
      p_dispositiu IN   DESCARREGA.dispositiu%TYPE,
      RSP          OUT  VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_DESCARREGA
   Funci:                   Emmagatzema una descrrega d'una aplicaci en el sistema
   Parmetre d'entrada:      - p_usuari: usuari que realitza la descrrega
                             - p_aplicacio: identificador de l'aplicaci
                             - p_data: data de descrrega de l'aplicaci
                             - p_pagament: mode de pagament 
                             - p_operador: operador de telefonia en el moment de la descrrega
                             _ p_dispositiu: identificador del dispositiu en qu s'ha fet la descrrega (IMEI).

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada       VARCHAR2(500);
error_msg           VARCHAR2(200);
valor_nul           VARCHAR2(100);
comptador           INTEGER;
resolucioAplicacio  NUMBER;
resolucioDispositiu NUMBER;
valor               VARCHAR2(100);
E_NUL               EXCEPTION;
E_FK                EXCEPTION;
E_RESOL             EXCEPTION;
E_DATE              EXCEPTION;




BEGIN
  param_entrada:= 'Usuari: '||p_usuari||
                  ', Aplicacio: '||p_aplicacio||
                  ', Data: '||p_data||
                  ', Pagament: '||p_pagament||
                  ', Operador: '||p_operador||
                  ', Pais: '||p_pais||
                  ', Dispositiu: '||p_dispositiu;

-- verifiquem que no hi hagi valors nuls

   IF (p_usuari IS NULL) THEN
       valor_nul:='usuari';
       RAISE E_NUL;
   ELSIF (p_aplicacio IS NULL) THEN
       valor_nul:='aplicacio';
       RAISE E_NUL;
   ELSIF (p_data IS NULL) THEN
       valor_nul:='data';
       RAISE E_NUL;
   ELSIF (p_pagament IS NULL) THEN
       valor_nul:='pagament';
       RAISE E_NUL;
   ELSIF (p_operador IS NULL) THEN
       valor_nul:='operador';
       RAISE E_NUL;
   ELSIF (p_pais IS NULL) THEN
       valor_nul:='pais';
       RAISE E_NUL;
   ELSIF (p_dispositiu IS NULL) THEN
       valor_nul:='dispositiu';
       RAISE E_NUL;
       
   END IF;

-- verifiquem que l'usuari existeix prviament en la taula USUARI

    SELECT COUNT(*) INTO comptador FROM USUARI WHERE NIF=p_usuari;
    
    IF(comptador=0) THEN
      valor:='usuari';
      RAISE E_FK;
    END IF;

-- verifiquem que l'aplicaci existeix prviament en la taula APLICACIO

    SELECT COUNT(*) INTO comptador FROM APLICACIO WHERE identificador=p_aplicacio;
    
    IF(comptador=0) THEN
      valor:='aplicacio';
      RAISE E_FK;
    END IF;

-- verifiquem que el dispositiu existeix prviament en la taula DISPOSITIU

    SELECT COUNT(*) INTO comptador FROM DISPOSITIU WHERE IMEI=p_dispositiu;
    
    IF(comptador=0) THEN
      valor:='dispositiu';
      RAISE E_FK;
    END IF;

-- verifiquem que la resoluci de pantalla sigui major que la donada per l'aplicaci

    SELECT resolucio INTO resolucioAplicacio FROM APLICACIO WHERE identificador=p_aplicacio;
    SELECT resolucio INTO resolucioDispositiu FROM DISPOSITIU WHERE IMEI=p_dispositiu;

    IF (resolucioAplicacio > resolucioDispositiu ) THEN
       RAISE E_RESOL;
    END IF;

-- verifiquem que la data de descarrega no sigui major que l'actual
    
    IF (to_date(p_data)>to_date(sysdate,'DD/MM/YY')) THEN
       RAISE E_DATE;
    END IF;


-- insertem la nova descrrega en la taula DESCARREGA

    INSERT INTO DESCARREGA(usuari,aplicacio,dataDesc,pagament,operador,pais,dispositiu) VALUES (p_usuari,p_aplicacio,p_data,p_pagament,p_operador,p_pais,p_dispositiu);

-- crida als procediments estadstics de manera que estiguin actualitzats en cada descrrega de la plataforma


       SP_ALTA_EST1;
       SP_ALTA_EST2;
       SP_ALTA_EST3(EXTRACT(YEAR FROM P_DATA));
       SP_ALTA_EST4(EXTRACT(YEAR FROM P_DATA));
       SP_ALTA_EST5(EXTRACT(YEAR FROM P_DATA));
       SP_ALTA_EST6(EXTRACT(YEAR FROM P_DATA),p_pais);
       SP_ALTA_EST7(EXTRACT(YEAR FROM P_DATA),p_pais);
       SP_ALTA_EST8(EXTRACT(YEAR FROM P_DATA),p_pais);


--  inserci correcta
    RSP:='OK';
    SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);



COMMIT;


EXCEPTION

   WHEN E_NUL THEN
        RSP:= 'ERROR: el valor ' || valor_nul|| ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
    WHEN E_FK THEN  
        RSP:= 'ERROR: el ' || valor || ' no existeix en la taula '||UPPER(valor);       
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
    

    WHEN E_RESOL THEN
         RSP:= 'ERROR: La resoluci del dispositiu s menor que la necessria per l-aplicaci';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

    WHEN E_DATE THEN
         RSP:= 'ERROR: La data de descrrega no pot ser major a l-actual';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

        
   WHEN OTHERS THEN
        ROLLBACK;
       
        
          error_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || error_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);
        
              
END SP_DESCARREGA;
    
/