
/********************************* ESTRUCTURA DE DADES CONSULTA *****************************************************************/


CREATE OR REPLACE TYPE LLISTA_DESENVOLUPADORS AS OBJECT (
       NIF             CHAR(9),
       llocNaixement   VARCHAR2(50),
       dataNaixement   DATE,
       pais            VARCHAR2(10),
       nomEmpresa      VARCHAR2(50),
       adrea          VARCHAR2(50),
       nombre          NUMBER
);
/

CREATE OR REPLACE TYPE TABLE1 AS TABLE OF LLISTA_DESENVOLUPADORS;
/


CREATE OR REPLACE FUNCTION DESENV_NOMBRE_APLICACIONS (
       p_pais      IN    DESENVOLUPADOR.pais%TYPE
)
RETURN TABLE1

/**********************************************************************************************************************************
   Nom:                 DESENV_NOMBRE_APLICACIONS
   Propsit:            Consulta que retorna tots els desenvolupadors i les seves dades principals per al pas donat 
                   
   Parmetres entrada:  -	p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents per cada fila, per al pas donat:
                                >> NIF
                                >> lloc de naixement
                                >> data de naixement
                                >> pas de registre
                                >> nom de l'organitzaci on treballa.
                                >> adrea de les oficines centrals de l'organitzaci
                                >> nombre d'aplicacions diferents publicades

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

--variables

NIF             CHAR(9);
llocNaixement   VARCHAR2(50);
dataNaixement   DATE;
pais            VARCHAR2(10);
nomEmpresa      VARCHAR2(50);
adrea          VARCHAR2(50);
nombre          NUMBER;
param_entrada   VARCHAR2(10);
i               INTEGER:=1;

RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
resultat        TABLE1:=TABLE1();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Pais: '||p_pais;

-- es verifica que no hi hagi parmetres d'entrada nuls
  IF(p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;

  END IF;


-- seleccionem en el cursor totes les dades del desenvolupador i l'empresa aix com el total d'aplicacions publicades

   OPEN CUR FOR SELECT d.NIF,d.llocNaixement,d.dataNaixement,d.pais, e.nom, e.adrea, count(cr.aplicacio) AS Total 
   FROM  APLICACIO ap, CREACIO cr, DESENVOLUPADOR d, TREBALLA t, EMPRESA e          
   WHERE  ap.identificador=cr.aplicacio AND cr.desenvolupador=d.NIF AND    d.NIF=t.desenvolupador
   AND t.empresa=e.codi AND d.pais=p_pais 
   GROUP BY d.NIF,d.llocNaixement,d.dataNaixement,d.pais,e.nom,e.adrea;

   LOOP 
     FETCH CUR INTO NIF,llocNaixement,dataNaixement,pais,nomEmpresa,adrea,nombre;
     EXIT WHEN CUR%NOTFOUND;

-- anem emmagatzeman els resultats de cada iteraci en la variable resultat

     resultat.extend;
     resultat(i):=LLISTA_DESENVOLUPADORS( NIF,llocNaixement,dataNaixement,pais,nomEmpresa,adrea,nombre);
     i:=i+1;
    END LOOP;
    CLOSE CUR;

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'DESENV_NOMBRE_APLICACIONS', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'DESENV_NOMBRE_APLICACIONS', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;

   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'DESENV_NOMBRE_APLICACIONS', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;        

END DESENV_NOMBRE_APLICACIONS;

/
/******************************************* ESTRUCTURA DE DADES CONSULTA 2 ******************************************************/


CREATE OR REPLACE TYPE LLISTA_APLICACIONS AS OBJECT (
       identificador       NUMBER,
       versio              VARCHAR2(10),
       dataPujada          DATE,
       URL                 VARCHAR2(30),
       activa              CHAR(2),
       resolucio           NUMBER,
       totalDescarregues   NUMBER
);
/

CREATE OR REPLACE TYPE TABLE2 AS TABLE OF LLISTA_APLICACIONS;
/


CREATE OR REPLACE FUNCTION APLICACIONS_ACTIVES 

RETURN TABLE2

/**********************************************************************************************************************************
   Nom:                 APLICACIONS_ACTIVES
   Propsit:            Consulta que retorna totes les aplicacions que estan actives amb les seves dades principals ordenades pel
                        nombre total de descrregues que han tingut a nivell mundial. 
                   
   Parmetres entrada:  -	Sense parmetres
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat amb les segents dades per cada fila:
                                >> Identificador numric de l'aplicaci
                                >> Versi de l'aplicaci
                                >> Data de pujada a la plataforma
                                >> URL d'enlla al vdeo de demostraci
                                >> Si est activa. Haur de retornar 'SI' per a totes les aplicacions
                                >> Resoluci mnima de pantalla que ha de tenir el dispositiu
                                >> Nombre de descrregues total que ha tingut l'aplicaci a nivell mundial

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

identificador   NUMBER;
versio          VARCHAR2(30);
dataPujada      DATE;
URL             VARCHAR2(100);
activa          CHAR(2);
resolucio       NUMBER;
total           NUMBER;
i               INTEGER:=1;

param_entrada   VARCHAR2(20);
RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
resultat        TABLE2:=TABLE2();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Sense parmetres';


 --selecionem un cursor amb les dades de les aplicacions tot fent un left outer join per seleccionar aquelles que no han tingut cap descrrega

   OPEN CUR FOR SELECT ap.identificador,ap.versio,ap.dataPujada,ap.URL,ap.activa,ap.resolucio,COUNT(de.aplicacio) AS total FROM APLICACIO ap
   LEFT OUTER JOIN DESCARREGA de ON ap.identificador=de.aplicacio WHERE  ap.activa='SI'
   GROUP BY ap.identificador,ap.versio,ap.dataPujada,ap.URL,ap.activa,ap.resolucio 
   ORDER BY total DESC;   

   LOOP
   FETCH CUR INTO identificador,versio,dataPujada,URL,activa,resolucio,total;
   EXIT WHEN CUR%NOTFOUND;
   resultat.extend;
   resultat(i):=LLISTA_APLICACIONS(identificador,versio,dataPujada,URL,activa,resolucio,total);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'APLICACIONS_ACTIVES', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'APLICACIONS_ACTIVES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END APLICACIONS_ACTIVES;

/

/*************************************************** ESTRUCTURA DE DADES CONSULTA 3 ***********************************/

CREATE OR REPLACE TYPE LLISTA_PAISOS AS OBJECT (
       pais            CHAR(2),
       aplicacio       NUMBER,
       total           NUMBER
       
);
/
CREATE OR REPLACE TYPE TABLE3 AS TABLE OF LLISTA_PAISOS;
/
CREATE OR REPLACE FUNCTION PAISOS_NOMBRE_DESCARREGUES (
      
       p_aplicacio  IN  NUMBER,
       p_any        IN  NUMBER
)
RETURN TABLE3

/**********************************************************************************************************************************
   Nom:                 PAISOS_NOMBRE_DESCARREGUES
   Propsit:            Consulta que retorna donada una aplicaci i un any concret retorna un llista amb tots els pasos on s'ha 
                        descarregat aquell any, aix com el nombre de descrregues que ha tingut en aquell pas.
                   
   Parmetres entrada:  -  p_aplicacio: identificador numric de l'aplicaci
                        -  p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> Pas en format ISO 3166-1-alfa-2
                                >> Identificador numric de l'aplicaci
                                >> Nombre total de descrregues sofertes en el pas i l'any donats.

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS


pais            CHAR(2);
param_entrada   VARCHAR2(50);
total           NUMBER;
i               INTEGER:=1;


RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
E_DATE          EXCEPTION;
resultat        TABLE3:=TABLE3();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Aplicacio: '||p_aplicacio||
                    ' Any: '||p_any;

-- verifiquem que no hi hagi parmetres d'entrada nuls
  IF(p_aplicacio IS NULL) THEN
     valor_nul:='aplicacio';
     RAISE E_NUL;
  ELSIF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;

  END IF;
 
-- verifiquem que l'any passat com parmetre no sigui major que l'actual 

  IF(p_any > to_number(to_char(sysdate,'YYYY'))) THEN
     RAISE E_DATE;
  
  END IF;

-- seleccionem un cursor amb el pas de descrrega i el nombre total d'usuaris que han descarregat l'aplicaci (per tant el nombre de descrregues)
-- `per al pas i aplicaci passats com a parmetre

   OPEN CUR FOR SELECT pais, COUNT(usuari) AS totalDesc FROM DESCARREGA 
   WHERE  EXTRACT(YEAR FROM dataDesc)=p_any and aplicacio=p_aplicacio
   GROUP BY pais ORDER BY totalDesc DESC;   
   
   LOOP
   FETCH CUR INTO pais,total;
   EXIT WHEN CUR%NOTFOUND;

-- emmagatzemem cada resultat de la iteraci de la consulta en la variable resultat

   resultat.extend;
   resultat(i):=LLISTA_PAISOS(pais,p_aplicacio,total);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
   WHEN E_DATE THEN
        RSP:= 'ERROR: l-any passat com a parmetre no pot ser ms gran que l-actual'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END PAISOS_NOMBRE_DESCARREGUES;

/

/************************************************** ESTRUCTURES DE DADES CONSULTA 4 ******************************************/
CREATE OR REPLACE TYPE INFO_USUARI AS OBJECT (
       NIF             CHAR(9),
       telefon         NUMBER,
       email           VARCHAR2(50),
       operador        VARCHAR2(30),
       identificador   NUMBER,
       versio          VARCHAR2(20),
       preu            NUMBER,
       descripcio      VARCHAR2(200)
       
);
/
CREATE OR REPLACE TYPE TABLE4 AS TABLE OF INFO_USUARI;
/
CREATE OR REPLACE FUNCTION ACTIVITAT_USUARI (
       p_telefon     IN    NUMBER
)
RETURN TABLE4

/**********************************************************************************************************************************
   Nom:                 ACTIVITAT_USUARI
   Propsit:            Consulta que donat un usuari final identificat pel seu nmero de telfon retorna tota la seva activitat de descrregues
                        en la plataforma
                   
   Parmetres entrada:  -	p_telefon: telfon de l'usuari
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> NIF d l'usuari
                                >> Telefon de l'usuari
                                >> E-mail de l'usuari
                                >> Operador de l'usuari en el moment de la descrrega
                                >> Identificador numric de l'aplicaci descarregada
                                >> Versi de l'aplicaci descarregada
                                >> Preu que va pagar per l'aplicaci
                                >> Descripci de l'aplicaci

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

NIF             CHAR(9);
telefon         NUMBER;
email           VARCHAR2(50);
operador        VARCHAR2(30);
identificador   NUMBER;
versio          VARCHAR2(20);
preu            NUMBER;
descripcio      VARCHAR2(200);

param_entrada   VARCHAR2(20);
i               INTEGER:=1;
tlf             INTEGER;


RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
E_TELF          EXCEPTION;
resultat        TABLE4:=TABLE4();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Telefon: '||p_telefon;

-- verifiquem que no hi hagi parmetres d'entrada nuls
  IF(p_telefon IS NULL) THEN
     valor_nul:='telefon';
     RAISE E_NUL;

  END IF;

--verifiquem que el telefon existeixi en la base de dades
   
   SELECT COUNT(telefon) INTO tlf FROM PERSONA WHERE telefon=p_telefon;
   
   IF (tlf=0) THEN
   RAISE E_TELF;
 
   END IF;
   
   OPEN CUR FOR SELECT p.NIF,p.telefon,p.email,de.operador,ap.identificador,ap.versio,r.preu,r.descripcio
   FROM PERSONA p, USUARI u, DESCARREGA de, APLICACIO ap, REGISTRE r
   WHERE p.NIF=u.NIF AND u.NIF=de.usuari AND de.aplicacio=ap.identificador AND ap.identificador=r.aplicacio AND p.telefon=p_telefon AND de.pais=r.pais
   GROUP BY p.nif,p.telefon,p.email,de.operador,ap.identificador,ap.versio,r.preu,r.descripcio;
   
   LOOP
   FETCH CUR INTO NIF,telefon,email,operador,identificador,versio,preu,descripcio;
   EXIT WHEN CUR%NOTFOUND;
   resultat.extend;
   resultat(i):=INFO_USUARI(NIF,telefon,email,operador,identificador,versio,preu,descripcio);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
   WHEN E_TELF THEN
        RSP:= 'ERROR: el telfon introdut no existeix en la base de dades'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END ACTIVITAT_USUARI;

/

/*********************************************** ESTRUCTURA DE DADES CONSULTA 5 **************************************************/

CREATE OR REPLACE TYPE LLISTA_USUARIS AS OBJECT (
       NIF             CHAR(9),
       telefon         NUMBER,
       email           VARCHAR2(50),
       total           NUMBER
       
);
/
CREATE OR REPLACE TYPE TABLE5 AS TABLE OF LLISTA_USUARIS;
/
CREATE OR REPLACE FUNCTION MAX_DESPESA (
       p_any     IN    NUMBER
)
RETURN TABLE5

/**********************************************************************************************************************************
   Nom:                 MAX_DESPESA
   Propsit:            Consulta que retorna per l'any passat per parmetre els 20 (com a mxim nombre) d'usuaris que ms diners s'han
                        gastat en la plataforma ordenats de ms a menys. 
                   
   Parmetres entrada:  -	p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> NIF de l'usuari
                                >> Telfon de l'usuari
                                >> E-mail de l'usuari
                                >> Total de diners gastats

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

NIF             CHAR(9);
telefon         NUMBER;
email           VARCHAR2(50);
pais            CHAR(2);
param_entrada   VARCHAR2(10);
total           NUMBER;
i               INTEGER:=1;


RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
E_DATE          EXCEPTION;
resultat        TABLE5:=TABLE5();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Any: '||p_any;

-- verifiquem que no hi hagi parmetres d'entrada nuls
  IF(p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;

  END IF;
 
-- verifiquem que l'any passat com parmetre no sigui major que l'actual 
  IF(p_any > to_number(to_char(sysdate,'YYYY'))) THEN
     RAISE E_DATE;
  
  END IF;

-- seleccionem un cursor amb les dades dels usuaris que ms han gastat ordenats per gasto descendent

   OPEN CUR FOR SELECT u.nif,p.telefon,p.email,SUM(r.preu) AS total FROM USUARI u, PERSONA p, DESCARREGA d, APLICACIO aP, REGISTRE r
   WHERE p.nif=u.nif AND u.nif=d.usuari AND d.aplicacio=aP.identificador AND aP.identificador=r.aplicacio AND d.pais=r.pais AND
   p_any=EXTRACT(year from d.dataDesc) GROUP BY u.nif,p.telefon,p.email ORDER BY total DESC;
   
   LOOP
   FETCH CUR INTO NIF,telefon,email,total;

-- obliguem que hi hagi un mxim de 20 files i anem emmagatzemant les dades en la variable resultat

   EXIT WHEN CUR%NOTFOUND OR CUR%ROWCOUNT>20;
   resultat.extend;
   resultat(i):=LLista_usuaris(NIF,telefon,email,total);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
   WHEN E_DATE THEN
        RSP:= 'ERROR: l-any passat com a parmetre no pot ser ms gran que l-actual'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END MAX_DESPESA;

/

/******************************************** ESTRUCTURA DE DADES CONSULTA 6 **********************************************/

CREATE OR REPLACE TYPE LLISTA_SO AS OBJECT (
       ao                  NUMBER,
       sistema              VARCHAR2(30),
       total                NUMBER
);

/

CREATE OR REPLACE TYPE TABLE6 AS TABLE OF LLISTA_SO;

/

CREATE OR REPLACE FUNCTION SO_MES_UTILITZATS(
         p_any       IN   NUMBER
)

RETURN TABLE6



/**********************************************************************************************************************************
   Nom:                 SO_MES_UTILITZATS
   Propsit:            Consulta que retorna per l'any passat per parmetre un llistat amb els sistemes operatius de telfons intelligents
                        que ms descrregues han fet junt amb aquest nombre (ordenats pel nombre total de descrregues)
   Parmetres entrada:  -	p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> Any de consulta
                                >> Nom del sistema operatiu
                                >> Total de descrregues que ha fet aquest sistema operatiu en la plataforma

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/


IS

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(500);
RSP             VARCHAR2(500);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
resultat        TABLE6:=TABLE6();
i               INTEGER:=1;

-- seleccionem un cursor amb l'any, el sistema operatiu i el nombre total de descrregues que ha fet en la plataforma en aquell any
CURSOR CUR IS SELECT EXTRACT(year from de.dataDesc),d.sistemaoperatiu,COUNT( d.sistemaoperatiu ) AS SO FROM DESCARREGA de,Dispositiu d
              WHERE   EXTRACT(year from de.datadesc)=p_any AND de.dispositiu=d.Imei
              GROUP BY d.sistemaoperatiu,EXTRACT(year from de.dataDesc) ORDER BY SO DESC;

sistema        VARCHAR2(40);
anyo           NUMBER;
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any;
                    

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  END IF;

--obrim cursor i iterem

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,sistema,total;
         EXIT WHEN CUR%NOTFOUND;

    -- anem emmagatzemant les dades en la variable resultat

          resultat.extend;
          resultat(i):=LLista_SO(anyo,sistema,total);
          i:=i+1;
       
       END LOOP;
     CLOSE CUR;   

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SO_MES_UTILITZATS', param_entrada, 'RSP: '||RSP);   

RETURN resultat;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SO_MES_UTILITZATS', param_entrada, 'RSP: '||RSP);   
         DBMS_OUTPUT.PUT_LINE(RSP);
         RETURN NULL;

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SO_MES_UTILITZATS', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
         RETURN NULL;
END SO_MES_UTILITZATS;