/********************************* ESTRUCTURA DE DADES CONSULTA *****************************************************************/


CREATE OR REPLACE TYPE LLISTA_DESENVOLUPADORS AS OBJECT (
       NIF             CHAR(9),
       llocNaixement   VARCHAR2(50),
       dataNaixement   DATE,
       pais            VARCHAR2(10),
       nomEmpresa      VARCHAR2(50),
       adrea          VARCHAR2(50),
       nombre          NUMBER
);
/

CREATE OR REPLACE TYPE TABLE1 AS TABLE OF LLISTA_DESENVOLUPADORS;
/


CREATE OR REPLACE FUNCTION DESENV_NOMBRE_APLICACIONS (
       p_pais      IN    DESENVOLUPADOR.pais%TYPE
)
RETURN TABLE1

/**********************************************************************************************************************************
   Nom:                 DESENV_NOMBRE_APLICACIONS
   Propsit:            Consulta que retorna tots els desenvolupadors i les seves dades principals per al pas donat 
                   
   Parmetres entrada:  -	p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents per cada fila, per al pas donat:
                                >> NIF
                                >> lloc de naixement
                                >> data de naixement
                                >> pas de registre
                                >> nom de l'organitzaci on treballa.
                                >> adrea de les oficines centrals de l'organitzaci
                                >> nombre d'aplicacions diferents publicades

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

--variables

NIF             CHAR(9);
llocNaixement   VARCHAR2(50);
dataNaixement   DATE;
pais            VARCHAR2(10);
nomEmpresa      VARCHAR2(50);
adrea          VARCHAR2(50);
nombre          NUMBER;
param_entrada   VARCHAR2(10);
i               INTEGER:=1;

RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
resultat        TABLE1:=TABLE1();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Pais: '||p_pais;

-- es verifica que no hi hagi parmetres d'entrada nuls
  IF(p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;

  END IF;


-- seleccionem en el cursor totes les dades del desenvolupador i l'empresa aix com el total d'aplicacions publicades

   OPEN CUR FOR SELECT d.NIF,d.llocNaixement,d.dataNaixement,d.pais, e.nom, e.adrea, count(cr.aplicacio) AS Total 
   FROM  APLICACIO ap, CREACIO cr, DESENVOLUPADOR d, TREBALLA t, EMPRESA e          
   WHERE  ap.identificador=cr.aplicacio AND cr.desenvolupador=d.NIF AND    d.NIF=t.desenvolupador
   AND t.empresa=e.codi AND d.pais=p_pais 
   GROUP BY d.NIF,d.llocNaixement,d.dataNaixement,d.pais,e.nom,e.adrea;

   LOOP 
     FETCH CUR INTO NIF,llocNaixement,dataNaixement,pais,nomEmpresa,adrea,nombre;
     EXIT WHEN CUR%NOTFOUND;

-- anem emmagatzeman els resultats de cada iteraci en la variable resultat

     resultat.extend;
     resultat(i):=LLISTA_DESENVOLUPADORS( NIF,llocNaixement,dataNaixement,pais,nomEmpresa,adrea,nombre);
     i:=i+1;
    END LOOP;
    CLOSE CUR;

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'DESENV_NOMBRE_APLICACIONS', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'DESENV_NOMBRE_APLICACIONS', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;

   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'DESENV_NOMBRE_APLICACIONS', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;        

END DESENV_NOMBRE_APLICACIONS;