/******************************************* ESTRUCTURA DE DADES CONSULTA 2 ******************************************************/


CREATE OR REPLACE TYPE LLISTA_APLICACIONS AS OBJECT (
       identificador       NUMBER,
       versio              VARCHAR2(10),
       dataPujada          DATE,
       URL                 VARCHAR2(30),
       activa              CHAR(2),
       resolucio           NUMBER,
       totalDescarregues   NUMBER
);
/

CREATE OR REPLACE TYPE TABLE2 AS TABLE OF LLISTA_APLICACIONS;
/


CREATE OR REPLACE FUNCTION APLICACIONS_ACTIVES 

RETURN TABLE2

/**********************************************************************************************************************************
   Nom:                 APLICACIONS_ACTIVES
   Propsit:            Consulta que retorna totes les aplicacions que estan actives amb les seves dades principals ordenades pel
                        nombre total de descrregues que han tingut a nivell mundial. 
                   
   Parmetres entrada:  -	Sense parmetres
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat amb les segents dades per cada fila:
                                >> Identificador numric de l'aplicaci
                                >> Versi de l'aplicaci
                                >> Data de pujada a la plataforma
                                >> URL d'enlla al vdeo de demostraci
                                >> Si est activa. Haur de retornar 'SI' per a totes les aplicacions
                                >> Resoluci mnima de pantalla que ha de tenir el dispositiu
                                >> Nombre de descrregues total que ha tingut l'aplicaci a nivell mundial

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

identificador   NUMBER;
versio          VARCHAR2(30);
dataPujada      DATE;
URL             VARCHAR2(100);
activa          CHAR(2);
resolucio       NUMBER;
total           NUMBER;
i               INTEGER:=1;

param_entrada   VARCHAR2(20);
RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
resultat        TABLE2:=TABLE2();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Sense parmetres';


 --selecionem un cursor amb les dades de les aplicacions tot fent un left outer join per seleccionar aquelles que no han tingut cap descrrega

   OPEN CUR FOR SELECT ap.identificador,ap.versio,ap.dataPujada,ap.URL,ap.activa,ap.resolucio,COUNT(de.aplicacio) AS total FROM APLICACIO ap
   LEFT OUTER JOIN DESCARREGA de ON ap.identificador=de.aplicacio WHERE  ap.activa='SI'
   GROUP BY ap.identificador,ap.versio,ap.dataPujada,ap.URL,ap.activa,ap.resolucio 
   ORDER BY total DESC;   

   LOOP
   FETCH CUR INTO identificador,versio,dataPujada,URL,activa,resolucio,total;
   EXIT WHEN CUR%NOTFOUND;
   resultat.extend;
   resultat(i):=LLISTA_APLICACIONS(identificador,versio,dataPujada,URL,activa,resolucio,total);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'APLICACIONS_ACTIVES', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'APLICACIONS_ACTIVES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END APLICACIONS_ACTIVES;