/*************************************************** ESTRUCTURA DE DADES CONSULTA 3 ***********************************/

CREATE OR REPLACE TYPE LLISTA_PAISOS AS OBJECT (
       pais            CHAR(2),
       aplicacio       NUMBER,
       total           NUMBER
       
);
/
CREATE OR REPLACE TYPE TABLE3 AS TABLE OF LLISTA_PAISOS;
/
CREATE OR REPLACE FUNCTION PAISOS_NOMBRE_DESCARREGUES (
      
       p_aplicacio  IN  NUMBER,
       p_any        IN  NUMBER
)
RETURN TABLE3

/**********************************************************************************************************************************
   Nom:                 PAISOS_NOMBRE_DESCARREGUES
   Propsit:            Consulta que retorna donada una aplicaci i un any concret retorna un llista amb tots els pasos on s'ha 
                        descarregat aquell any, aix com el nombre de descrregues que ha tingut en aquell pas.
                   
   Parmetres entrada:  -  p_aplicacio: identificador numric de l'aplicaci
                        -  p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> Pas en format ISO 3166-1-alfa-2
                                >> Identificador numric de l'aplicaci
                                >> Nombre total de descrregues sofertes en el pas i l'any donats.

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS


pais            CHAR(2);
param_entrada   VARCHAR2(50);
total           NUMBER;
i               INTEGER:=1;


RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
E_DATE          EXCEPTION;
resultat        TABLE3:=TABLE3();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Aplicacio: '||p_aplicacio||
                    ' Any: '||p_any;

-- verifiquem que no hi hagi parmetres d'entrada nuls
  IF(p_aplicacio IS NULL) THEN
     valor_nul:='aplicacio';
     RAISE E_NUL;
  ELSIF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;

  END IF;
 
-- verifiquem que l'any passat com parmetre no sigui major que l'actual 

  IF(p_any > to_number(to_char(sysdate,'YYYY'))) THEN
     RAISE E_DATE;
  
  END IF;

-- seleccionem un cursor amb el pas de descrrega i el nombre total d'usuaris que han descarregat l'aplicaci (per tant el nombre de descrregues)
-- `per al pas i aplicaci passats com a parmetre

   OPEN CUR FOR SELECT pais, COUNT(usuari) AS totalDesc FROM DESCARREGA 
   WHERE  EXTRACT(YEAR FROM dataDesc)=p_any and aplicacio=p_aplicacio
   GROUP BY pais ORDER BY totalDesc DESC;   
   
   LOOP
   FETCH CUR INTO pais,total;
   EXIT WHEN CUR%NOTFOUND;

-- emmagatzemem cada resultat de la iteraci de la consulta en la variable resultat

   resultat.extend;
   resultat(i):=LLISTA_PAISOS(pais,p_aplicacio,total);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
   WHEN E_DATE THEN
        RSP:= 'ERROR: l-any passat com a parmetre no pot ser ms gran que l-actual'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END PAISOS_NOMBRE_DESCARREGUES;
