/************************************************** ESTRUCTURES DE DADES CONSULTA 4 ******************************************/
CREATE OR REPLACE TYPE INFO_USUARI AS OBJECT (
       NIF             CHAR(9),
       telefon         NUMBER,
       email           VARCHAR2(50),
       operador        VARCHAR2(30),
       identificador   NUMBER,
       versio          VARCHAR2(20),
       preu            NUMBER,
       descripcio      VARCHAR2(200)
       
);
/
CREATE OR REPLACE TYPE TABLE4 AS TABLE OF INFO_USUARI;
/
CREATE OR REPLACE FUNCTION ACTIVITAT_USUARI (
       p_telefon     IN    NUMBER
)
RETURN TABLE4

/**********************************************************************************************************************************
   Nom:                 ACTIVITAT_USUARI
   Propsit:            Consulta que donat un usuari final identificat pel seu nmero de telfon retorna tota la seva activitat de descrregues
                        en la plataforma
                   
   Parmetres entrada:  -	p_telefon: telfon de l'usuari
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> NIF d l'usuari
                                >> Telefon de l'usuari
                                >> E-mail de l'usuari
                                >> Operador de l'usuari en el moment de la descrrega
                                >> Identificador numric de l'aplicaci descarregada
                                >> Versi de l'aplicaci descarregada
                                >> Preu que va pagar per l'aplicaci
                                >> Descripci de l'aplicaci

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

NIF             CHAR(9);
telefon         NUMBER;
email           VARCHAR2(50);
operador        VARCHAR2(30);
identificador   NUMBER;
versio          VARCHAR2(20);
preu            NUMBER;
descripcio      VARCHAR2(200);

param_entrada   VARCHAR2(20);
i               INTEGER:=1;
tlf             INTEGER;


RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
E_TELF          EXCEPTION;
resultat        TABLE4:=TABLE4();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Telefon: '||p_telefon;

-- verifiquem que no hi hagi parmetres d'entrada nuls
  IF(p_telefon IS NULL) THEN
     valor_nul:='telefon';
     RAISE E_NUL;

  END IF;

--verifiquem que el telefon existeixi en la base de dades
   
   SELECT COUNT(telefon) INTO tlf FROM PERSONA WHERE telefon=p_telefon;
   
   IF (tlf=0) THEN
   RAISE E_TELF;
 
   END IF;
   
   OPEN CUR FOR SELECT p.NIF,p.telefon,p.email,de.operador,ap.identificador,ap.versio,r.preu,r.descripcio
   FROM PERSONA p, USUARI u, DESCARREGA de, APLICACIO ap, REGISTRE r
   WHERE p.NIF=u.NIF AND u.NIF=de.usuari AND de.aplicacio=ap.identificador AND ap.identificador=r.aplicacio AND p.telefon=p_telefon AND de.pais=r.pais
   GROUP BY p.nif,p.telefon,p.email,de.operador,ap.identificador,ap.versio,r.preu,r.descripcio;
   
   LOOP
   FETCH CUR INTO NIF,telefon,email,operador,identificador,versio,preu,descripcio;
   EXIT WHEN CUR%NOTFOUND;
   resultat.extend;
   resultat(i):=INFO_USUARI(NIF,telefon,email,operador,identificador,versio,preu,descripcio);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
   WHEN E_TELF THEN
        RSP:= 'ERROR: el telfon introdut no existeix en la base de dades'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END ACTIVITAT_USUARI;
