/*********************************************** ESTRUCTURA DE DADES CONSULTA 5 **************************************************/

CREATE OR REPLACE TYPE LLISTA_USUARIS AS OBJECT (
       NIF             CHAR(9),
       telefon         NUMBER,
       email           VARCHAR2(50),
       total           NUMBER
       
);
/
CREATE OR REPLACE TYPE TABLE5 AS TABLE OF LLISTA_USUARIS;
/
CREATE OR REPLACE FUNCTION MAX_DESPESA (
       p_any     IN    NUMBER
)
RETURN TABLE5

/**********************************************************************************************************************************
   Nom:                 MAX_DESPESA
   Propsit:            Consulta que retorna per l'any passat per parmetre els 20 (com a mxim nombre) d'usuaris que ms diners s'han
                        gastat en la plataforma ordenats de ms a menys. 
                   
   Parmetres entrada:  -	p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> NIF de l'usuari
                                >> Telfon de l'usuari
                                >> E-mail de l'usuari
                                >> Total de diners gastats

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

NIF             CHAR(9);
telefon         NUMBER;
email           VARCHAR2(50);
pais            CHAR(2);
param_entrada   VARCHAR2(10);
total           NUMBER;
i               INTEGER:=1;


RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
E_DATE          EXCEPTION;
resultat        TABLE5:=TABLE5();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Any: '||p_any;

-- verifiquem que no hi hagi parmetres d'entrada nuls
  IF(p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;

  END IF;
 
-- verifiquem que l'any passat com parmetre no sigui major que l'actual 
  IF(p_any > to_number(to_char(sysdate,'YYYY'))) THEN
     RAISE E_DATE;
  
  END IF;

-- seleccionem un cursor amb les dades dels usuaris que ms han gastat ordenats per gasto descendent

   OPEN CUR FOR SELECT u.nif,p.telefon,p.email,SUM(r.preu) AS total FROM USUARI u, PERSONA p, DESCARREGA d, APLICACIO aP, REGISTRE r
   WHERE p.nif=u.nif AND u.nif=d.usuari AND d.aplicacio=aP.identificador AND aP.identificador=r.aplicacio AND d.pais=r.pais AND
   p_any=EXTRACT(year from d.dataDesc) GROUP BY u.nif,p.telefon,p.email ORDER BY total DESC;
   
   LOOP
   FETCH CUR INTO NIF,telefon,email,total;

-- obliguem que hi hagi un mxim de 20 files i anem emmagatzemant les dades en la variable resultat

   EXIT WHEN CUR%NOTFOUND OR CUR%ROWCOUNT>20;
   resultat.extend;
   resultat(i):=LLista_usuaris(NIF,telefon,email,total);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
   WHEN E_DATE THEN
        RSP:= 'ERROR: l-any passat com a parmetre no pot ser ms gran que l-actual'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END MAX_DESPESA;
