/******************************************** ESTRUCTURA DE DADES CONSULTA 6 **********************************************/

CREATE OR REPLACE TYPE LLISTA_SO AS OBJECT (
       ao                  NUMBER,
       sistema              VARCHAR2(30),
       total                NUMBER
);

/

CREATE OR REPLACE TYPE TABLE6 AS TABLE OF LLISTA_SO;

/

CREATE OR REPLACE FUNCTION SO_MES_UTILITZATS(
         p_any       IN   NUMBER
)

RETURN TABLE6

CREATE OR REPLACE TYPE LLISTA_SO AS OBJECT (
       ao                  NUMBER,
       sistema              VARCHAR2(30),
       total                NUMBER
);
/
CREATE OR REPLACE TYPE TABLE6 AS TABLE OF LLISTA_SO;
/
CREATE OR REPLACE FUNCTION SO_MES_UTILITZATS(
         p_any       IN   NUMBER
)

RETURN TABLE6


/**********************************************************************************************************************************
   Nom:                 SO_MES_UTILITZATS
   Propsit:            Consulta que retorna per l'any passat per parmetre un llistat amb els sistemes operatius de telfons intelligents
                        que ms descrregues han fet junt amb aquest nombre (ordenats pel nombre total de descrregues)
   Parmetres entrada:  -	p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> Any de consulta
                                >> Nom del sistema operatiu
                                >> Total de descrregues que ha fet aquest sistema operatiu en la plataforma

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/


IS

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(500);
RSP             VARCHAR2(500);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
resultat        TABLE6:=TABLE6();
i               INTEGER:=1;

-- seleccionem un cursor amb l'any, el sistema operatiu i el nombre total de descrregues que ha fet en la plataforma en aquell any
CURSOR CUR IS SELECT EXTRACT(year from de.dataDesc),d.sistemaoperatiu,COUNT( d.sistemaoperatiu ) AS SO FROM DESCARREGA de,Dispositiu d
              WHERE   EXTRACT(year from de.datadesc)=p_any AND de.dispositiu=d.Imei
              GROUP BY d.sistemaoperatiu,EXTRACT(year from de.dataDesc) ORDER BY SO DESC;

sistema        VARCHAR2(40);
anyo           NUMBER;
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any;
                    

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  END IF;

--obrim cursor i iterem

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,sistema,total;
         EXIT WHEN CUR%NOTFOUND;

    -- anem emmagatzemant les dades en la variable resultat

          resultat.extend;
          resultat(i):=LLista_SO(anyo,sistema,total);
          i:=i+1;
       
       END LOOP;
     CLOSE CUR;   

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SO_MES_UTILITZATS', param_entrada, 'RSP: '||RSP);   

RETURN resultat;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SO_MES_UTILITZATS', param_entrada, 'RSP: '||RSP);   
         DBMS_OUTPUT.PUT_LINE(RSP);
         RETURN NULL;

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SO_MES_UTILITZATS', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
         RETURN NULL;
END SO_MES_UTILITZATS;