

--Creaci de taules

CREATE TABLE Persona
(
  NIF             CHAR(9)      NOT NULL,
  telefon         NUMBER        NOT NULL,
  email           VARCHAR2(30)  NOT NULL,

  CONSTRAINT PK_Persona PRIMARY KEY (NIF),
  CONSTRAINT UK_Telefon UNIQUE (telefon)
);

CREATE TABLE Usuari
(
  NIF             CHAR(9)       NOT NULL,
  pais            CHAR(2)       NOT NULL,
  operador        VARCHAR2(30)  NOT NULL,

  CONSTRAINT PK_Usuari PRIMARY KEY (NIF),
  CONSTRAINT FK_Usuari FOREIGN KEY (NIF) REFERENCES Persona(NIF)
);
CREATE TABLE Desenvolupador
(
  NIF              CHAR(9)      NOT NULL,
  dataNaixement    DATE  ,       
  llocNaixement    VARCHAR2(30) ,
  pais             VARCHAR2(30)  NOT NULL,

  CONSTRAINT PK_Desenvolupador PRIMARY KEY (NIF),
  CONSTRAINT FK_Desenvolupador FOREIGN KEY (NIF) REFERENCES Persona(NIF)
);

CREATE TABLE SO
(
  sistemaOperatiu  VARCHAR2(30) NOT NULL,
 
  CONSTRAINT PK_SO PRIMARY KEY (sistemaOperatiu)
);

CREATE TABLE Dispositiu
(
   IMEI            NUMBER       NOT NULL,
   resolucio       NUMBER       NOT NULL,
   modelo          VARCHAR2(30) NOT NULL,
   usuaris         CHAR(9)      NOT NULL,
   sistemaOperatiu VARCHAR2(30) NOT NULL,

  CONSTRAINT PK_Dispositiu PRIMARY KEY (IMEI),
  CONSTRAINT FK_SODispositiu FOREIGN KEY (sistemaOperatiu) REFERENCES SO(sistemaOperatiu),
  CONSTRAINT FK_UsuariDispositiu FOREIGN KEY (usuaris) REFERENCES Usuari(NIF)
);

CREATE TABLE Pais
(
   codiPais        CHAR(2)       NOT NULL, 

   CONSTRAINT PK_Pais PRIMARY KEY (codiPais)
);


CREATE TABLE Aplicacio
(
   identificador   NUMBER        NOT NULL,
   versio          VARCHAR2(30)  NOT NULL,
   dataPujada      DATE          NOT NULL,
   URL             VARCHAR2(100) , 
   activa          CHAR(2)       NOT NULL,
   resolucio       NUMBER        NOT NULL,

   CONSTRAINT PK_Aplicacio PRIMARY KEY (identificador),
   CONSTRAINT CK_Aplicacio CHECK (activa IN('SI', 'NO'))
);

CREATE TABLE Empresa
(
   codi            VARCHAR2(30)  NOT NULL,
   nom             VARCHAR2(30)  NOT NULL,
   representant    VARCHAR2(30)  NOT NULL,
   pais            CHAR(2)       NOT NULL,
   adrea          VARCHAR2(30)  NOT NULL,
   telefon         NUMBER        NOT NULL,

   CONSTRAINT PK_Empresa PRIMARY KEY (codi),
   CONSTRAINT UK_EmpresaTlf UNIQUE (telefon)
);

CREATE TABLE Descarrega
(
   usuari          CHAR(9)       NOT NULL,
   aplicacio       NUMBER        NOT NULL,
   dataDesc        DATE          NOT NULL,
   pagament        VARCHAR2(30)  NOT NULL,
   operador        VARCHAR2(30)  NOT NULL,
   pais            CHAR(2)       NOT NULL,
   dispositiu      NUMBER        NOT NULL,

   CONSTRAINT PK_Descarrega PRIMARY KEY (usuari, aplicacio),
   CONSTRAINT FK_UsuariDescarrega FOREIGN KEY (usuari) REFERENCES Usuari(NIF),
   CONSTRAINT FK_AplicacioDescarrega FOREIGN KEY (aplicacio) REFERENCES Aplicacio(identificador),
   CONSTRAINT FK_DispositiuDescarrega FOREIGN KEY (dispositiu) REFERENCES  Dispositiu(IMEI)
);

CREATE TABLE Creacio
(
   desenvolupador  CHAR(9)  NOT NULL,
   aplicacio       NUMBER   NOT NULL,

   CONSTRAINT PK_Creacio PRIMARY KEY (desenvolupador,aplicacio),
   CONSTRAINT FK_DesenvolupadorCreacio FOREIGN KEY (desenvolupador) REFERENCES Desenvolupador(NIF),
   CONSTRAINT FK_AplicacioCreacio FOREIGN KEY (aplicacio) REFERENCES Aplicacio(identificador)
);

CREATE TABLE Treballa
(
   desenvolupador  CHAR(9)           NOT NULL,
   empresa         VARCHAR2(30)      NOT NULL,

   CONSTRAINT PK_Treballa PRIMARY KEY (desenvolupador, empresa),
   CONSTRAINT FK_DesenvolupadorTreballa FOREIGN KEY (desenvolupador) REFERENCES Desenvolupador(NIF),
   CONSTRAINT FK_EmpresaTreballa FOREIGN KEY (empresa) REFERENCES Empresa(codi)
);


CREATE TABLE Viable
(
   aplicacio      NUMBER        NOT NULL,
   so             VARCHAR2(30)  NOT NULL,
   mida           NUMBER        NOT NULL,
   enlla         VARCHAR2(100) ,
 
   CONSTRAINT PK_Viable PRIMARY KEY (aplicacio, so),
   CONSTRAINT FK_AplicacioViable FOREIGN KEY (aplicacio) REFERENCES Aplicacio(identificador),
   CONSTRAINT FK_SOViable FOREIGN KEY (so) REFERENCES SO(sistemaOperatiu),
   CONSTRAINT CK_Viable CHECK (mida>0)
);

CREATE TABLE Registre
(
   aplicacio      NUMBER         NOT NULL,
   pais           CHAR(2)        NOT NULL,
   preu           NUMBER         NOT NULL,
   descripcio     VARCHAR2(100),
  
   CONSTRAINT PK_Registre PRIMARY KEY (aplicacio, pais),
   CONSTRAINT FK_AplicacioRegistre FOREIGN KEY (aplicacio) REFERENCES Aplicacio(identificador),
   CONSTRAINT FK_PaisRegistre FOREIGN KEY (pais) REFERENCES Pais(codiPais),
   CONSTRAINT CK_Registre CHECK (preu>0)
);

CREATE TABLE EST1
(
   totalDescarregues  NUMBER            NOT NULL,

   CONSTRAINT PK_EST1 PRIMARY KEY (totalDescarregues)

);

CREATE TABLE EST2
(
  
   totalEuros         NUMBER NOT NULL,

   CONSTRAINT PK_EST2 PRIMARY KEY (totalEuros)

);

CREATE TABLE EST3
(
   ao       NUMBER             NOT NULL,
   mitjana   NUMBER   DEFAULT 0 NOT NULL,

   CONSTRAINT PK_EST3 PRIMARY KEY (ao)

);

CREATE TABLE EST4
(
   ident          NUMBER           NOT NULL,
   ao            NUMBER           NOT NULL,
   desenvolupador CHAR(9)          NOT NULL,
   nombre         NUMBER DEFAULT 0 NOT NULL,

   CONSTRAINT PK_EST4 PRIMARY KEY (ident)

);

CREATE TABLE EST5
( 
   ident          NUMBER           NOT NULL,
   ao            NUMBER           NOT NULL,
   desenvolupador CHAR(9)          NOT NULL,
   aplicacio      NUMBER           NOT NULL,
   recaptacio     NUMBER  NOT NULL,

   CONSTRAINT PK_EST5 PRIMARY KEY (ident)


);

CREATE TABLE EST6
(
   ident          NUMBER           NOT NULL,
   ao            NUMBER           NOT NULL,
   pais           CHAR(2)          NOT NULL,
   nombreUsuaris  NUMBER DEFAULT 0 NOT NULL, 

   CONSTRAINT PK_EST6 PRIMARY KEY (ident)

);

CREATE TABLE EST7
(
   ident          NUMBER           NOT NULL,
   ao            NUMBER           NOT NULL,
   pais           CHAR(2)          NOT NULL,
   ingressos     NUMBER DEFAULT 0  NOT NULL, 

   CONSTRAINT PK_EST7 PRIMARY KEY (ident)


);

CREATE TABLE EST8
(
   ident               NUMBER           NOT NULL,
   ao                 NUMBER           NOT NULL,
   pais                CHAR(2)          NOT NULL,
   nombreDescarregues  NUMBER DEFAULT 0 NOT NULL, 

   CONSTRAINT PK_EST8 PRIMARY KEY (ident)


);

CREATE TABLE Logs
(
   ident         NUMBER       NOT NULL,
   dataHora      VARCHAR2(30) NOT NULL,
   procediment   VARCHAR2(30) NOT NULL,
   param_entrada VARCHAR2(220) NOT NULL,
   param_sortida VARCHAR2(300) NOT NULL,

   CONSTRAINT PK_LOG PRIMARY KEY (ident)
);


/

-- Parell seqncia - disparador per inserci de codi automtic en el mdul estadstic i LOGS

CREATE SEQUENCE SQ_LOGS INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_LOGS 
  BEFORE INSERT ON LOGS
  FOR EACH ROW
BEGIN
  SELECT SQ_LOGS.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_LOGS; 
/

CREATE SEQUENCE SQ_EST4 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_EST4
  BEFORE INSERT ON EST4
  FOR EACH ROW
BEGIN
  SELECT SQ_EST4.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_EST4; 
/


CREATE SEQUENCE SQ_EST5 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_EST5
  BEFORE INSERT ON EST5
  FOR EACH ROW
BEGIN
  SELECT SQ_EST5.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_EST5; 
/

CREATE SEQUENCE SQ_EST6 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_EST6
  BEFORE INSERT ON EST6
  FOR EACH ROW
BEGIN
  SELECT SQ_EST6.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_EST6; 
/

CREATE SEQUENCE SQ_EST7 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_EST7
  BEFORE INSERT ON EST7
  FOR EACH ROW
BEGIN
  SELECT SQ_EST7.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_EST7; 
/
CREATE SEQUENCE SQ_EST8 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_EST8
  BEFORE INSERT ON EST8
  FOR EACH ROW
BEGIN
  SELECT SQ_EST8.NEXTVAL INTO :NEW.ident
  FROM DUAL;
END TR_EST8; 
/

create or replace PROCEDURE SP_ALTA_LOGS (
   p_datahora        IN LOGS.datahora%TYPE,
   p_procediment     IN LOGS.procediment%TYPE,
   p_param_entrada   IN LOGS.param_entrada%TYPE,
   p_param_sortida   IN LOGS.param_sortida%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_LOGS
   Propsit:            afegeix una entrada a la taula LOGS
                   
   Parmetres entrada:  p_data_hora, p_nom_procediment, p_param_entrada, p_param_sortida            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP  VARCHAR2 (500);
error_msg  VARCHAR2 (200);
valor_nul VARCHAR2(20);
E_NUL     EXCEPTION;

PRAGMA AUTONOMOUS_TRANSACTION;

BEGIN

   param_entrada:= 'p_datahora: '||p_datahora||
                 ', pprocediment: '||p_procediment||
                 ', p_param_entrada: '||p_param_entrada||
                 ', p_param_sortida: '||p_param_sortida;

  --verifica que els camps obligatoris no siguin nuls
   IF (p_datahora IS NULL) THEN 
     valor_nul:= 'dataHora';
     RAISE E_NUL;
   ELSIF (p_procediment IS NULL) THEN 
     valor_nul:= 'procediment';     
     RAISE E_NUL;
   ELSIF (p_param_entrada IS NULL) THEN 
     valor_nul:= 'param_entrada';
     RAISE E_NUL;     
   ELSIF (p_param_sortida IS NULL) THEN 
     valor_nul:= 'param_sortida';
     RAISE E_NUL;     
   END IF;
 

  -- inserta la nova entrada a la taula log
   INSERT INTO LOGS (datahora, procediment, param_entrada, param_sortida)
   VALUES (p_datahora, p_procediment, p_param_entrada, p_param_sortida);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR LOGS: el camp ' || valor_nul || ' no pot ser nul'; 
        INSERT INTO LOGS (datahora, procediment, param_entrada, param_sortida)
        VALUES (to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LOGS', p_param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);

        
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR LOGS: ' || error_msg;
        INSERT INTO LOGS (datahora, procediment, param_entrada, param_sortida)
        VALUES (to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LOGS', p_param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);

              
END SP_ALTA_LOGS;
/


/***********************************************************  EST1    *******************************************************/
/***********************************************************************************************************************************************
   Nom:                 SP_ALTA_EST1
   Propsit:            Fa el clcul de totes les descrregues fetes i si s el primer valor calculat, l'afegeix  a la taula EST1
                        i si no s el primer, l'afegeix a EST1 havent esborrat abans l'antic valor de la taula. Aix doncs, en la taula 
                        sempre i haur un nic valor: el total de descrregues fetes.
   Parmetres entrada:  Sense parmetres           

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
***************************************************************************************************************************************************/

Create or replace PROCEDURE SP_ALTA_EST1

IS

--variables

param_entrada VARCHAR2(500);
totalDesc     INTEGER;
comptador     INTEGER;
RSP           VARCHAR2(500);
error_msg     VARCHAR2(500);


BEGIN
   param_entrada:='Sense parmetres';

-- calculem les descrregues fetes i el total de files inserides en la taula EST1

   SELECT COUNT(*) INTO totalDesc FROM DESCARREGA;
   SELECT COUNT(*) INTO comptador FROM EST1;

-- si la taula EST1 est buida afegim el valor del total de descrregues que s'han fet i si no est buida
-- esborrem l'ltim valor de la taula EST1 i afegim el nou valor (que ser l'antic +1)

   IF(comptador=0) THEN
   
   INSERT INTO EST1(totalDescarregues) VALUES (totaldesc);

   ELSE

    SELECT totalDescarregues INTO comptador FROM EST1;
    DELETE FROM EST1 WHERE totalDescarregues=comptador;
    INSERT INTO EST1(totalDescarregues) VALUES (totalDesc);

   END IF;

--  inserci correcta
   
    RSP:='OK';
    SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST1', param_entrada, 'RSP: '||RSP);
  


COMMIT;

EXCEPTION
 
 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST1', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST1;

/
/*************************************************************** EST2  *****************************************************************************************/
/*****************************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST2
   Propsit:            Calcula el total d'euros generats per la plataforma en descrregues fins el moment
                        del clcul. Afegeix aquest nombre a la taula EST2 si la taula est buida i sin, esborra
                        l'antic valor de EST2 i afegeix el nou valor: per tant, sempre hi haur un nic valor
                        que ser el total d'euros generats per la plataforma.
                   
   Parmetres entrada:  Sense parmetres            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
*****************************************************************************************************************************************************************/

Create or replace PROCEDURE SP_ALTA_EST2

IS

--variables

param_entrada VARCHAR2(500);
comptador     INTEGER;


SUMA            INTEGER:=0;
PRICE           NUMBER;
RSP             VARCHAR2(500);
error_msg       VARCHAR2(500);
TYPE REFCURSOR  IS REF CURSOR;
CUR REFCURSOR;


BEGIN

  param_entrada:='Sense parmetres';

--obrim un cursor amb tots els preus de cada pas de cada aplicaci descarregada. S'itera sumant cada preu per obtenir el total de euros recaudats

  OPEN CUR FOR SELECT R.PREU  FROM REGISTRE R,DESCARREGA DE,APLICACIO AP WHERE R.PAIS=DE.PAIS AND AP.IDENTIFICADOR=DE.APLICACIO AND
  AP.IDENTIFICADOR=R.APLICACIO;
  LOOP
   
    FETCH CUR INTO PRICE;
    EXIT WHEN CUR%NOTFOUND;
 
    suma:=suma+price;

  END LOOP;
  CLOSE CUR;

-- anem modificant el valor de la taula EST2 per cada descrrega. Esborrem el valor antic de la taula EST2 i inserim la nova suma.

  SELECT COUNT(*) INTO comptador FROM EST2;

  IF(comptador=0) THEN

     INSERT INTO EST2(totalEuros) VALUES (suma);

  ELSE
   
     SELECT totalEuros INTO comptador FROM EST2;
     DELETE FROM EST2 WHERE totalEuros=comptador;
     INSERT INTO EST2(totalEuros) VALUES (suma);
    
  END IF;

--  inserci correcta

    RSP:='OK';
    SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST2', param_entrada, 'RSP: '||RSP);
  


COMMIT;

EXCEPTION

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST2', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST2;

/

/*********************************************************   EST3    ***********************************************************************/
/********************************************************************************************************************************************
   Nom:                 SP_ALTA_EST3
   Propsit:            Donat un any passat com a parmetre d'entrada calcula la mitjana d'aplicacions descarregades, s a dir,
                        el total d'aplicacions descarregades en l'any donat dividit pel total d'usuaris diferents que han fet descrregues
                        aquell any.
                   
   Parmetres entrada:  p_any           

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
*******************************************************************************************************************************************************/
CREATE OR REPLACE PROCEDURE SP_ALTA_EST3 (
    p_any              NUMBER
    )
    
IS

--variables


param_entrada     VARCHAR2(50);
error_msg         VARCHAR2(100);
RSP               VARCHAR2(200);
mitja             NUMBER(9,2);
comptador         INTEGER;
nombreUsuaris     INTEGER;
nombreAplicacions INTEGER;
E_NUL             EXCEPTION;


BEGIN

  param_entrada:= 'Any: '||p_any;

--controlem que els parmetres d'entrada no siguin NULL

  IF(p_any IS NULL) THEN
     RAISE E_NUL;
  END IF;
  
--calculem el total d'aplicacions descarregades i el total d'usuaris diferents que  han descarregat aplicacions; Calculem la mitja

  SELECT COUNT(aplicacio) INTO nombreAplicacions FROM DESCARREGA de WHERE  EXTRACT(YEAR FROM dataDesc)=p_any;
  SELECT COUNT(DISTINCT usuari) INTO nombreUsuaris FROM DESCARREGA WHERE EXTRACT(YEAR FROM dataDesc)=p_any;
  
  mitja:=nombreAplicacions/nombreUsuaris;

--inserim els valors si la taula EST3 est buida o la modifiquem amb la nova mitja sin est buida
  
  SELECT COUNT(*) INTO comptador FROM EST3 WHERE ao=p_any;
  
  IF(comptador=0) THEN
  
      INSERT INTO EST3(ao, mitjana) VALUES (p_any,mitja);
      
  ELSE
    
      UPDATE EST3 SET mitjana=mitja WHERE ao=p_any;
      
  END IF;

--inserci o modificaci correcta
   
    RSP:='OK';
    SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST3', param_entrada, 'RSP: '||RSP);
  

    
COMMIT;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST3', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
 

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST3', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST3;

/

/****************************************************************   EST4   ************************************************************************/
/**************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST4
   Propsit:            Donat un any passat com a parmetre d'entrada, retorna el desenvolupador o desenvolupadors que han 
                        creat de l'aplicaci o aplicacions ms descarregades en la plataforma l'any donat, junt amb el nombre de 
                        descrregues que han tingut.
   Parmetres entrada:  p_any            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
****************************************************************************************************************************************************/
create or replace PROCEDURE SP_ALTA_EST4(
     p_any          NUMBER
     )
IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
desenv          VARCHAR2(10);
totalDesc       INTEGER;
comptador       INTEGER;
maxim           INTEGER;

E_NUL           EXCEPTION;

type refcursor  IS REF CURSOR;
CUR  refcursor;


BEGIN

   param_entrada:='Any: '||p_any;

-- controlem que el parmetre d'entrada no sigui null

   IF (p_any IS NULL) THEN
      RAISE E_NUL;
   END IF;

--obrim un cursor amb tots els desenvolupadors i el nombre d'aplicacions descarregades en l'any donat
   
   OPEN CUR FOR SELECT cr.desenvolupador, COUNT(de.aplicacio) AS total from DESCARREGA de,APLICACIO ap,CREACIO cr
   WHERE de.aplicacio=ap.identificador  AND ap.identificador=cr.aplicacio AND EXTRACT(YEAR FROM DE.dataDesc)=p_any
   GROUP BY cr.desenvolupador ORDER BY total ASC;

-- anem inserint en la taula EST4 el valor de les variables per cada iteraci de manera que no hi hagi files repetides en les iteracions
   
   LOOP
     FETCH CUR INTO desenv,totalDesc;
     EXIT WHEN CUR%NOTFOUND;

     SELECT COUNT(*) INTO comptador FROM EST4 WHERE ao=p_any AND desenvolupador=desenv AND nombre=totalDesc;

     
     IF(comptador=0) THEN
     INSERT INTO EST4(ao,desenvolupador,nombre) VALUES(p_any,desenv,totalDesc);
     END IF;

  END LOOP;
  CLOSE CUR;

--seleccionem el nombre mxim de descrregues i esborrem totes aquelles files que no tinguin el mxim nombre de descrregues; 
-- aix en la taula quedaran nomes l'any, i els desenvolupadors que han tingut mxim nombre de descrregues junt amb aquest nombre.
   
   
  
       SELECT MAX(nombre) INTO maxim FROM EST4 WHERE ao=p_any;

       DELETE FROM EST4 WHERE ao=p_any AND nombre<>maxim;
       
       
  

--inserci o modificaci correcta
  
   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST4', param_entrada, 'RSP: '||RSP);
 


COMMIT;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST4', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST4', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

END SP_ALTA_EST4;

/

/*******************************************************************    EST5   ******************************************************************/
/********************************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST5
   Propsit:            Donat un any passat com a parmetre d'entrada, retorna el desenvolupador o desenvolupadors que han creat l'aplicaci o aplicacions (en cas
                        de que dues aplicacions hagin recaudat el mateix) que han recaudat ms diners en la plataforma. 
                   
   Parmetres entrada:  p_any            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
*******************************************************************************************************************************************************************/

create or replace PROCEDURE SP_ALTA_EST5(
     p_any          NUMBER
     )
IS


--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);

aplic           NUMBER;
price           NUMBER;
desenv          VARCHAR2(10);
maxRecap        NUMBER;
total           INTEGER:=0;

E_NUL           EXCEPTION;

-- emmagatzemem en el cursor els desenvolupadors, aplicacions i suma de preus obtingut en la descrrega

CURSOR CUR IS SELECT DISTINCT cr.desenvolupador, de.aplicacio, SUM(r.preu) FROM creacio cr, DESCARREGA de,
   APLICACIO ap,registre r
   WHERE de.aplicacio=ap.identificador AND ap.identificador=r.aplicacio  AND de.pais=r.pais 
   AND cr.aplicacio=ap.identificador  AND EXTRACT(YEAR FROM DE.dataDesc)=p_any
   GROUP BY cr.desenvolupador,de.aplicacio;


BEGIN

   param_entrada:='Any: '||p_any;

--controlem que el parmetre d'entrada no sigui null

   IF (p_any IS NULL) THEN
       RAISE E_NUL;
   END IF;

--obrim el cursor i iterem. S'insereixen noms aquells valors que no estiguin repetits per evitar repeticions posteriorment de registres
-- iguals amb la mxima recaptaci.
   
   OPEN CUR; 
   
   LOOP
     FETCH CUR INTO desenv,aplic,price;
     EXIT WHEN CUR%NOTFOUND;
      SELECT COUNT(*) INTO total FROM EST5 WHERE desenvolupador=desenv AND aplicacio=aplic AND recaptacio=price;
       IF (total=0) THEN
       INSERT INTO EST5(ao,desenvolupador,aplicacio,recaptacio) VALUES (p_any,desenv,aplic,price);
       END IF;
   END LOOP;
   CLOSE CUR;

--calculem la mxima recaptaci de cada any i esborrem tots aquells registres que no tinguin la recaptaci mxima
  
   SELECT MAX(recaptacio) INTO maxRecap FROM EST5 WHERE ao=p_any ;
   DELETE FROM est5 WHERE ao=p_any AND recaptacio<>maxRecap;
   
-- execuci correcta

   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST5', param_entrada, 'RSP: '||RSP);  
 

   

COMMIT;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST5', param_entrada, 'RSP: '||RSP);  
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST5', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

END SP_ALTA_EST5;

/


/******************************************************************  EST6 *************************************************************************/
/****************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST6
   Propsit:            Donats un any i un pas passats com a parmetre d'entrada retorna el nombre d'usuaris que han fet 
                        com mnim una descrrega. 
                   
   Parmetres entrada:  p_any, p_pais            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
**********************************************************************************************************************************************/
create or replace PROCEDURE SP_ALTA_EST6(
        p_any        IN EST6.ao%TYPE,
        p_pais       IN EST6.pais%TYPE
  )

IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
valor_nul       VARCHAR2(30);
comptador       INTEGER;
suma            INTEGER;
E_NUL           EXCEPTION;

--emmagatzemem en el cursor l'any, el pais i el nombre total d'usuaris diferents que ha fet descrregues en aquell any i pas.

CURSOR CUR IS SELECT EXTRACT(year from datadesc),pais,COUNT(DISTINCT usuari) FROM DESCARREGA WHERE pais=p_pais and
              EXTRACT(year from datadesc)=p_any GROUP BY EXTRACT(year from datadesc),pais;

anyo           NUMBER;
country        CHAR(2);
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any||
                      ', Pais: '||p_pais;

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;
  END IF;

--obrim el cursor i iterem. Anem inserint en la taula EST6 l'any, el pas i el total de d'usuaris diferents que han fet descrregues i de manera
-- que no hi hagi files repetides

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,country,total;
         EXIT WHEN CUR%NOTFOUND;
        
            SELECT COUNT(*) INTO suma FROM EST6 WHERE ao=p_any AND pais=p_pais AND nombreUsuaris=total;

            IF(suma=0) THEN
            INSERT INTO EST6(ao,pais,nombreUsuaris) VALUES (p_any,p_pais,total);
            END IF;
       
       END LOOP;
     CLOSE CUR;
       
-- esborrem tots el registres que no tinguin el mxim nmero d'usuaris que fan descrregues.

       SELECT MAX(nombreusuaris) INTO total FROM est6 WHERE pais=p_pais AND ao=p_any;
       DELETE FROM EST6 WHERE nombreusuaris<>total AND ao=p_any AND pais=p_pais;
     

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST6', param_entrada, 'RSP: '||RSP);
       



EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST6', param_entrada, 'RSP: '||RSP);  
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST6', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST6;


/

/*******************************************************************  EST7   ************************************************************************/
/*******************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST7
   Propsit:            Donat una any i un pas passats com a parmetre d'entrada retorna els ingressos que han generat els usuaris d'aquell
                        pas en aquell any en descrregues d'aplicacions. 
                   
   Parmetres entrada:  p_any,p_pais            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************************************************************************************/
create or replace PROCEDURE SP_ALTA_EST7(
        p_any        IN EST7.ao%TYPE,
        p_pais       IN EST7.pais%TYPE
  )

IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
valor_nul       VARCHAR2(30);
comptador       INTEGER;
suma            INTEGER;
E_NUL           EXCEPTION;

-- emmagatzemem en el cursor l'any, el pas de descrrega i la suma d'ingresos per aquell pas i any

CURSOR CUR IS SELECT EXTRACT(year from de.dataDesc),de.pais,SUM(r.preu) FROM DESCARREGA de,APLICACIO ap,REGISTRE r WHERE r.pais=p_pais AND
              de.pais=p_pais AND extract(year from de.datadesc)=p_any AND de.aplicacio=ap.identificador AND ap.identificador=r.aplicacio
              GROUP BY de.pais,EXTRACT(year from de.dataDesc);

country        CHAR(2);
anyo           NUMBER;
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any||
                      ', Pais: '||p_pais;

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;
  END IF;

--es van inserint en la taula EST7 de manera que no hi hagi registre repetits amb el mxim nombre d'ingressos.

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,country,total;
         EXIT WHEN CUR%NOTFOUND;
         SELECT COUNT(*) INTO comptador FROM EST7 WHERE p_pais=pais AND ao=p_any AND total=ingressos;
         IF(comptador=0) THEN
            INSERT INTO EST7(ao,pais,ingressos) VALUES (p_any,p_pais,total);
         END IF;
       
       END LOOP;
     CLOSE CUR;
       
-- esborrem de la taula tots aquells registres que no tinguin el major nombre d'ingressos per a l'any i pas donat.

       SELECT MAX(ingressos) INTO total FROM est7 WHERE pais=p_pais AND ao=p_any;
       DELETE FROM EST7 WHERE ingressos<>total AND ao=p_any AND pais=p_pais;
     

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST7', param_entrada, 'RSP: '||RSP); 
        



EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST7', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST7', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST7;


/

/*****************************************************************  EST8 ************************************************************************/
/************************************************************************************************************************************************
   Nom:                 SP_ALTA_EST8
   Propsit:            Donat un any i un pas passats com a parmetre d'entrada retorna el nombre d'aplicacions diferents descarregades
                        almenys una vegada en aquell any i pas.
                   
   Parmetres entrada:  p_any, p_pais            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
************************************************************************************************************************************************/

create or replace
PROCEDURE SP_ALTA_EST8(
        p_any        IN EST8.ao%TYPE,
        p_pais       IN EST8.pais%TYPE
  )

IS

--variables

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(100);
RSP             VARCHAR2(200);
valor_nul       VARCHAR2(30);
comptador       INTEGER;
suma            INTEGER;
E_NUL           EXCEPTION;

-- emmagatzemem l'any, el pas i el nombre total de descrregues d'aplicacions diferents 

CURSOR CUR IS SELECT EXTRACT(year from dataDesc),pais,COUNT(DISTINCT aplicacio) FROM DESCARREGA 
              WHERE pais=p_pais AND EXTRACT(year from datadesc)=p_any 
              GROUP BY pais,EXTRACT(year from dataDesc);

country        CHAR(2);
anyo           NUMBER;
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any||
                      ', Pais: '||p_pais;

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;
  END IF;

--es van inserint en la taula EST8 de manera que no hi hagi registre repetits amb el mxim total d'aplicacions diferents

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,country,total;
         EXIT WHEN CUR%NOTFOUND;
         SELECT COUNT(*) INTO comptador FROM EST8 WHERE p_pais=pais AND ao=p_any AND total=nombreDescarregues;
         IF(comptador=0) THEN
            INSERT INTO EST8(ao,pais,nombreDescarregues) VALUES (p_any,p_pais,total);
         END IF;
       
       END LOOP;
     CLOSE CUR;

--esborrem tots aquells registres en qu el nombre d'aplicacions diferents s diferent de l'ltim comptat. D'aquest manera noms hi haur una 
-- fila en la taula.
       
       SELECT MAX(nombreDescarregues) INTO total FROM EST8 WHERE pais=p_pais AND ao=p_any;
       DELETE FROM EST8 WHERE nombreDescarregues<>total AND ao=p_any AND pais=p_pais;
     

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST8', param_entrada, 'RSP: '||RSP); 
      



EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' any no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST8', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'EST8', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


END SP_ALTA_EST8;
/


CREATE OR REPLACE PROCEDURE SP_ALTA_USUARI (
    p_NIF      IN  PERSONA.NIF%TYPE,
    p_telefon  IN  PERSONA.telefon%TYPE,
    p_email    IN  PERSONA.email%TYPE,
    p_pais     IN  USUARI.pais%TYPE,
    p_operador IN  USUARI.operador%TYPE,
    RSP        OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_ALTA_USUARI
   Funci:                   Afegeix un usuari a la taula USUARI i si no existeix en la taula PERSONA, l'afegeix tamb
   Parmetre d'entrada:      - p_NIF: el NIF de l'usuari
                             - p_telefon: telefon de l'usuari
                             - p_email: email de l'usuari
                             - p_pais: pais de registre de l'usuari 
                             - p_operador: operador telefnic de l'usuari

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/

param_entradaPersona  VARCHAR2(500);
param_entradaUsuari   VARCHAR2(500);
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;
E_PK1 EXCEPTION;

BEGIN

  param_entradaPersona:= 'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', email: '||p_email;

  param_entradaUsuari:= 'NIF: '||p_NIF||
                        ', Pais: '||p_pais||
                        ', Operador: '||p_operador;
 
  param_entrada:=       'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', Email: '||p_email||
                         ', Pais: '||p_pais||
                         ', Operador: '||p_operador;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  ELSIF (p_operador IS NULL) THEN
      valor_nul:='operador';
      RAISE E_NUL;
  END IF;

-- inserim el usuari en la taula Persona si prviament no existeix; sin generem una exepci

   SELECT COUNT(*) INTO comptador FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0) THEN
      
       RSP:='OK';
   
      INSERT INTO PERSONA (NIF,telefon,email) VALUES (p_NIF,p_telefon,p_email);
  
   ELSE 

       RAISE E_PK1;

   END IF;

-- inserim el nou usuari si no existeix; sin generem una excepci

   SELECT COUNT(*) INTO comptador FROM USUARI U WHERE U.NIF=p_NIF;
 
   IF (comptador=0) THEN

      INSERT INTO USUARI(NIF, pais, operador) VALUES (p_NIF,p_pais, p_operador);
  
   ELSE
      RAISE E_PK;
  
   END IF;

-- inserci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:='ERROR: ja existeix aquest usuari en la taula USUARI';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
    WHEN E_PK1 THEN
      RSP:='ERROR: ja existeix aquest usuari en la taula PERSONA';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
      ROLLBACK;

      --violaci de la clau nica que s el telfon, ja que les altres claus niques estan controlades per E_PK i E_PK1
      IF (SQLCODE=-00001) THEN
          RSP:='ERROR: ja existeix un usuari o un desenvolupador amb el mateix telefon';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entradaPersona, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);

      ELSE
         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

      END IF;
END SP_ALTA_USUARI;
    
/

CREATE OR REPLACE PROCEDURE SP_BAIXA_USUARI (
    p_NIF      IN  PERSONA.NIF%TYPE,
    RSP        OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_BAIXA_USUARI
   Funci:                   Esborra un usuari a la taula USUARI i si no existeix en la taula DESENVOLUPADOR
                             l'esborra de la taula PERSONA
   Parmetre d'entrada:      - p_NIF: el NIF de l'usuari
                             

   Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(50);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
comptador2 INTEGER;
E_NUL  EXCEPTION;
E_EXISTS EXCEPTION;


BEGIN
 
  param_entrada:= 'NIF: '||p_NIF;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  
  END IF;
  
  -- esborrem l'usuari de la taula USUARI; sin existeix generem una excepci
   
   SELECT COUNT(*) INTO comptador FROM USUARI U WHERE U.NIF=p_NIF;
 
   IF (comptador>0) THEN

      DELETE FROM USUARI WHERE NIF=p_NIF;
  
   ELSE
     
      RAISE E_EXISTS;
  
   END IF;


-- si l'usuari no s desenvolupador alhora, l'esborrem de la taula PERSONA

   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR D WHERE D.NIF=p_NIF;
   SELECT COUNT(*) INTO comptador2 FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0 AND comptador2=1) THEN
   
      DELETE FROM PERSONA WHERE NIF=p_NIF;
  
   END IF;


-- esborrament correcte

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_EXISTS THEN
      RSP:='ERROR: aquest usuari no existeix en la taula USUARI';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

    
   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARI', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


      
END SP_BAIXA_USUARI;
    
/

CREATE OR REPLACE PROCEDURE SP_MODIFICACIO_USUARI (
    p_NIF      IN  PERSONA.NIF%TYPE,
    p_telefon  IN  PERSONA.telefon%TYPE,
    p_email    IN  PERSONA.email%TYPE,
    p_pais     IN  USUARI.pais%TYPE,
    p_operador IN  USUARI.operador%TYPE,
    RSP        OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_MODIFICACIO_USUARI
   Funci:                   Modifica les dades d'un usuari en la taula USUARI i PERSONA
                             - p_telefon: telefon de l'usuari 
                             - p_email: email de l'usuari
                             - p_pais: pais de registre de l'usuari 
                             - p_operador: operador telefnic de l'usuari

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/

param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
comptador2 INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;
E_UQ EXCEPTION;


BEGIN

 
 
  param_entrada:=       'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', email: '||p_email||
                         ', Pais: '||p_pais||
                         ', Operador: '||p_operador;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  ELSIF (p_operador IS NULL) THEN
      valor_nul:='operador';
      RAISE E_NUL;
  END IF;

--verifiquem que l'usuari existeixi en la taula PERSONA i en la taula USUARI sin generem una excepci

   SELECT COUNT(*) INTO comptador FROM USUARI  WHERE NIF=p_NIF;
   SELECT COUNT(*) INTO comptador2 FROM PERSONA WHERE NIF=p_NIF;
 
   IF (comptador=0 AND comptador2=0) THEN

      RAISE E_PK;
  
   END IF;
  

-- verifiquem que el nmero de telefon que es vol modificar no existeixi sin generem una excepci

   
   SELECT COUNT(*) INTO comptador FROM PERSONA  WHERE telefon=p_telefon;
   
   IF (comptador>0) THEN
   
       RAISE E_UQ;
   
   END IF;

-- modificaci dels valors
   
   UPDATE PERSONA SET telefon=p_telefon, email=p_email WHERE NIF=p_NIF;
   UPDATE USUARI SET pais=p_pais, operador=p_operador WHERE NIF=p_NIF;

-- modificaci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARI', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_PK THEN
      RSP:='ERROR: No existeix aquest usuari en la taula USUARI';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_UQ THEN
      RSP:='ERROR: el telfon que es vol modificar ja existeix en la base da dades';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARI', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

    
   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARI', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);


     
END SP_MODIFICACIO_USUARI;
    
/

CREATE OR REPLACE PROCEDURE SP_ALTA_DESENVOLUPADOR (
    p_NIF           IN  PERSONA.NIF%TYPE,
    p_telefon       IN  PERSONA.telefon%TYPE,
    p_email         IN  PERSONA.email%TYPE,
    p_pais          IN  USUARI.pais%TYPE,
    p_dataNaixement IN  DESENVOLUPADOR.dataNaixement%TYPE,
    p_llocNaixement IN  DESENVOLUPADOR.llocNaixement%TYPE,
    RSP             OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_ALTA_DESENVOLUPADOR
   Funci:                   Afegeix un desenvolupador a la taula DESENVOLUPADOR i si no existeix en la taula PERSONA,
                             l'afegeix tamb
   Parmetre d'entrada:      - p_NIF: el NIF del desenvolupador
                             - p_telefon: telefon del desenvolupador
                             - p_email: email del desenvolupador
                             - p_pais: pais de registre del desenvolupador
                             - p_dataNaixement: data de naixement del desenvolupador
                             - P_llocNaixement: poblaci de naixement del desenvolupador

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entradaPersona  VARCHAR2(500);
param_entradaDesenvolupador   VARCHAR2(500);
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK_DESENVOLUPADOR EXCEPTION;

BEGIN

  param_entradaPersona:= 'NIF: '||p_NIF||
                         ', telefon: '||p_telefon||
                         ', email: '||p_email;

  param_entradaDesenvolupador:= 'NIF: '||p_NIF||
                        ', pais: '||p_pais||
                        ', dataNaixement: '||p_dataNaixement||
                        ', llocNaixement: '||p_llocNaixement;
 
  param_entrada:=        'NIF: '||p_NIF||
                         ', telefon: '||p_telefon||
                         ', email: '||p_email||
                         ', pais: '||p_pais||
                         ', dataNaixement: '||p_dataNaixement||
                         ', llocNaixement: '||p_llocNaixement;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  ELSIF (p_dataNaixement IS NULL) THEN
      valor_nul:='dataNaixement';
      RAISE E_NUL;
  END IF;

-- inserim el usuari en la taula Persona si prviament no existeix.

   SELECT COUNT(*) INTO comptador FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0) THEN
      
       INSERT INTO PERSONA (NIF,telefon,email) VALUES (p_NIF,p_telefon,p_email);
   
   END IF;


-- inserim el nou usuari sin existeix en la taula DESENVOLUPADOR; si existeix generem una excepci

   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR D WHERE D.NIF=p_NIF;
 
   IF (comptador=0) THEN

      INSERT INTO DESENVOLUPADOR(NIF, pais, dataNaixement, llocNaixement) VALUES (p_NIF,p_pais, p_dataNaixement,p_llocNaixement);
     
   ELSE
     
      RAISE E_PK_DESENVOLUPADOR;
  
   END IF;

-- inserci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entradaDesenvolupador, 'RSP: '||RSP);
   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
  
    WHEN E_PK_DESENVOLUPADOR THEN
      RSP:='ERROR: Ja existeix aquest desenvolupador en la taula DESENVOLUPADOR';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entradaDesenvolupador, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
      ROLLBACK;

      --violaci de la clau nica
      IF (SQLCODE=-00001) THEN
          RSP:='ERROR: ja existeix un usuari o desenvolupador amb el mateix telfon';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entradaPersona, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);

      -- qualsevol error no descrit anteriorment
      ELSE
         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

      END IF;
END SP_ALTA_DESENVOLUPADOR;
    
    
/

CREATE OR REPLACE PROCEDURE SP_BAIXA_DESENVOLUPADOR (
    p_NIF      IN  PERSONA.NIF%TYPE,
    RSP        OUT VARCHAR2
)
IS


/************************************************************************************************************************
   Nom:                      SP_BAIXA_DESENVOLUPADOR
   Funci:                   Esborra un desenvolupador a la taula DESENVOLUPADOR i si no existeix en la taula USUARI
                             l'esborra, tamb, de la taula PERSONA
   Parmetre d'entrada:      - p_NIF: el NIF del desenvolupador

   Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(50);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
comptador2 INTEGER;
E_NUL  EXCEPTION;
E_EXISTS EXCEPTION;


BEGIN
 
  param_entrada:= 'NIF: '||p_NIF;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  
  END IF;
  
  -- esborrem, si existeix, el desenvolupador de la taula DESENVOLUPADOR 
   
   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR  WHERE NIF=p_NIF;
 
   IF (comptador>0) THEN

      DELETE FROM DESENVOLUPADOR WHERE NIF=p_NIF;
  
   ELSE
     
      RAISE E_EXISTS;
  
   END IF;

-- si el desenvolupador no s usuari alhora, l'esborrem de la taula PERSONA

   SELECT COUNT(*) INTO comptador FROM USUARI U WHERE U.NIF=p_NIF;
   SELECT COUNT(*) INTO comptador2 FROM PERSONA P WHERE P.NIF=p_NIF;
 
   IF (comptador=0 AND comptador2=1) THEN
   
      DELETE FROM PERSONA WHERE NIF=p_NIF;
  
   END IF;



-- esborrament correcte

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_EXISTS THEN
      RSP:='ERROR: aquest desenvolupador no existeix en la taula DESENVOLUPADOR';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
    
   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

      
END SP_BAIXA_DESENVOLUPADOR;
    
/

CREATE OR REPLACE PROCEDURE SP_MODIFICACIO_DESENVOLUPADOR (
    p_NIF           IN  PERSONA.NIF%TYPE,
    p_telefon       IN  PERSONA.telefon%TYPE,
    p_email         IN  PERSONA.email%TYPE,
    p_llocNaixement IN  DESENVOLUPADOR.llocNaixement%TYPE,
    p_dataNaixement IN  DESENVOLUPADOR.dataNaixement%TYPE,
    p_pais          IN  DESENVOLUPADOR.pais%TYPE,
    RSP             OUT VARCHAR2
)
IS


/************************************************************************************************************************
   Nom:                      SP_MODIFICACIO_DESENVOLUPADOR
   Funci:                   Modifica les dades d'un desenvolupador de la taula DESENVOLUPADOR i de la taula PERSONA
   Parmetre d'entrada:      - p_NIF: el NIF del desenvolupador
                             - p_telefon: telefon del desenvolupador
                             - p_email: email del desenvolupador
                             - p_pais: pais de registre del desenvolupador
                             - p_dataNaixement: data de naixement del desenvolupador
                             - P_llocNaixement: poblaci de naixement del desenvolupador

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
comptador2 INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;
E_UQ EXCEPTION;


BEGIN

 
  param_entrada:=        'NIF: '||p_NIF||
                         ', Telefon: '||p_telefon||
                         ', email: '||p_email||
                         ', LlocNaixement: '||p_llocNaixement||
                         ', DataNaixement: '||p_dataNaixement||
                         ', Pais: '||p_pais;



-- verificaci de parmetres no nuls

  IF (p_NIF IS NULL) THEN
      valor_nul:='NIF';
      RAISE E_NUL;
  ELSIF (p_telefon IS NULL) THEN
      valor_nul:='telefon';
      RAISE E_NUL;
  ELSIF (p_email IS NULL) THEN
      valor_nul:='email';
      RAISE E_NUL;
  ELSIF (p_pais IS NULL) THEN
      valor_nul:='pais';
      RAISE E_NUL;
  END IF;

--verifiquem que el desenvolupador existeixi en la taula PERSONA i en la taula DESENVOLUPADOR

   SELECT COUNT(*) INTO comptador FROM DESENVOLUPADOR  WHERE NIF=p_NIF;
   SELECT COUNT(*) INTO comptador2 FROM PERSONA WHERE NIF=p_NIF;
 
   IF (comptador=0 AND comptador2=0) THEN

      RAISE E_PK;
  
   END IF;
  

-- verifiquem que el nmero de telefon que es vol modificar no existeixi.

   
   SELECT COUNT(*) INTO comptador FROM PERSONA WHERE telefon=p_telefon;
   
   IF (comptador>0) THEN
   
      RAISE E_UQ;
   
   END IF;

-- modificaci dels valors
   
   UPDATE PERSONA SET telefon=p_telefon, email=p_email WHERE NIF=p_NIF;
   UPDATE DESENVOLUPADOR SET pais=p_pais, dataNaixement=p_dataNaixement,llocNaixement=p_llocNaixement WHERE NIF=p_NIF;

-- modificaci correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:='ERROR: No existeix aquest desenvolupador en la taula DESENVOLUPADOR';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_UQ THEN
      RSP:='ERROR: el telfon que es vol modificar ja existeix en la base da dades';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
    
   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

     
END SP_MODIFICACIO_DESENVOLUPADOR;
    
/

CREATE OR REPLACE PROCEDURE SP_ALTA_APLICACIO (
    p_identificador IN  APLICACIO.identificador%TYPE,
    p_versio        IN  APLICACIO.versio%TYPE,
    p_dataPujada    IN  APLICACIO.dataPujada%TYPE,
    p_URL           IN  APLICACIO.URL%TYPE,
    p_activa        IN  APLICACIO.activa%TYPE,
    p_resolucio     IN APLICACIO.resolucio%TYPE,
    RSP             OUT VARCHAR2
)
IS


/************************************************************************************************************************
   Nom:                      SP_ALTA_APLICACIO
   Funci:                   Afegeix una aplicaci a la taula APLICACIO
   Parmetre d'entrada:      - p_identificador: identificador de l'aplicaci
                             - p_versio: versi de l'aplicaci
                             - p_dataPujada: data de pujada de l'aplicaci
                             - p_URL: enlla al video de demostraci de l'aplicaci
                             - p_resoluci: resoluci mnima de pantalla que ha de tenir el dispositiu on es descarregar

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador  INTEGER;
E_NUL      EXCEPTION;
E_DATE     EXCEPTION;


BEGIN

 
  param_entrada:=        'Identificador: '||p_identificador||
                         ', Versio: '||p_versio||
                         ', DataPujada: '||p_dataPujada||
                         ', URL: '||p_URL||
                         ', Activa: '||p_activa||
                         ', Resolucio: '||p_resolucio;



-- verificaci de parmetres no nuls

  IF (p_identificador IS NULL) THEN
      valor_nul:='identificador';
      RAISE E_NUL;
  ELSIF (p_versio IS NULL) THEN
      valor_nul:='versio';
      RAISE E_NUL;
  ELSIF (p_dataPujada IS NULL) THEN
      valor_nul:='dataPujada';
      RAISE E_NUL;
  ELSIF (p_activa IS NULL) THEN
      valor_nul:='activa';
      RAISE E_NUL;
  ELSIF (p_resolucio IS NULL) THEN
      valor_nul:='resolucio';
      RAISE E_NUL;
  END IF;

-- verifiquem que la data de descarrega no sigui major que l'actual
    
    IF (to_date(p_dataPujada)>to_date(sysdate,'DD/MM/YY')) THEN
       
       RAISE E_DATE;
  
    END IF;



-- inserim l'aplicaci
   
   
      INSERT INTO APLICACIO(identificador,versio,dataPujada,URL,activa,resolucio) VALUES (p_identificador,p_versio,p_dataPujada,p_URL,p_activa,p_resolucio);
  

-- inserci correcta

     RSP:='OK';
     SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN E_DATE THEN
         RSP:= 'ERROR: La data de pujada no pot ser major a l-actual';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
  
   WHEN OTHERS THEN
      ROLLBACK;

      --es comproba que les entrades amb clusules check siguin correctes
      IF (SQLCODE=-02290) THEN
          RSP:='ERROR: el valor activa ha de ser SI  NO';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);
    
      ELSIF(SQLCODE=-00001) THEN
          RSP:='ERROR: la aplicaci ja existeix en la taula APLICACIO';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);
          
      ELSE
         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
         
      END IF;

END SP_ALTA_APLICACIO;
    
/

CREATE OR REPLACE PROCEDURE SP_BAIXA_APLICACIO (
    p_ident       IN  APLICACIO.identificador%TYPE,
    RSP           OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_BAIXA_APLICACIO
   Funci:                   Esborra una aplicaci de la taula APLICACIO
   Parmetre d'entrada:      - p_identificador: identificador de l'aplicaci

   Parmetre de sortida:     RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;

BEGIN

 
  param_entrada:=        'Identificador: '||p_ident;
                         

-- verificaci de parmetres no nuls

  IF (p_ident IS NULL) THEN
      valor_nul:='ident';
      RAISE E_NUL;
  
  END IF;

-- comprovaci de que l'aplicaci existeixi
   
   
      SELECT COUNT(*) INTO comptador FROM APLICACIO WHERE identificador=p_ident;
      
      IF(comptador>0) THEN

        DELETE FROM APLICACIO WHERE identificador=p_ident;
  
      ELSE
        
         RAISE E_PK;
   
     END IF; 
  -- operacio correcta

     RSP:='OK';
     SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);


   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:= 'ERROR: L-aplicaci no existeix en la taula APLICACIO';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
      
END SP_BAIXA_APLICACIO;
    
/

CREATE OR REPLACE PROCEDURE SP_MODIFICACIO_APLICACIO (
    p_ident       IN  APLICACIO.identificador%TYPE,
    p_versio      IN  APLICACIO.versio%TYPE,
    p_dataPujada  IN  APLICACIO.dataPujada%TYPE,
    p_URL         IN  APLICACIO.URL%TYPE,
    p_activa      IN  APLICACIO.activa%TYPE,
    p_resolucio   IN APLICACIO.resolucio%TYPE,
    RSP           OUT VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_MOFICACIO_APLICACIO
   Funci:                   Modifica les dades d'una aplicaci de la taula APLICACI
   Parmetre d'entrada:      - p_identificador: identificador de l'aplicaci
                             - p_versio: versi de l'aplicaci
                             - p_dataPujada: data de pujada de l'aplicaci
                             - p_URL: enlla al video de demostraci de l'aplicaci
                             - p_resoluci: resoluci mnima de pantalla que ha de tenir el dispositiu on es descarregar

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada VARCHAR2(500);
error_msg  VARCHAR2(200);
valor_nul  VARCHAR2(50);
comptador INTEGER;
E_NUL  EXCEPTION;
E_PK EXCEPTION;

BEGIN

 
  param_entrada:=        'Identificador: '||p_ident||
                         ' Versio: '||p_versio||
                         ', DataPujada: '||p_dataPujada||
                         ', URL: '||p_URL||
                         ', Activa: '||p_activa||
                         ', Resolucio: '||p_resolucio;



-- verificaci de parmetres no nuls

  IF (p_ident IS NULL) THEN
      valor_nul:='identificador';
      RAISE E_NUL;
  ELSIF(p_versio IS NULL) THEN
      valor_nul:='versio';
  ELSIF (p_dataPujada IS NULL) THEN
      valor_nul:='dataPujada';
      RAISE E_NUL;
  ELSIF (p_activa IS NULL) THEN
      valor_nul:='activa';
      RAISE E_NUL;
  ELSIF (p_resolucio IS NULL) THEN
      valor_nul:='resolucio';
      RAISE E_NUL;
  END IF;

-- comprovaci de que l'aplicaci existeixi
   
   
      SELECT COUNT(*) INTO comptador FROM APLICACIO WHERE identificador=p_ident;
      
      IF(comptador>0) THEN

         UPDATE APLICACIO SET versio=p_versio,dataPujada=p_dataPujada,URL=p_URL,activa=UPPER(p_activa),resolucio=p_resolucio
         WHERE identificador=p_ident;
  
      ELSE
        
         RAISE E_PK;
   
      END IF;

  -- inserci correcta

     RSP:='OK';
     SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada, 'RSP: '||RSP);


   COMMIT;

EXCEPTION

   WHEN E_NUL THEN
      RSP:= 'ERROR: el valor '||valor_nul||' no pot ser nul';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada, 'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      
   WHEN E_PK THEN
      RSP:= 'ERROR: l-aplicaci no existeix en la taula APLICACIO';
      SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada,'RSP: '||RSP);
      DBMS_OUTPUT.PUT_LINE(RSP);
      

   WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIO', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
      
END SP_MODIFICACIO_APLICACIO;
    
/

CREATE OR REPLACE PROCEDURE SP_DESCARREGA (
      p_usuari     IN   DESCARREGA.usuari%TYPE,
      p_aplicacio  IN   DESCARREGA.aplicacio%TYPE,
      p_data       IN   DESCARREGA.dataDesc%TYPE,
      p_pagament   IN   DESCARREGA.pagament%TYPE,
      p_operador   IN   DESCARREGA.operador%TYPE,
      p_pais       IN   DESCARREGA.pais%TYPE,
      p_dispositiu IN   DESCARREGA.dispositiu%TYPE,
      RSP          OUT  VARCHAR2
)
IS

/************************************************************************************************************************
   Nom:                      SP_DESCARREGA
   Funci:                   Emmagatzema una descrrega d'una aplicaci en el sistema
   Parmetre d'entrada:      - p_usuari: usuari que realitza la descrrega
                             - p_aplicacio: identificador de l'aplicaci
                             - p_data: data de descrrega de l'aplicaci
                             - p_pagament: mode de pagament 
                             - p_operador: operador de telefonia en el moment de la descrrega
                             _ p_dispositiu: identificador del dispositiu en qu s'ha fet la descrrega (IMEI).

  Parmetre de sortida:      RSP--> Retorna OK si s'ha executat el procediment correctament o ERROR amb una descripci
                                    de l'error que s'ha donat

*************************************************************************************************************************/
param_entrada       VARCHAR2(500);
error_msg           VARCHAR2(200);
valor_nul           VARCHAR2(100);
comptador           INTEGER;
resolucioAplicacio  NUMBER;
resolucioDispositiu NUMBER;
valor               VARCHAR2(100);
E_NUL               EXCEPTION;
E_FK                EXCEPTION;
E_RESOL             EXCEPTION;
E_DATE              EXCEPTION;




BEGIN
  param_entrada:= 'Usuari: '||p_usuari||
                  ', Aplicacio: '||p_aplicacio||
                  ', Data: '||p_data||
                  ', Pagament: '||p_pagament||
                  ', Operador: '||p_operador||
                  ', Pais: '||p_pais||
                  ', Dispositiu: '||p_dispositiu;

-- verifiquem que no hi hagi valors nuls

   IF (p_usuari IS NULL) THEN
       valor_nul:='usuari';
       RAISE E_NUL;
   ELSIF (p_aplicacio IS NULL) THEN
       valor_nul:='aplicacio';
       RAISE E_NUL;
   ELSIF (p_data IS NULL) THEN
       valor_nul:='data';
       RAISE E_NUL;
   ELSIF (p_pagament IS NULL) THEN
       valor_nul:='pagament';
       RAISE E_NUL;
   ELSIF (p_operador IS NULL) THEN
       valor_nul:='operador';
       RAISE E_NUL;
   ELSIF (p_pais IS NULL) THEN
       valor_nul:='pais';
       RAISE E_NUL;
   ELSIF (p_dispositiu IS NULL) THEN
       valor_nul:='dispositiu';
       RAISE E_NUL;
       
   END IF;

-- verifiquem que l'usuari existeix prviament en la taula USUARI

    SELECT COUNT(*) INTO comptador FROM USUARI WHERE NIF=p_usuari;
    
    IF(comptador=0) THEN
      valor:='usuari';
      RAISE E_FK;
    END IF;

-- verifiquem que l'aplicaci existeix prviament en la taula APLICACIO

    SELECT COUNT(*) INTO comptador FROM APLICACIO WHERE identificador=p_aplicacio;
    
    IF(comptador=0) THEN
      valor:='aplicacio';
      RAISE E_FK;
    END IF;

-- verifiquem que el dispositiu existeix prviament en la taula DISPOSITIU

    SELECT COUNT(*) INTO comptador FROM DISPOSITIU WHERE IMEI=p_dispositiu;
    
    IF(comptador=0) THEN
      valor:='dispositiu';
      RAISE E_FK;
    END IF;

-- verifiquem que la resoluci de pantalla sigui major que la donada per l'aplicaci

    SELECT resolucio INTO resolucioAplicacio FROM APLICACIO WHERE identificador=p_aplicacio;
    SELECT resolucio INTO resolucioDispositiu FROM DISPOSITIU WHERE IMEI=p_dispositiu;

    IF (resolucioAplicacio > resolucioDispositiu ) THEN
       RAISE E_RESOL;
    END IF;

-- verifiquem que la data de descarrega no sigui major que l'actual
    
    IF (to_date(p_data)>to_date(sysdate,'DD/MM/YY')) THEN
       RAISE E_DATE;
    END IF;


-- insertem la nova descrrega en la taula DESCARREGA

    INSERT INTO DESCARREGA(usuari,aplicacio,dataDesc,pagament,operador,pais,dispositiu) VALUES (p_usuari,p_aplicacio,p_data,p_pagament,p_operador,p_pais,p_dispositiu);

-- crida als procediments estadstics de manera que estiguin actualitzats en cada descrrega de la plataforma


       SP_ALTA_EST1;
       SP_ALTA_EST2;
       SP_ALTA_EST3(EXTRACT(YEAR FROM P_DATA));
       SP_ALTA_EST4(EXTRACT(YEAR FROM P_DATA));
       SP_ALTA_EST5(EXTRACT(YEAR FROM P_DATA));
       SP_ALTA_EST6(EXTRACT(YEAR FROM P_DATA),p_pais);
       SP_ALTA_EST7(EXTRACT(YEAR FROM P_DATA),p_pais);
       SP_ALTA_EST8(EXTRACT(YEAR FROM P_DATA),p_pais);


--  inserci correcta
    RSP:='OK';
    SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);



COMMIT;


EXCEPTION

   WHEN E_NUL THEN
        RSP:= 'ERROR: el valor ' || valor_nul|| ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
    WHEN E_FK THEN  
        RSP:= 'ERROR: el ' || valor || ' no existeix en la taula '||UPPER(valor);       
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
    

    WHEN E_RESOL THEN
         RSP:= 'ERROR: La resoluci del dispositiu s menor que la necessria per l-aplicaci';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

    WHEN E_DATE THEN
         RSP:= 'ERROR: La data de descrrega no pot ser major a l-actual';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);

        
   WHEN OTHERS THEN
        ROLLBACK;
       
        
          error_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || error_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_DESCARREGA', param_entrada, 'RSP: '||RSP);
          DBMS_OUTPUT.PUT_LINE(RSP);
        
              
END SP_DESCARREGA;
    
/

/********************************* ESTRUCTURA DE DADES CONSULTA *****************************************************************/


CREATE OR REPLACE TYPE LLISTA_DESENVOLUPADORS AS OBJECT (
       NIF             CHAR(9),
       llocNaixement   VARCHAR2(50),
       dataNaixement   DATE,
       pais            VARCHAR2(10),
       nomEmpresa      VARCHAR2(50),
       adrea          VARCHAR2(50),
       nombre          NUMBER
);
/

CREATE OR REPLACE TYPE TABLE1 AS TABLE OF LLISTA_DESENVOLUPADORS;
/


CREATE OR REPLACE FUNCTION DESENV_NOMBRE_APLICACIONS (
       p_pais      IN    DESENVOLUPADOR.pais%TYPE
)
RETURN TABLE1

/**********************************************************************************************************************************
   Nom:                 DESENV_NOMBRE_APLICACIONS
   Propsit:            Consulta que retorna tots els desenvolupadors i les seves dades principals per al pas donat 
                   
   Parmetres entrada:  -	p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents per cada fila, per al pas donat:
                                >> NIF
                                >> lloc de naixement
                                >> data de naixement
                                >> pas de registre
                                >> nom de l'organitzaci on treballa.
                                >> adrea de les oficines centrals de l'organitzaci
                                >> nombre d'aplicacions diferents publicades

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

--variables

NIF             CHAR(9);
llocNaixement   VARCHAR2(50);
dataNaixement   DATE;
pais            VARCHAR2(10);
nomEmpresa      VARCHAR2(50);
adrea          VARCHAR2(50);
nombre          NUMBER;
param_entrada   VARCHAR2(10);
i               INTEGER:=1;

RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
resultat        TABLE1:=TABLE1();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Pais: '||p_pais;

-- es verifica que no hi hagi parmetres d'entrada nuls
  IF(p_pais IS NULL) THEN
     valor_nul:='pais';
     RAISE E_NUL;

  END IF;


-- seleccionem en el cursor totes les dades del desenvolupador i l'empresa aix com el total d'aplicacions publicades

   OPEN CUR FOR SELECT d.NIF,d.llocNaixement,d.dataNaixement,d.pais, e.nom, e.adrea, count(cr.aplicacio) AS Total 
   FROM  APLICACIO ap, CREACIO cr, DESENVOLUPADOR d, TREBALLA t, EMPRESA e          
   WHERE  ap.identificador=cr.aplicacio AND cr.desenvolupador=d.NIF AND    d.NIF=t.desenvolupador
   AND t.empresa=e.codi AND d.pais=p_pais 
   GROUP BY d.NIF,d.llocNaixement,d.dataNaixement,d.pais,e.nom,e.adrea;

   LOOP 
     FETCH CUR INTO NIF,llocNaixement,dataNaixement,pais,nomEmpresa,adrea,nombre;
     EXIT WHEN CUR%NOTFOUND;

-- anem emmagatzeman els resultats de cada iteraci en la variable resultat

     resultat.extend;
     resultat(i):=LLISTA_DESENVOLUPADORS( NIF,llocNaixement,dataNaixement,pais,nomEmpresa,adrea,nombre);
     i:=i+1;
    END LOOP;
    CLOSE CUR;

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'DESENV_NOMBRE_APLICACIONS', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'DESENV_NOMBRE_APLICACIONS', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;

   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'DESENV_NOMBRE_APLICACIONS', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;        

END DESENV_NOMBRE_APLICACIONS;

/
/******************************************* ESTRUCTURA DE DADES CONSULTA 2 ******************************************************/


CREATE OR REPLACE TYPE LLISTA_APLICACIONS AS OBJECT (
       identificador       NUMBER,
       versio              VARCHAR2(10),
       dataPujada          DATE,
       URL                 VARCHAR2(30),
       activa              CHAR(2),
       resolucio           NUMBER,
       totalDescarregues   NUMBER
);
/

CREATE OR REPLACE TYPE TABLE2 AS TABLE OF LLISTA_APLICACIONS;
/


CREATE OR REPLACE FUNCTION APLICACIONS_ACTIVES 

RETURN TABLE2

/**********************************************************************************************************************************
   Nom:                 APLICACIONS_ACTIVES
   Propsit:            Consulta que retorna totes les aplicacions que estan actives amb les seves dades principals ordenades pel
                        nombre total de descrregues que han tingut a nivell mundial. 
                   
   Parmetres entrada:  -	Sense parmetres
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat amb les segents dades per cada fila:
                                >> Identificador numric de l'aplicaci
                                >> Versi de l'aplicaci
                                >> Data de pujada a la plataforma
                                >> URL d'enlla al vdeo de demostraci
                                >> Si est activa. Haur de retornar 'SI' per a totes les aplicacions
                                >> Resoluci mnima de pantalla que ha de tenir el dispositiu
                                >> Nombre de descrregues total que ha tingut l'aplicaci a nivell mundial

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

identificador   NUMBER;
versio          VARCHAR2(30);
dataPujada      DATE;
URL             VARCHAR2(100);
activa          CHAR(2);
resolucio       NUMBER;
total           NUMBER;
i               INTEGER:=1;

param_entrada   VARCHAR2(20);
RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
resultat        TABLE2:=TABLE2();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Sense parmetres';


 --selecionem un cursor amb les dades de les aplicacions tot fent un left outer join per seleccionar aquelles que no han tingut cap descrrega

   OPEN CUR FOR SELECT ap.identificador,ap.versio,ap.dataPujada,ap.URL,ap.activa,ap.resolucio,COUNT(de.aplicacio) AS total FROM APLICACIO ap
   LEFT OUTER JOIN DESCARREGA de ON ap.identificador=de.aplicacio WHERE  ap.activa='SI'
   GROUP BY ap.identificador,ap.versio,ap.dataPujada,ap.URL,ap.activa,ap.resolucio 
   ORDER BY total DESC;   

   LOOP
   FETCH CUR INTO identificador,versio,dataPujada,URL,activa,resolucio,total;
   EXIT WHEN CUR%NOTFOUND;
   resultat.extend;
   resultat(i):=LLISTA_APLICACIONS(identificador,versio,dataPujada,URL,activa,resolucio,total);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'APLICACIONS_ACTIVES', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'APLICACIONS_ACTIVES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END APLICACIONS_ACTIVES;

/

/*************************************************** ESTRUCTURA DE DADES CONSULTA 3 ***********************************/

CREATE OR REPLACE TYPE LLISTA_PAISOS AS OBJECT (
       pais            CHAR(2),
       aplicacio       NUMBER,
       total           NUMBER
       
);
/
CREATE OR REPLACE TYPE TABLE3 AS TABLE OF LLISTA_PAISOS;
/
CREATE OR REPLACE FUNCTION PAISOS_NOMBRE_DESCARREGUES (
      
       p_aplicacio  IN  NUMBER,
       p_any        IN  NUMBER
)
RETURN TABLE3

/**********************************************************************************************************************************
   Nom:                 PAISOS_NOMBRE_DESCARREGUES
   Propsit:            Consulta que retorna donada una aplicaci i un any concret retorna un llista amb tots els pasos on s'ha 
                        descarregat aquell any, aix com el nombre de descrregues que ha tingut en aquell pas.
                   
   Parmetres entrada:  -  p_aplicacio: identificador numric de l'aplicaci
                        -  p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> Pas en format ISO 3166-1-alfa-2
                                >> Identificador numric de l'aplicaci
                                >> Nombre total de descrregues sofertes en el pas i l'any donats.

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS


pais            CHAR(2);
param_entrada   VARCHAR2(50);
total           NUMBER;
i               INTEGER:=1;


RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
E_DATE          EXCEPTION;
resultat        TABLE3:=TABLE3();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Aplicacio: '||p_aplicacio||
                    ' Any: '||p_any;

-- verifiquem que no hi hagi parmetres d'entrada nuls
  IF(p_aplicacio IS NULL) THEN
     valor_nul:='aplicacio';
     RAISE E_NUL;
  ELSIF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;

  END IF;
 
-- verifiquem que l'any passat com parmetre no sigui major que l'actual 

  IF(p_any > to_number(to_char(sysdate,'YYYY'))) THEN
     RAISE E_DATE;
  
  END IF;

-- seleccionem un cursor amb el pas de descrrega i el nombre total d'usuaris que han descarregat l'aplicaci (per tant el nombre de descrregues)
-- `per al pas i aplicaci passats com a parmetre

   OPEN CUR FOR SELECT pais, COUNT(usuari) AS totalDesc FROM DESCARREGA 
   WHERE  EXTRACT(YEAR FROM dataDesc)=p_any and aplicacio=p_aplicacio
   GROUP BY pais ORDER BY totalDesc DESC;   
   
   LOOP
   FETCH CUR INTO pais,total;
   EXIT WHEN CUR%NOTFOUND;

-- emmagatzemem cada resultat de la iteraci de la consulta en la variable resultat

   resultat.extend;
   resultat(i):=LLISTA_PAISOS(pais,p_aplicacio,total);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
   WHEN E_DATE THEN
        RSP:= 'ERROR: l-any passat com a parmetre no pot ser ms gran que l-actual'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PAISOS_NOMBRE_DESCARREGUES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END PAISOS_NOMBRE_DESCARREGUES;

/

/************************************************** ESTRUCTURES DE DADES CONSULTA 4 ******************************************/
CREATE OR REPLACE TYPE INFO_USUARI AS OBJECT (
       NIF             CHAR(9),
       telefon         NUMBER,
       email           VARCHAR2(50),
       operador        VARCHAR2(30),
       identificador   NUMBER,
       versio          VARCHAR2(20),
       preu            NUMBER,
       descripcio      VARCHAR2(200)
       
);
/
CREATE OR REPLACE TYPE TABLE4 AS TABLE OF INFO_USUARI;
/
CREATE OR REPLACE FUNCTION ACTIVITAT_USUARI (
       p_telefon     IN    NUMBER
)
RETURN TABLE4

/**********************************************************************************************************************************
   Nom:                 ACTIVITAT_USUARI
   Propsit:            Consulta que donat un usuari final identificat pel seu nmero de telfon retorna tota la seva activitat de descrregues
                        en la plataforma
                   
   Parmetres entrada:  -	p_telefon: telfon de l'usuari
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> NIF d l'usuari
                                >> Telefon de l'usuari
                                >> E-mail de l'usuari
                                >> Operador de l'usuari en el moment de la descrrega
                                >> Identificador numric de l'aplicaci descarregada
                                >> Versi de l'aplicaci descarregada
                                >> Preu que va pagar per l'aplicaci
                                >> Descripci de l'aplicaci

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

NIF             CHAR(9);
telefon         NUMBER;
email           VARCHAR2(50);
operador        VARCHAR2(30);
identificador   NUMBER;
versio          VARCHAR2(20);
preu            NUMBER;
descripcio      VARCHAR2(200);

param_entrada   VARCHAR2(20);
i               INTEGER:=1;
tlf             INTEGER;


RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
E_TELF          EXCEPTION;
resultat        TABLE4:=TABLE4();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Telefon: '||p_telefon;

-- verifiquem que no hi hagi parmetres d'entrada nuls
  IF(p_telefon IS NULL) THEN
     valor_nul:='telefon';
     RAISE E_NUL;

  END IF;

--verifiquem que el telefon existeixi en la base de dades
   
   SELECT COUNT(telefon) INTO tlf FROM PERSONA WHERE telefon=p_telefon;
   
   IF (tlf=0) THEN
   RAISE E_TELF;
 
   END IF;
   
   OPEN CUR FOR SELECT p.NIF,p.telefon,p.email,de.operador,ap.identificador,ap.versio,r.preu,r.descripcio
   FROM PERSONA p, USUARI u, DESCARREGA de, APLICACIO ap, REGISTRE r
   WHERE p.NIF=u.NIF AND u.NIF=de.usuari AND de.aplicacio=ap.identificador AND ap.identificador=r.aplicacio AND p.telefon=p_telefon AND de.pais=r.pais
   GROUP BY p.nif,p.telefon,p.email,de.operador,ap.identificador,ap.versio,r.preu,r.descripcio;
   
   LOOP
   FETCH CUR INTO NIF,telefon,email,operador,identificador,versio,preu,descripcio;
   EXIT WHEN CUR%NOTFOUND;
   resultat.extend;
   resultat(i):=INFO_USUARI(NIF,telefon,email,operador,identificador,versio,preu,descripcio);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
   WHEN E_TELF THEN
        RSP:= 'ERROR: el telfon introdut no existeix en la base de dades'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'ACTIVITAT_USUARI', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END ACTIVITAT_USUARI;

/

/*********************************************** ESTRUCTURA DE DADES CONSULTA 5 **************************************************/

CREATE OR REPLACE TYPE LLISTA_USUARIS AS OBJECT (
       NIF             CHAR(9),
       telefon         NUMBER,
       email           VARCHAR2(50),
       total           NUMBER
       
);
/
CREATE OR REPLACE TYPE TABLE5 AS TABLE OF LLISTA_USUARIS;
/
CREATE OR REPLACE FUNCTION MAX_DESPESA (
       p_any     IN    NUMBER
)
RETURN TABLE5

/**********************************************************************************************************************************
   Nom:                 MAX_DESPESA
   Propsit:            Consulta que retorna per l'any passat per parmetre els 20 (com a mxim nombre) d'usuaris que ms diners s'han
                        gastat en la plataforma ordenats de ms a menys. 
                   
   Parmetres entrada:  -	p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> NIF de l'usuari
                                >> Telfon de l'usuari
                                >> E-mail de l'usuari
                                >> Total de diners gastats

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/

IS

NIF             CHAR(9);
telefon         NUMBER;
email           VARCHAR2(50);
pais            CHAR(2);
param_entrada   VARCHAR2(10);
total           NUMBER;
i               INTEGER:=1;


RSP             VARCHAR2(200);
error_msg       VARCHAR2(200);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
E_DATE          EXCEPTION;
resultat        TABLE5:=TABLE5();
type ccursor    IS  REF CURSOR;
CUR             ccursor;

BEGIN

   param_entrada:=  'Any: '||p_any;

-- verifiquem que no hi hagi parmetres d'entrada nuls
  IF(p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;

  END IF;
 
-- verifiquem que l'any passat com parmetre no sigui major que l'actual 
  IF(p_any > to_number(to_char(sysdate,'YYYY'))) THEN
     RAISE E_DATE;
  
  END IF;

-- seleccionem un cursor amb les dades dels usuaris que ms han gastat ordenats per gasto descendent

   OPEN CUR FOR SELECT u.nif,p.telefon,p.email,SUM(r.preu) AS total FROM USUARI u, PERSONA p, DESCARREGA d, APLICACIO aP, REGISTRE r
   WHERE p.nif=u.nif AND u.nif=d.usuari AND d.aplicacio=aP.identificador AND aP.identificador=r.aplicacio AND d.pais=r.pais AND
   p_any=EXTRACT(year from d.dataDesc) GROUP BY u.nif,p.telefon,p.email ORDER BY total DESC;
   
   LOOP
   FETCH CUR INTO NIF,telefon,email,total;

-- obliguem que hi hagi un mxim de 20 files i anem emmagatzemant les dades en la variable resultat

   EXIT WHEN CUR%NOTFOUND OR CUR%ROWCOUNT>20;
   resultat.extend;
   resultat(i):=LLista_usuaris(NIF,telefon,email,total);
   i:=i+1;
   
   END LOOP;
   CLOSE CUR;
  

-- consulta correcta

   RSP:='OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);

  
RETURN resultat;

EXCEPTION
  
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || valor_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
   WHEN E_DATE THEN
        RSP:= 'ERROR: l-any passat com a parmetre no pot ser ms gran que l-actual'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
   
   WHEN OTHERS THEN
        ROLLBACK;
        error_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || error_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'MAX_DESPESA', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN NULL;
        
END MAX_DESPESA;

/

/******************************************** ESTRUCTURA DE DADES CONSULTA 6 **********************************************/

CREATE OR REPLACE TYPE LLISTA_SO AS OBJECT (
       ao                  NUMBER,
       sistema              VARCHAR2(30),
       total                NUMBER
);

/

CREATE OR REPLACE TYPE TABLE6 AS TABLE OF LLISTA_SO;

/

CREATE OR REPLACE FUNCTION SO_MES_UTILITZATS(
         p_any       IN   NUMBER
)

RETURN TABLE6



/**********************************************************************************************************************************
   Nom:                 SO_MES_UTILITZATS
   Propsit:            Consulta que retorna per l'any passat per parmetre un llistat amb els sistemes operatius de telfons intelligents
                        que ms descrregues han fet junt amb aquest nombre (ordenats pel nombre total de descrregues)
   Parmetres entrada:  -	p_pais: pais donat en format ISO 3166-1-alfa-2
                        
   Parmetres sortida:  -	RSP
   
   Retorna:             	Si l'execuci es realitza amb xit retorna un llistat de dades segents, per al pas donat:
                                >> Any de consulta
                                >> Nom del sistema operatiu
                                >> Total de descrregues que ha fet aquest sistema operatiu en la plataforma

                        - 'ERROR: tipus d'error' si ha fracassat            
**************************************************************************************************************************************/


IS

param_entrada   VARCHAR2(50);
error_msg       VARCHAR2(500);
RSP             VARCHAR2(500);
valor_nul       VARCHAR2(30);
E_NUL           EXCEPTION;
resultat        TABLE6:=TABLE6();
i               INTEGER:=1;

-- seleccionem un cursor amb l'any, el sistema operatiu i el nombre total de descrregues que ha fet en la plataforma en aquell any
CURSOR CUR IS SELECT EXTRACT(year from de.dataDesc),d.sistemaoperatiu,COUNT( d.sistemaoperatiu ) AS SO FROM DESCARREGA de,Dispositiu d
              WHERE   EXTRACT(year from de.datadesc)=p_any AND de.dispositiu=d.Imei
              GROUP BY d.sistemaoperatiu,EXTRACT(year from de.dataDesc) ORDER BY SO DESC;

sistema        VARCHAR2(40);
anyo           NUMBER;
total          NUMBER;


BEGIN

   param_entrada:=    'Any: '||p_any;
                    

--controlem que els parmetres d'entrada no siguin nuls

  IF (p_any IS NULL) THEN
     valor_nul:='any';
     RAISE E_NUL;
  END IF;

--obrim cursor i iterem

    OPEN CUR;
       LOOP
         FETCH CUR INTO anyo,sistema,total;
         EXIT WHEN CUR%NOTFOUND;

    -- anem emmagatzemant les dades en la variable resultat

          resultat.extend;
          resultat(i):=LLista_SO(anyo,sistema,total);
          i:=i+1;
       
       END LOOP;
     CLOSE CUR;   

--execuci correcta

         RSP:= 'OK';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SO_MES_UTILITZATS', param_entrada, 'RSP: '||RSP);   

RETURN resultat;

EXCEPTION

 WHEN E_NUL THEN
         RSP:= 'ERROR: el valor '||valor_nul||' no pot estar buit';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SO_MES_UTILITZATS', param_entrada, 'RSP: '||RSP);   
         DBMS_OUTPUT.PUT_LINE(RSP);
         RETURN NULL;

 WHEN OTHERS THEN
      ROLLBACK;

         error_msg:=SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR: '||error_msg;
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SO_MES_UTILITZATS', param_entrada, 'RSP: '||RSP);
         DBMS_OUTPUT.PUT_LINE(RSP);
         RETURN NULL;
END SO_MES_UTILITZATS;
/

--USUARIS
DECLARE
  RSP VARCHAR2(500);

BEGIN

  SP_ALTA_USUARI('00000001A', 666000001, 'jjtta@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001B', 666000002, 'asta@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001C', 666000003, 'fortta@yahoo.es', 'ES', 'movistar',RSP);
  SP_ALTA_USUARI('00000001D', 666000004, 'ditta@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001E', 666000005, 'smita@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001F', 666000006, 'abel@yahoo.es', 'ES', 'vodafone',RSP);
  SP_ALTA_USUARI('00000001G', 666000007, 'jony@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001H', 666000008, 'juana@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001I', 666000009, 'jamala@yahoo.es', 'ES', 'jazztel',RSP);
  SP_ALTA_USUARI('00000001J', 666000011, 'jonas@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001K', 666000021, 'luis@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001L', 666000031, 'roberto@gmail.com','ES', 'euskatel',RSP);
  SP_ALTA_USUARI('00000001M', 666000041, 'fikit@hotmail.com', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001N', 666000051, 'joti@uoc.edu', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('X0000001A', 666000061, 'jopitu@yahoo.es', 'AD', 'movistar',RSP);
  SP_ALTA_USUARI('X0000001B', 666000071, 'longu@yahoo.es', 'CU', 'orange',RSP);
  SP_ALTA_USUARI('X0000001C', 666000081, 'sari@yahoo.es', 'EG', 'orange',RSP);
  SP_ALTA_USUARI('X0000001D', 666000091, 'mari@yahoo.es', 'AD', 'jazztel',RSP);
  SP_ALTA_USUARI('X0000001E', 666000101, 'luisa@yahoo.es', 'GB', 'orange',RSP);
  SP_ALTA_USUARI('X0000001F', 666000201, 'kate@yahoo.es', 'GB', 'movistar',RSP);
  SP_ALTA_USUARI('X0000001G', 666000301, 'rilar@yahoo.es', 'GR', 'orange',RSP);
  SP_ALTA_USUARI('X0000001H', 666000311, 'sari12@yahoo.es', 'US', 'orange',RSP);
  SP_ALTA_USUARI('X0000001I', 666000321, 'sari123@yahoo.es', 'US', 'orange',RSP);
  SP_ALTA_USUARI('X0000001J', 6660003311, 'sRRR1@yahoo.es', 'US', 'orange',RSP);
  SP_ALTA_USUARI('00000001O', 666000401, 'sour@yahoo.es', 'ES', 'vodafone',RSP);
  SP_ALTA_USUARI('00000001P', 666000501, 'jjttttra@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001Q', 666000601, 'j22222@orange.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001R', 666000701, 'julian@yahoo.es', 'ES', 'jazztel',RSP);
  SP_ALTA_USUARI('00000001S', 666000801, 'rene@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000001T', 666000901, 'sandy@hotmail.com', 'ES', 'movistar',RSP);
  SP_ALTA_USUARI('00000001U', 666001001, 'lory@gmail.com', 'ES', 'ono',RSP);
  SP_ALTA_USUARI('00000001V', 666002001, 'abeli@facebook.com', 'ES', 'ono',RSP);
  SP_ALTA_USUARI('00000001W', 666003001, 'jjt223a@yahoo.es', 'ES', 'movistar',RSP);
  SP_ALTA_USUARI('00000001X', 666004001, 'j112tta@yahoo.es', 'ES', 'jaztel',RSP);
  SP_ALTA_USUARI('00000001Y', 666005001, 'jjtta@orange.es', 'ES', 'ono',RSP);
  SP_ALTA_USUARI('00000001Z', 666006001, 'j334455@orange.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('00000011A', 666007001, 'ariadna@yahoo.es', 'ES', 'orange',RSP);
  SP_ALTA_USUARI('10000000A', 699000001, 'wilt@orange.es', 'ES', 'orange',RSP);

END;
/

--DESENVOLUPADORS
DECLARE
   RSP VARCHAR(500);

BEGIN

  SP_ALTA_DESENVOLUPADOR('10000000A',699000001,'wilt@orange.es', 'ES', '11/11/1990','Cadiz',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000B',699000002,'wally@orange.es', 'ES', '11/09/1992','Lleida',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000C',699000003,'wenny@orange.es', 'ES', '11/11/1993','Barcelona',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000D',699000004,'anita@yahoo.es', 'ES', '13/11/1990','Madrid',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000E',699000005,'ramoncin@gmail.com', 'ES', '16/11/1990','Burgos',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000F',699000006,'josito@orange.es', 'ES', '11/01/1990','Soria',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000G',699000007,'oscar1@orange.es', 'ES', '11/11/1990','Castellon',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000H',699000008,'luismi@yahoo.es', 'ES', '11/12/1990','Cadiz',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000I',699000009,'roman@facebook.es', 'ES', '11/11/1967','Cadiz',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000J',699000011,'renato@facebook.es', 'ES', '11/11/1980','Almeria',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000K',699000012,'ruperto@hotmai.com', 'ES', '11/11/1960','Madrid',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000L',699000013,'r1111@gmail.com', 'ES', '01/01/1990','Lleida',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000M',699000014,'z2222@gmail.com', 'ES', '17/06/1990','Lleida',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000N',699000015,'erty1@orange.es', 'ES', '11/01/1970','Lleida',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000O',699000016,'josepe@orange.es', 'ES', '22/10/1987','Huesca',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000P',699000017,'samarra@orange.es', 'ES', '11/11/1983','Vielha',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000Q',699000018,'li9nust@orange.es', 'ES', '11/04/1990','Gerona',RSP);
  SP_ALTA_DESENVOLUPADOR('10000000R',699000019,'rr2dd2@ono.es', 'ES', '11/03/1990','Tarragona',RSP);
  SP_ALTA_DESENVOLUPADOR('X1000000A',699000010,'cam1@yahoo.es', 'AD', '11/01/1990','Ordino',RSP);
  SP_ALTA_DESENVOLUPADOR('X1000000Z',699000021,'w1234@orange.es', 'US', '18/11/1990','Chicago',RSP);
  SP_ALTA_DESENVOLUPADOR('X1000000X',699000022,'w2233orange.es', 'RU', '24/11/1990','Moscu',RSP);
  SP_ALTA_DESENVOLUPADOR('X1000000B',699000023,'wqwwq@orange.es', 'PL', '29/11/1990','Varsovia',RSP);
  SP_ALTA_DESENVOLUPADOR('X1000000C',699000024,'wilt0@facebook.com', 'GR', '11/11/1955','ElCairo',RSP);
  SP_ALTA_DESENVOLUPADOR('X1000000D',699000025,'martinSS@orange.es', 'GB', '11/11/1956','Londres',RSP);
  SP_ALTA_DESENVOLUPADOR('X1000000E',699000026,'scoott@orange.es', 'US', '11/11/1957','Boston',RSP);
  SP_ALTA_DESENVOLUPADOR('X1000000F',699000027,'wery@hotmail.com', 'CU', '11/11/1958','LaHabana',RSP);
  SP_ALTA_DESENVOLUPADOR('X1000000G',699000028,'wusi@ono.es', 'US', '11/11/1969','LosAngeles',RSP);
  SP_ALTA_DESENVOLUPADOR('X1000000H',699000029,'gaip1@ono.es', 'US', '11/11/1940','Chicago',RSP);


END;
/

--APLICACIONS
DECLARE
   RSP VARCHAR2(500);

BEGIN

SP_ALTA_APLICACIO(1,'V1.0a','11/12/10','http://www.video1.com','SI',22,RSP);
SP_ALTA_APLICACIO(2,'V1.0a','11/02/11','http://www.video2.com','SI',22,RSP);
SP_ALTA_APLICACIO(3,'V1.0a','11/03/11','http://www.video3.com','SI',22,RSP);
SP_ALTA_APLICACIO(4,'V1.1a','11/04/11','http://www.video4.com','SI',22,RSP);
SP_ALTA_APLICACIO(5,'V1.1a','11/05/11','http://www.video5.com','SI',22,RSP);
SP_ALTA_APLICACIO(6,'V1.1a','11/06/11','http://www.video6.com','SI',22,RSP);
SP_ALTA_APLICACIO(7,'V1.2a','11/07/11','http://www.video7.com','SI',22,RSP);
SP_ALTA_APLICACIO(8,'V1.2a','11/08/11','http://www.video8.com','SI',22,RSP);
SP_ALTA_APLICACIO(9,'V1.2b','11/12/11','http://www.video9.com','SI',22,RSP);
SP_ALTA_APLICACIO(10,'V1.2c','11/09/11','http://www.video10.com','SI',22,RSP);
SP_ALTA_APLICACIO(11,'V2.0a','11/12/11','http://www.video11.com','SI',22,RSP);
SP_ALTA_APLICACIO(12,'V2.0a','11/10/11','http://www.video12.com','NO',12.4,RSP);
SP_ALTA_APLICACIO(13,'V2.0a','11/11/11','http://www.video13.com','SI',22,RSP);
SP_ALTA_APLICACIO(14,'V2.1a','11/12/11','http://www.video14.com','SI',22,RSP);
SP_ALTA_APLICACIO(15,'V2.2a','11/01/12','http://www.video15.com','SI',12.4,RSP);
SP_ALTA_APLICACIO(16,'V2.2b','11/02/12','http://www.video16.com','NO',16.8,RSP);
SP_ALTA_APLICACIO(17,'V2.2c','11/03/12','http://www.video17.com','SI',12,RSP);
SP_ALTA_APLICACIO(18,'V2.3a','11/04/12','http://www.video18.com','SI',7.4,RSP);
SP_ALTA_APLICACIO(19,'V2.3b','11/05/12','http://www.video19.com','SI',6.8,RSP);
SP_ALTA_APLICACIO(20,'V2.3c','11/06/12','http://www.video20.com','NO',12,RSP);
SP_ALTA_APLICACIO(21,'V3.0a','11/07/12','http://www.video21.com','SI',12,RSP);

END;
/
--inserci de valors en taules


-- insercio en la taula PAIS
 
   INSERT INTO PAIS(codiPais) VALUES ('ES');
   INSERT INTO PAIS(codiPais) VALUES ('AD');
   INSERT INTO PAIS(codiPais) VALUES ('CU');
   INSERT INTO PAIS(codiPais) VALUES ('EG');
   INSERT INTO PAIS(codiPais) VALUES ('GB');
   INSERT INTO PAIS(codiPais) VALUES ('GR');
   INSERT INTO PAIS(codiPais) VALUES ('PL');
   INSERT INTO PAIS(codiPais) VALUES ('RU');
   INSERT INTO PAIS(codiPais) VALUES ('US');

-- insercio en la taula SO (sistema operatiu)

    INSERT INTO SO(sistemaOperatiu) VALUES ('Symbian');
    INSERT INTO SO(sistemaOperatiu) VALUES ('Android');
    INSERT INTO SO(sistemaOperatiu) VALUES ('WindowPhone');
    INSERT INTO SO(sistemaOperatiu) VALUES ('Blackberry');
    INSERT INTO SO(sistemaOperatiu) VALUES ('HP WebOS');
    INSERT INTO SO(sistemaOperatiu) VALUES ('Bada');

-- inserci en la taula EMPRESA

    INSERT INTO EMPRESA(codi,nom,representant,pais,adrea,telefon) VALUES ('B1231233A','AppDevelopers', 'Jonh Smith', 'GB','Barrigton 65-London',00346506660);
    INSERT INTO EMPRESA(codi,nom,representant,pais,adrea,telefon) VALUES ('A1234567J','AppStore','Domenico Theodorus','GR','Thodir 13-Athenas',003456565659);
    INSERT INTO EMPRESA(codi,nom,representant,pais,adrea,telefon) VALUES ('B7654321R', 'Desarrollers','Juan Ortiz','ES','Gracia 17-Barcelona',0034934545454);
    INSERT INTO EMPRESA(codi,nom,representant,pais,adrea,telefon) VALUES ('C1212345D','MovileDep','Joan Calas','ES','Ballester 6-Lleida',0034973452321);
    INSERT INTO EMPRESA(codi,nom,representant,pais,adrea,telefon) VALUES ('C9898989C', 'MOVLAPP', 'Gary Friends','US','Barrow 19-Detroit',0034343433433);

--inserci en la taula Dispositiu

    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111111,23,'Sony Ericson','00000001A','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111112,22,'XPeria','00000001A','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111113,22,'Nokia E63','00000001A','Symbian');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111114,26,'Nokia E63','00000001A','Symbian');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111115,6,'Nokia E63','00000001A','Symbian');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111121,22,'it500','00000001N','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111122,22,'Torch','00000001N','Blackberry');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111131,22,'HTC Hero','X0000001E','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111132,22,'S3','X0000001E','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111133,22,'Motorola Charm','X0000001E','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111141,22,'HP III','00000001R','HP WebOS');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111151,22,'HTC Dream','00000001S','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111161,22,'iTouch','X0000001H','WindowPhone');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111162,22,'Droid PRO','X0000001H','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111171,22,'XPeria','00000001S','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111172,23.4,'Nokia N8','00000001S','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111181,33.1,'Nokia N8','X0000001J','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111182,33.1,'Xperia','X0000001B','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111183,30.1,'Nokia N8','X0000001C','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111184,43,'iTouch','X0000001I','WindowPhone');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111185,39.1,'Nokia N8','X0000001F','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111186,27.1,'Nokia N8','X0000001G','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111187,35.8,'Sony Ericson','00000001O','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111188,33.1,'Nokia N8','00000001E','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111189,38.1,'it500','00000001D','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111190,37.1,'Nokia N8','00000001C','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111191,36.1,'Nokia N8','00000001B','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111192,35,'Torch','00000001T','Blackberry');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111193,34.1,'Nokia N8','00000001U','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111194,33.1,'it500','00000001V','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111195,32.1,'Nokia N8','00000001W','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111196,32.1,'Torch','00000001X','Blackberry');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111197,31.1,'it500','00000001Y','Android');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111198,53.1,'Nokia N8','00000001Z','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111991,33,'Nokia N8','00000001K','Bada');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111981,33,'Torch','00000001L','Blackberry');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111982,33,'Torch','00000001F','Blackberry');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111983,33,'Torch','00000001G','Blackberry');
    INSERT INTO DISPOSITIU(IMEI,resolucio,modelo,usuaris,sistemaOperatiu) VALUES (1111111111984,33,'Torch','00000001H','Blackberry');


    
--insercio en la taula TREBALLA

    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000A','B1231233A');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000B','B1231233A');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000C','B1231233A');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000D','B1231233A');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000E','B1231233A');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000F','B1231233A'); 
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000G','B1231233A');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000H','B1231233A');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000I','B1231233A');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000J','B1231233A');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000K','B1231233A');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000L','A1234567J');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000M','A1234567J');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000N','A1234567J');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000O','A1234567J');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000P','B7654321R');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000Q','B7654321R');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000R','B7654321R');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('10000000R','C1212345D');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('X1000000A','C1212345D');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('X1000000Z','C1212345D');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('X1000000B','C9898989C');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('X1000000C','C9898989C');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('X1000000D','C9898989C');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('X1000000E','C9898989C');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('X1000000F','C9898989C');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('X1000000G','C9898989C');
    INSERT INTO TREBALLA(desenvolupador, empresa) VALUES ('X1000000H','C9898989C');

-- inserci en la taula CREACI
    

    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',1);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',2);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',3);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',4);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',5);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',6);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',7);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',8);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',9);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',10);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',11);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',12);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',13);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',14);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',15);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000A',16);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000B',17);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000B',18);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000B',19);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000B',20);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000B',21);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000B',1);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000B',3);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000B',7);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000B',11);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000H',12);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000H',13);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000H',14);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000H',15);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000R',6);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000R',7);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('10000000R',8);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000A',4);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000A',5);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000A',6);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000A',7);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000A',8);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000A',9);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000A',10);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000A',11);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000B',12);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000B',13);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000B',14);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000B',15);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000B',16);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000B',17);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000B',18);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000C',19);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000C',20);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000C',21);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000C',13);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000D',1);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000D',5);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000D',2);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000D',4);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000D',6);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000D',8);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000D',10);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000D',12);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000D',14);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000D',16);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000E',3);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000E',5);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000E',7);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000E',9);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000E',11);
    INSERT INTO CREACIO(desenvolupador, aplicacio) VALUES ('X1000000E',17);


-- inserci en la taula REGISTRE

    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (1,'ES',25,'Sirve para ver programas de TV');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (1,'AD',30,'Serveix per veure programes de TV');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (1,'CU',15,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (1,'EG',30,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (1,'GB',45,'The function: to watch TV programs');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (1,'GR',30,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (1,'PL',20,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (1,'RU',22,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (1,'US',40,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (2,'ES',150,'Sirve de GPS');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (2,'AD',130,'Serveix de GPS');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (2,'CU',110,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (2,'EG',100,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (2,'GB',160,'This function is GPS');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (2,'GR',120,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (2,'US',180,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (3,'ES',100,'Sirve para modificar fotos');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (3,'AD',110,'Serveix per modificar fotos');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (3,'CU',90,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (3,'EG',80,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (3,'GB',120,'The functions is to modify photos');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (3,'GR',90,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (3,'US',130,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (4,'ES',160,'Traductor de idiomas');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (4,'AD',170,'Traductor d-idiomes');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (4,'CU',150,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (4,'EG',130,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (4,'GB',160,'To translate languages');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (4,'GR',120,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (4,'US',200,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (5,'ES',180,'Crear tu propio juego');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (5,'AD',180,'Crea el teu propi joc');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (5,'CU',170,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (5,'EG',150,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (5,'GB',190,'Make yout own game');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (5,'GR',180,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (5,'US',210,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (6,'ES',210,'Leer libros');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (6,'AD',210,'Llegir llibres');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (6,'CU',210,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (6,'EG',200,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (6,'GB',230,'Read books');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (6,'GR',220,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (6,'US',250,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (7,'ES',230,'Aprender idiomas');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (7,'AD',230,'Aprendre idiomes');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (7,'CU',230,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (7,'EG',210,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (7,'GB',240,'Learn languages');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (7,'GR',210,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (7,'US',230,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (8,'ES',300,'Escuchar musica mundial');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (8,'AD',300,'Escoltar msica mundial');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (8,'CU',300,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (8,'EG',300,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (8,'GB',300,'Listen world music');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (8,'GR',300,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (8,'US',300,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (9,'ES',290,'Colorear mapas');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (9,'AD',300,'Pintar mapas');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (9,'CU',200,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (9,'EG',180,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (9,'GB',330,'Draw maps');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (9,'GR',100,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (9,'US',350,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (20,'ES',100,'Leer todos los diarios online');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (20,'AD',100,'Llegir tots els diaris online');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (20,'CU',80,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (20,'EG',70,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (20,'GB',110,'Read all the online newspaper');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (20,'GR',70,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (20,'US',150,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (21,'ES',400,'Guardar la notas personales');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (21,'AD',400,'Emmagatzemar la notes personals');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (21,'CU',300,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (21,'EG',300,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (21,'GB',450,'Store all your personal notes');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (21,'GR',200,'NULL');
    INSERT INTO REGISTRE(aplicacio,pais,preu,descripcio) VALUES (21,'US',500,'NULL');


-- inserci en la taula VIABLE

    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (1,'Android',35,'http://binariAndroid1');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (2,'Android',30,'http://binariAndroid2');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (3,'Android',36,'http://binariAndroid3');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (4,'Android',45,'http://binariAndroid4');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (5,'Android',65,'http://binariAndroid5');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (6,'Android',35,'http://binariAndroid6');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (7,'Android',85,'http://binariAndroid7');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (8,'Android',75,'http://binariAndroid8');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (9,'Android',25,'http://binariAndroid9');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (20,'Android',35,'http://binariAndroid20');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (21,'Android',55,'http://binariAndroid21');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (1,'WindowPhone',31,'http://binariWindowPhone1');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (2,'WindowPhone',25,'http://binariWindowPhone2');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (13,'WindowPhone',135,'http://binariWindowPhone13');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (16,'WindowPhone',351,'http://binariWindowPhone16');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (18,'WindowPhone',359,'http://binariWindowPhone18');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (19,'WindowPhone',235,'http://binariWindowPhone19');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (1,'Blackberry',15,'http://binariBalckberry1');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (11,'Blackberry',135,'http://binariBalckberry11');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (12,'Blackberry',325,'http://binariBalckberry12');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (13,'Blackberry',352,'http://binariBalckberry13');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (14,'Blackberry',35,'http://binariBalckberry14');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (15,'Blackberry',335,'http://binariBalckberry15');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (1,'HP WebOS',67,'http://binariHP_WebOS1');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (2,'HP WebOS',90,'http://binariHP_WebOS2');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (3,'HP WebOS',90,'http://binariHP_WebOS3');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (4,'HP WebOS',90,'http://binariHP_WebOS4');
    INSERT INTO VIABLE(aplicacio,so,mida,enlla) VALUES (1,'Bada',46,'http://binariBada1');
    




COMMIT;


/

-- DESCARREGUES

DECLARE
   RSP VARCHAR2(500);

BEGIN

SP_DESCARREGA('00000001A',1,'13/11/10','paypal','Orange','ES',1111111111111,RSP);
SP_DESCARREGA('00000001A',21,'17/09/12','paypal','Orange','ES',1111111111111,RSP);
SP_DESCARREGA('00000001A',20,'13/08/12','paypal','Orange','ES',1111111111111,RSP);
SP_DESCARREGA('00000001N',3,'13/04/12','paypal','Orange','ES',1111111111111,RSP);
SP_DESCARREGA('00000001N',2,'13/01/12','paypal','Orange','ES',1111111111111,RSP);
SP_DESCARREGA('00000001A',2,'12/09/12','paypal','Movistar','ES',1111111111111,RSP);
SP_DESCARREGA('00000001A',3,'12/10/12','paypal','Movistar','ES',1111111111112,RSP);
SP_DESCARREGA('00000001A',4,'12/08/12','paypal','Movistar','ES',1111111111112,RSP);
SP_DESCARREGA('00000001A',5,'12/10/12','paypal','Movistar','ES',1111111111112,RSP);
SP_DESCARREGA('00000001A',6,'12/11/11','paypal','Orange','ES',1111111111112,RSP);
SP_DESCARREGA('00000001A',7,'12/11/10','paypal','Mobiland','AD',1111111111112,RSP);
SP_DESCARREGA('00000001A',8,'12/04/12','paypal','Movistar','ES',1111111111113,RSP);
SP_DESCARREGA('00000001A',9,'12/04/12','paypal','Movistar','ES',1111111111113,RSP);
SP_DESCARREGA('00000001N',1,'08/11/11','visa','Movistar','ES',1111111111121,RSP);
SP_DESCARREGA('00000001N',4,'12/11/11','visa','Movistar','ES',1111111111121,RSP);
SP_DESCARREGA('00000001N',16,'12/08/12','visa','Movistar','ES',1111111111121,RSP);
SP_DESCARREGA('00000001N',17,'12/11/11','visa','Orange','ES',1111111111122,RSP);
SP_DESCARREGA('00000001N',20,'12/11/10','visa','Movistar','ES',1111111111122,RSP);
SP_DESCARREGA('00000001N',19,'12/09/12','visa','Orange','ES',1111111111122,RSP);
SP_DESCARREGA('X0000001E',1,'12/12/10','MasterCard','Vodafone','GB',1111111111131,RSP);
SP_DESCARREGA('X0000001E',4,'12/11/11','MasterCard','Vodafone','GB',1111111111131,RSP);
SP_DESCARREGA('X0000001E',5,'12/11/11','MasterCard','Vodafone','GB',1111111111131,RSP);
SP_DESCARREGA('X0000001E',11,'12/03/12','MasterCard','Vodafone','GB',1111111111132,RSP);
SP_DESCARREGA('X0000001E',12,'12/03/12','MasterCard','Vodafone','GB',1111111111132,RSP);
SP_DESCARREGA('X0000001E',13,'07/07/12','MasterCard','Vodafone','GB',1111111111132,RSP);
SP_DESCARREGA('X0000001E',14,'12/08/12','MasterCard','Vodafone','GB',1111111111132,RSP);
SP_DESCARREGA('X0000001E',15,'18/09/12','MasterCard','Vodafone','GB',1111111111133,RSP);
SP_DESCARREGA('X0000001E',16,'02/10/12','MasterCard','Orange','GB',1111111111133,RSP);
SP_DESCARREGA('X0000001E',17,'08/09/12','MasterCard','Orange','GB',1111111111133,RSP);
SP_DESCARREGA('00000001R',1,'12/10/12','Visa','movistar','ES',1111111111141,RSP);
SP_DESCARREGA('00000001R',2,'12/10/12','Visa','Mobiland','AD',1111111111141,RSP);
SP_DESCARREGA('00000001R',3,'19/10/12','Visa','mobiland','AD',1111111111141,RSP);
SP_DESCARREGA('00000001R',4,'19/10/12','Visa','mobiland','AD',1111111111141,RSP);
SP_DESCARREGA('00000001R',20,'19/10/12','Visa','mobiland','AD',1111111111141,RSP);
SP_DESCARREGA('00000001R',13,'19/10/12','Visa','mobiland','AD',1111111111141,RSP);
SP_DESCARREGA('00000001R',21,'19/10/12','Visa','mobiland','AD',1111111111141,RSP);
SP_DESCARREGA('00000001R',7,'21/08/12','Visa','movistar','ES',1111111111141,RSP);
SP_DESCARREGA('00000001S',1,'12/11/12','Neopago','movistar','AD',1111111111151,RSP);
SP_DESCARREGA('00000001S',4,'12/11/11','Neopago','movistar','AD',1111111111151,RSP);
SP_DESCARREGA('00000001S',11,'12/10/12','Neopago','movistar','AD',1111111111151,RSP);
SP_DESCARREGA('X0000001H',1,'19/11/10','paypal','Sprint','US',1111111111161,RSP);
SP_DESCARREGA('X0000001H',4,'12/11/11','paypal','Sprint','US',1111111111161,RSP);
SP_DESCARREGA('X0000001H',15,'12/10/12','paypal','Sprint','US',1111111111162,RSP);
SP_DESCARREGA('X0000001H',16,'24/10/12','paypal','Sprint','US',1111111111162,RSP);
SP_DESCARREGA('X0000001H',17,'12/09/12','paypal','Sprint','US',1111111111162,RSP);
SP_DESCARREGA('X0000001H',20,'10/11/12','paypal','Sprint','US',1111111111162,RSP);
SP_DESCARREGA('00000001S',21,'12/09/12','Visa','Mobiland','AD',1111111111172,RSP);
SP_DESCARREGA('00000001B',21,'13/11/12','Visa','Mobiland','AD',1111111111191,RSP);
SP_DESCARREGA('00000001C',21,'19/09/12','MasterCard','Mobiland','AD',1111111111190,RSP);
SP_DESCARREGA('00000001D',21,'12/11/12','Neopago','Mobiland','AD',1111111111189,RSP);
SP_DESCARREGA('00000001E',3,'12/11/12','Visa','Movistar','ES',1111111111188,RSP);
SP_DESCARREGA('00000001K',7,'12/09/12','Visa','Movistar','ES',1111111111991,RSP);
SP_DESCARREGA('00000001L',7,'12/11/12','MasterCard','Mobiland','AD',1111111111981,RSP);
SP_DESCARREGA('X0000001B',1,'15/11/12','Visa','Sprint','US',1111111111182,RSP);
SP_DESCARREGA('X0000001C',21,'12/11/12','Visa','Orange','GB',1111111111183,RSP);
SP_DESCARREGA('X0000001I',4,'14/08/12','Paypal','Sprint','US',1111111111184,RSP);
SP_DESCARREGA('X0000001F',4,'12/11/12','Visa','Mobiland','AD',1111111111185,RSP);
SP_DESCARREGA('00000001X',5,'12/11/12','MasterCard','Sprint','US',1111111111196,RSP);
SP_DESCARREGA('00000001Y',5,'12/03/12','Visa','Mobiland','AD',1111111111197,RSP);
SP_DESCARREGA('00000001Z',4,'19/03/12','Neopago','Movistar','ES',1111111111198,RSP);
SP_DESCARREGA('X0000001G',3,'14/08/12','Paypal','Sprint','US',1111111111186,RSP);
SP_DESCARREGA('00000001O',4,'14/08/12','Paypal','Jazztel','ES',1111111111187,RSP);
SP_DESCARREGA('00000001F',4,'14/11/12','Paypal','Jazztel','ES',1111111111982,RSP);
SP_DESCARREGA('00000001G',5,'19/11/12','Paypal','Jazztel','ES',1111111111983,RSP);
SP_DESCARREGA('00000001H',6,'21/11/12','Paypal','Jazztel','ES',1111111111984,RSP);



END;
/

SET serveroutput on;
spool PROVES.log
set echo on


-- TRACTAMENT D'ERRORS DELS PROCEDIMENTS ABM I SORTIDA DE LES CONSULTES DEL MDUL ESTADSTIC



/**************************************************** ABM USUARIS  ******************************************************/
DECLARE 
rsp VARCHAR2(500);
BEGIN
-- valors null
SP_ALTA_USUARI('11100001A', 666045001, NULL, 'ES', 'orange',RSP);
--telefon repetit
SP_ALTA_USUARI('12300001A', 666000001, 'lonji@yahoo.es', 'ES', 'orange',RSP);
--usuari repetit
SP_ALTA_USUARI('00000001A', 666088001, 'mairi@hotmail.com', 'ES', 'orange',RSP);
--insercio correcta
SP_ALTA_USUARI('11111111A',666999777,'liunyt@gmail.com','AD','Mobiland',RSP);
DBMS_OUTPUT.PUT_LINE(RSP);
--baixa correcta
SP_BAIXA_USUARI('11111111A',RSP);
DBMS_OUTPUT.PUT_LINE(RSP);
--usuari per donar de baixa no existeix
SP_BAIXA_USUARI('XX123411A',RSP);
--modificacio incorrecta: el telfon ja existeix en la bases de dades
SP_MODIFICACIO_USUARI('00000001B',666000001,'JJTTEE@yahoo.es','ES','Euskatel',RSP);
--modificacio incorrecta: l'usuari no existei
SP_MODIFICACIO_USUARI('00090001A',666000001,'JJTTEE@yahoo.es','ES','Euskatel',RSP);
--modificacio correcta
SP_MODIFICACIO_USUARI('00000001A',666110002,'JJTTEE@yahoo.es','ES','Euskatel',RSP);
DBMS_OUTPUT.PUT_LINE(RSP);

END;
/

/**************************************************** ABM DESENVOLUPADORS **************************************************/
DECLARE
RSP  VARCHAR2(500);
BEGIN
-- valors null
SP_ALTA_DESENVOLUPADOR('10320000A',699097001,'wilt@orange.es', NULL, '11/07/89','Cadiz',RSP);
-- el desenvolupador ja existeix
SP_ALTA_DESENVOLUPADOR('10000000A',699003331,'wilt@orange.es', 'ES', '11/11/1990','Cadiz',RSP);
-- el telefon ja existeix
SP_ALTA_DESENVOLUPADOR('12220000A',699000001,'wilt@orange.es', 'ES', '11/11/1990','Cadiz',RSP);
--alta correcta
SP_ALTA_DESENVOLUPADOR('12345600A',688888001,'wilt@orange.es', 'ES', '11/11/1990','Cadiz',RSP);
DBMS_OUTPUT.PUT_LINE(RSP);
-- baixa correcta
SP_BAIXA_DESENVOLUPADOR('12345600A',RSP);
DBMS_OUTPUT.PUT_LINE(RSP);
--modificacio amb valors null
SP_MODIFICACIO_DESENVOLUPADOR('10000000A',699000001,NULL, 'ES', '11/11/1990','Cadiz',RSP);
-- modificaci amb un telfon que ja existia
SP_MODIFICACIO_DESENVOLUPADOR('10000000A',699000017,'wilt@orange.es', 'ES', '11/11/1990','Cadiz',RSP);
--modificaci de un desenvolupador que no existeix
SP_MODIFICACIO_DESENVOLUPADOR('X0123450A',699000001,'wilt@orange.es', 'ES', '11/11/1990','Cadiz',RSP);
--modificacio correcta
SP_MODIFICACIO_DESENVOLUPADOR('10000000A',699440001,'wilt@orange.es', 'ES', '11/11/1990','Tarragona',RSP);
DBMS_OUTPUT.PUT_LINE(RSP);


END;
/
/*************************************************** ABM APLICACIONS *******************************************/
DECLARE
RSP   VARCHAR2(500);
BEGIN

-- no pot tenir valors null
SP_ALTA_APLICACIO(23,'V1.0a',NULL,'http://www.video1.com','SI',22,RSP);
-- no es poden repetir identificador d'aplicacions (claus primries)
SP_ALTA_APLICACIO(1,'V1.0a','11/12/10','http://www.video1.com','SI',22,RSP);
-- la data d'inserci de la aplicaci no pot ser superior a l'actual
SP_ALTA_APLICACIO(50,'V1.9a','11/12/14','http://www.video50.com','SI',32,RSP);
-- el valor de l'atribut activa ha de se SI   NO
SP_ALTA_APLICACIO(50,'V1.9a','11/12/11','http://www.video50.com','RT',32,RSP);
--alta correcta
SP_ALTA_APLICACIO(22,'V3.0a','11/11/12','http://www.video23.com','SI',22,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);

--baixa correcta
SP_BAIXA_APLICACIO(22,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);

-- baixa d'una aplicaci que no existeix
SP_BAIXA_APLICACIO(33,RSP);
--modificacio amb valors null
SP_MODIFICACIO_APLICACIO(1,'V1.0a',NULL,'http://www.video1.com','SI',22,RSP);
-- modificacio d'una aplicaci que no existeix
SP_MODIFICACIO_APLICACIO(90,'V1.0a','11/12/10','http://www.video1.com','SI',22,RSP);
--modificacio correcta
SP_MODIFICACIO_APLICACIO(1,'V1.1h','11/12/10','http://www.video1.com','SI',18,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);


END;
/
/************************************************* ALTA DESCRREGUES ********************************************/
DECLARE
RSP VARCHAR2(500);
BEGIN
-- valors null
SP_DESCARREGA('00000001N',1,'13/11/10',NULL,'movistar','ES',1111111111111,RSP);
-- aplicacio no existeix
SP_DESCARREGA('00000001A',40,'13/11/10','paypal','movistar','ES',1111111111111,RSP);
--usuari no existeix
SP_DESCARREGA('02320001A',1,'13/11/10','paypal','movistar','ES',1111111111111,RSP);
--dispositiu no existeix
SP_DESCARREGA('00000001A',1,'13/11/10','paypal','movistar','ES',0001111111111,RSP);
-- la data de descarrega no pot ser major a l'actual
SP_DESCARREGA('00000001A',1,'13/11/14','paypal','movistar','ES',1111111111114,RSP);
--la resoluci del dispositiu ha de ser major que la suportada per l'aplicaci
SP_DESCARREGA('00000001A',15,'13/11/10','paypal','movistar','ES',1111111111115,RSP);


END;
/
/************************************************** CONSULTES  *****************************************************/


--CONSULTA 1



-- fem la consulta1 sobre el pas EEUU 'US'
select * from table(DESENV_NOMBRE_APLICACIONS('US'));
/
--fem la consulta1 sobre el pas Gran Bretanya GB
select * from table(DESENV_NOMBRE_APLICACIONS('GB'));
/
-- fem la consulta1 sobre el pas Espanya ES
select * from table(DESENV_NOMBRE_APLICACIONS('ES'));
/


-- CONSULTA 2 



--fem la consulta 
select * from table(APLICACIONS_ACTIVES);
/
--afegim una nova aplicaci
DECLARE
RSP VARCHAR2(500);
BEGIN
SP_ALTA_APLICACIO(22,'V3.0a','12/08/12','http://www.video22.com','SI',9.9,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);

END;
/
--tornem a fer la consulta i observem els canvis
select * from table(APLICACIONS_ACTIVES);

/



-- CONSULTA 3



--fem la consulta per l'aplicaci 1 i l'any 2010
select * from table(PAISOS_NOMBRE_DESCARREGUES(1,2010));

--afegim una descrrega de l'aplicaci 1 en l'any 2010
DECLARE
RSP VARCHAR2(500);
BEGIN 
SP_DESCARREGA('X0000001J',1,'11/11/10','Visa','movistar','US',1111111111181,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);

END;
/
--tornem a fer la consulta i observem els canvis
select * from table(PAISOS_NOMBRE_DESCARREGUES(1,2010));
/



-- CONSULTA 4



--fem una consulta per l'usuari amb aquest nmero de telfon
select * from table(ACTIVITAT_USUARI(666000801));
/
--afegim una altra descrrega per aquest usuari
DECLARE
RSP VARCHAR2(500);
BEGIN 
SP_DESCARREGA('00000001S',6,'11/11/10','MoneyTransfer','Movistar','US',1111111111151,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);

END;
/
--tornem a executar la consulta4 sobre el mateix usuari
select * from table(ACTIVITAT_USUARI(666000801));
/



--CONSULTA 5 



select * from table(MAX_DESPESA(2010));
/
select * from table(MAX_DESPESA(2011));
/
select * from table(MAX_DESPESA(2012));

/
-- CONSULTA 6



select * from table(SO_MES_UTILITZATS(2010));
/
select * from table(SO_MES_UTILITZATS(2011));
/
select * from table(SO_MES_UTILITZATS(2012));



/************************************** ESTADSTIQUES ****************************************************/


-- ESTADSTICA 1 ----------------retorna el nombre total de descrregues en la plataforma


SELECT * FROM EST1;
/
-- afegim una descrrega
DECLARE
RSP VARCHAR2(500);
BEGIN 
SP_DESCARREGA('X0000001J',3,'11/11/10','Paypal','movistar','US',1111111111181,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);

END;
/
--tornem a fer la consulta
SELECT * FROM EST1;
/


-- ESTADSTICA 2 -----------------retorna el nombre total d'euros generats en la plataforma fins el moment de la consulta

SELECT * FROM EST2;
/
--afegim una altra descarrega de preu en AD:180
DECLARE
RSP VARCHAR2(500);
BEGIN 
SP_DESCARREGA('X0000001J',5,'13/11/10','Visa','movistar','AD',1111111111181,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);

END;
/
--tornem a fer la consulta
SELECT * FROM EST2;
/



-- ESTADSTICA 3   -------------- retorna, per un any donat, el nombre mig de aplicacions descarregades per un usari. s a dir el total d'aplicacions descarregades
-- en un any dividit pel total d'usuaris diferents que han fet descarregues aquell any    RET=SUM(aplicacions)/usuaris_diferents


SELECT * FROM EST3 WHERE ao='2012';
/
DECLARE
RSP VARCHAR2(500);
BEGIN 
SP_DESCARREGA('X0000001J',2,'13/11/12','Visa','movistar','AD',1111111111181,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);

END;
/
--tornem a executar la consulta per al mateix any i observem la variaci
SELECT * FROM EST3 WHERE ao='2012';
/
--fem les consultes per altres anys
SELECT * FROM EST3 WHERE ao='2011';
/
SELECT * FROM EST3 WHERE ao='2010';
/




--ESTADSTICA 4 --------------------- retorna, en per un any donat, el desenvolupador o desenvolupadors del qual s'han fet ms descarregues junt amb aquest nombre


--fem la consulta per un any
SELECT * FROM EST4 WHERE ao='2010';
/
DECLARE
RSP VARCHAR2(500);
BEGIN 
SP_DESCARREGA('X0000001J',8,'13/11/10','Visa','movistar','AD',1111111111181,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);

END;
/
--repetim la consulta i observem els canvis
SELECT * FROM EST4 WHERE ao='2010';
/ 
--fem la consulta per d'altres anys
SELECT * FROM EST4 WHERE ao='2011';
/
SELECT * FROM EST4 WHERE ao='2012';
/



-- ESTADSTICA 5 -----------------  retorna, per un any donat, l'aplicaci o aplicacions (en cas d'empat) que ms ha recaudat i els desenvolupadors que l'han creat

--fem una consulta per un any
SELECT * FROM EST5 WHERE ao='2010';
/
--afegim una nova descrrega de l'aplicaci 9 
DECLARE
RSP VARCHAR2(500);
BEGIN 
SP_DESCARREGA('00000001Z',9,'13/12/10','Visa','Movistar','AD',1111111111198,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);
END;
/
--tornem a fer la consulta i observem els canvis i observem que passen a ser dues apps.
SELECT * FROM EST5 WHERE ao='2010';
/
--fem consultes per altres anys
SELECT * FROM EST5 WHERE ao='2011';
/
SELECT * FROM EST5 WHERE ao='2012';
/


-- ESTADSTICA 6 --------- donats un pas i un any retorna els usuaris diferents que han fet almenys una descrrega

-- fem la consulta per l'any 2010 i el pas ES
SELECT * FROM EST6 WHERE ao='2010' AND pais='ES';
/
--fem una nova descrrega per aquest any i pas i un usuari diferent
DECLARE
RSP VARCHAR2(500);
BEGIN 
SP_DESCARREGA('00000001Y',20,'13/12/10','Visa','Movistar','ES',1111111111197,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);
END;
/
--tornem a fer la consulta
SELECT * FROM EST6 WHERE ao='2010' AND pais='ES';
/
--fem consultes per d'altres pasos i anys
SELECT * FROM EST6 WHERE ao='2011' AND pais='US';
/
SELECT * FROM EST6 WHERE ao='2012' AND pais='AD';
/


-- ESTADSITCA 7 -------- per una any i un pas donat retorna els ingressos totals generats en apps en aquell any i pas

--fem la consulta per un any i pas
SELECT * FROM EST7 WHERE ao='2010' AND pais='AD';
/
--afegim una descrrega en aquell pas i per aquell any
DECLARE
RSP VARCHAR2(500);
BEGIN 
SP_DESCARREGA('00000001Z',2,'13/12/10','Visa','Movistar','AD',1111111111198,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);
END;
/
--tornem a fer la consulta i observem els canvis
SELECT * FROM EST7 WHERE ao='2010' AND pais='AD';
/
--fem d'altres consultes per d'altres anys i pasos
SELECT * FROM EST7 WHERE ao='2011' AND pais='ES';
/
SELECT * FROM EST7 WHERE ao='2012' AND pais='US';
/


-- ESTADSTICA 8 ----Donats un any i un pas, retorna el nombre d'aplicacions diferents descarregades com a mnim una vegada
--                    en aquell any i pas

--fem una consulta per l'any 2010 i el pas ES
SELECT * FROM EST8 WHERE pais='ES' AND ao='2010';
/
--afegim una aplicaci no descarregada en aquell any
DECLARE
RSP VARCHAR2(500);
BEGIN 
SP_DESCARREGA('00000001Z',8,'13/07/10','Visa','Movistar','ES',1111111111198,RSP);
DBMS_OUTPUT.PUT_LINE(RSP);
END;
/

--tornem a fer la consulta
SELECT * FROM EST8 WHERE ao='2010' AND pais='ES';
/

--fem consultes per d'altres anys i pasos
SELECT * FROM EST8 WHERE ao='2011' AND pais='US';
/
SELECT * FROM EST8 WHERE ao='2012' AND pais='GB';
/

/******************************** LOGS  ***************************************************/

-- EXECUTEM LA CONSULTA A LA TAULA LOGS
SELECT * FROM LOGS ORDER BY 1 DESC;
/


spool off
exit































































  










